/*
 *  linux/arch/arm/mach-pxa/korebot.c
 *
 *  Support for K-Team Korebot robotic developement board.
 *  
 *  Author:     Pierre Bureau
 *  Created:	March 21, 2003
 *  Copyright:  K-Team
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 */
#include <linux/init.h>
#include <linux/major.h>
#include <linux/fs.h>
#include <linux/interrupt.h>
#include <linux/sched.h>

#include <asm/types.h>
#include <asm/setup.h>
#include <asm/memory.h>
#include <asm/mach-types.h>
#include <asm/hardware.h>
#include <asm/irq.h>
#include <asm/delay.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/irq.h>

#include <asm/arch/pxa-regs.h>
#include <asm/arch/korebot.h>
#include <asm/arch/udc.h>

#include "generic.h"

/*
static unsigned long armonie_irq_en_mask;

static void armonie_mask_and_ack_irq(unsigned int irq)
{
	int armonie_irq = (irq - ARMONIE_IRQ(0));
	armonie_irq_en_mask &= ~(1 << armonie_irq);
	LUB_IRQ_MASK_EN &= ~(1 << armonie_irq);
	LUB_IRQ_SET_CLR &= ~(1 << armonie_irq);
}

static void armonie_mask_irq(unsigned int irq)
{
	int armonie_irq = (irq - ARMONIE_IRQ(0));
	armonie_irq_en_mask &= ~(1 << armonie_irq);
	LUB_IRQ_MASK_EN &= ~(1 << armonie_irq);
}

static void armonie_unmask_irq(unsigned int irq)
{
	int armonie_irq = (irq - ARMONIE_IRQ(0));
	armonie_irq_en_mask |= (1 << armonie_irq);
	LUB_IRQ_MASK_EN |= (1 << armonie_irq);
}

void armonie_irq_demux(int irq, void *dev_id, struct pt_regs *regs)
{
	unsigned long irq_status;
	int i;

	while ((irq_status = LUB_IRQ_SET_CLR & armonie_irq_en_mask)) {
		for (i = 0; i < 6; i++) {
			if(irq_status & (1<<i)) 
				do_IRQ(ARMONIE_IRQ(i), regs);
		}
	}
}
*/

/* USB Device Controller */
static struct pxa2xx_udc_mach_info udc_info __initdata = {
	.gpio_pullup		= KOREBOT_GPIO_USB_PULLUP,
//	.gpio_vbus		= KOREBOT_GPIO_USB_VBUS,
};


static void __init korebot_init_irq(void)
{
//    int irq;

    pxa_init_irq_low();
    pxa_init_irq_gpio(PXA_LAST_GPIO);
}

#define KOREBOT_USB_RST		33
static void __init korebot_init(void)
{
  printk("Init KoreBot Machine\n");
  // Make sure the reset is high
  GPSR1 |= GPIO_bit(KOREBOT_USB_RST);
  // set directions to output
  GPDR(KOREBOT_USB_RST) |= GPIO_bit(KOREBOT_USB_RST);

  //reset the otg
  GPCR1 |= GPIO_bit(KOREBOT_USB_RST);
  udelay( 50 );
  GPSR1 |= GPIO_bit(KOREBOT_USB_RST);
  //setup the udc info
  pxa_set_udc_info(&udc_info);
}


//__initcall(korebot_init);

static void __init
fixup_korebot(struct machine_desc *desc, struct tag *params, char **cmdline, struct meminfo *mi)
{
	SET_BANK (0, 0xa0000000, 64*1024*1024);
	mi->nr_banks = 1;
	printk("Fixup korebot\n");
}

static struct map_desc korebot_io_desc[] __initdata = {};

static void __init korebot_map_io(void)
{
	pxa_map_io();
	iotable_init(korebot_io_desc,ARRAY_SIZE(korebot_io_desc));

	printk("Init KoreBot IO\n");
	/* This is for the Flash chip select */
	pxa_gpio_mode(GPIO15_nCS_1_MD);
	
	/* This select the SSP mode by default */
	pxa_gpio_mode(GPIO23_SCLK_MD);
	pxa_gpio_mode(GPIO24_SFRM_MD);
	pxa_gpio_mode(GPIO25_STXD_MD);
	pxa_gpio_mode(GPIO26_SRXD_MD);
	pxa_gpio_mode(GPIO27_SEXTCLK_MD);
	
	/* This enables the BTUART */
	pxa_gpio_mode(GPIO42_BTRXD_MD);
	pxa_gpio_mode(GPIO43_BTTXD_MD);
	pxa_gpio_mode(GPIO44_BTCTS_MD);
	pxa_gpio_mode(GPIO45_BTRTS_MD);

	/* This enables the STUART */
	pxa_gpio_mode(GPIO46_STRXD_MD);
	pxa_gpio_mode(GPIO47_STTXD_MD);
	
	/* Set GPIO2 to output for USB connect */
	GPDR0 |= 0x04;
	GPCR0 = 0x04;

	/* setup sleep mode values */
	PWER  = 0x00000002;
	PFER  = 0x00000000;
	PRER  = 0x00000002;
	PGSR0 = 0x00008000;
	PGSR1 = 0x003F0202;
	PGSR2 = 0x0001C000;
	PCFR |= PCFR_OPDE;
}

MACHINE_START(KOREBOT, "K-Team Robotic development board")
	/* MAINTAINER("Pierre Bureau / Stphane Magnenat") */
	.phys_io	= 0x40000000,
	//BOOT_MEM(0xa0000000, 0x40000000, io_p2v(0x40000000))
	.io_pg_offst	= (io_p2v(0x40000000) >> 18) & 0xfffc,
	.boot_params	= 0xa0000100,
	.fixup		= fixup_korebot,
  	.init_machine	= korebot_init,
	.map_io		= korebot_map_io,
	.init_irq	= korebot_init_irq,
	.timer		= &pxa_timer,
MACHINE_END
