/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: dumpreg.c
 * Author: Biao Jia		2/01/02
 * Description:
 *    Dump register contents of OTG243 chip.
 */

#include "os.h"
#include "stack_urb.h"
#include "otg.h"

static void *otg243_map = 0;


static char* regNames[] =
{
	"HardwareMode",			/* 0 */
	"MNPControlStatus",		/* 4 */
	"OTGTimer1",			/* 8 */
	"OTGTimer2",			/* C */
	"ClockControl",			/* 10 */
	"DMA0Config",			/* 14 */
	"DMA0ETD",				/* 18 */
	"DMA1Config",			/* 1C */
	"DMA1ETD",				/* 20 */
	"DMA0DataPort",			/* 24 */
	"DMA1DataPort",			/* 28 */
	"PIODataPort",			/* 2C */
	"FuncRamStart",			/* 30 */
	"ChipIntrStatus",		/* 34 */
	"ChipIntrEnable",		/* 38 */
	"ChipIntrClear",		/* 3C */
	"ChipIntrSet",			/* 40 */
	"IOReady",				/* 44 */
	"IOConfig1",			/* 48 */
	"IOConfig2",			/* 4C */
	"ShortFrame",			/* 50 */
	"NoRegister54",			/* 54 */
	"NoRegister58",			/* 58 */
	"NoRegister5C",			/* 5C */
	"NoRegister60",			/* 60 */
	"NoRegister64",			/* 64 */
	"NoRegister68",			/* 68 */
	"NoRegister6C",			/* 6C */
	"NoRegister70",			/* 70 */
	"NoRegister74",			/* 74 */
	"NoRegister78",			/* 78 */
	"NoRegister7C",			/* 7C */
	"HcRevision",			/* 80 */
	"HcControl",			/* 84 */
	"HcCommandStatus",		/* 88 */
	"HcIntrStatus",			/* 8C */
	"HcIntrEnbale",			/* 90 */
	"HcIntrDisable",		/* 94 */
	"HcETDIntrStatus",		/* 98 */
	"HcETDIntrEnable",		/* 9C */
	"HcETDIntrDisable",		/* A0 */
	"HcStartAddr",			/* A4 */
	"HcXYBufferStatus",		/* A8 */
	"HcXYBufferSet",		/* AC */
	"HcXYBufferClear",		/* B0 */
	"HcNoRegister34",		/* B4 */
	"HcETDEnableSet",		/* B8 */
	"HcETDEnableClear",		/* BC */
	"HcETDEnableStatus",	/* C0 */
	"HcNoRegister44",		/* C4 */
	"HcNoRegister48",		/* C8 */
	"HcDirectRouting",		/* CC */
	"HcImmediateIntr",		/* D0 */
	"HcDoneQueue",			/* D4 */
	"HcFmInterval",			/* D8 */
	"HcFmRemaining",		/* DC */
	"HcFmNumber",			/* E0 */
	"HcLSThreshold",		/* E4 */
	"HcRhDescriptorA",		/* E8 */
	"HcRhDescriptorB",		/* EC */
	"HcRhStatus",			/* F0 */
	"HcRhPortStatus0",		/* F4 */
	"HcRhPortStatus1",		/* F8 */
	"HcRhPortStatus2",		/* FC */
	"FcDeviceAddress",		/* 100 */
	"FcCommand",			/* 104 */
	"FcEP0Control",			/* 108 */
	"FcEP0PacketSize",		/* 10C */
	"FcEP1Config",			/* 110 */
	"FcEP1BufferStart",		/* 114 */
	"FcEP1PacketControl",	/* 118 */
	"FcNoRegister1C",		/* 11C */
	"FcEP2Config",			/* 120 */
	"FcEP2BufferStart",		/* 124 */
	"FcEP2PacketControl",	/* 128 */
	"FcNoRegister2C",		/* 12C */
	"FcEP3Config",			/* 130 */
	"FcEP3BufferStart",		/* 134 */
	"FcEP3PacketControl",	/* 138 */
	"FcNoRegister3C",		/* 13C */
	"FcEP4Config",			/* 140 */
	"FcEP4BufferStart",		/* 144 */
	"FcEP4PacketControl",	/* 148 */
	"FcNoRegister4C",		/* 14C */
	"FcEP5Config",			/* 150 */
	"FcEP5BufferStart",		/* 154 */
	"FcEP5PacketControl",	/* 158 */
	"FcNoRegister5C",		/* 15C */
	"FcEP6Config",			/* 160 */
	"FcEP6BufferStart",		/* 164 */
	"FcEP6PacketControl",	/* 168 */
	"FcFcNoRegister6C",		/* 16C */
	"FcEP7Config",			/* 170 */
	"FcEP7BufferStart",		/* 174 */
	"FcEP7PacketControl",	/* 178 */
	"FcNoRegister7C",		/* 17C */
	"FcNoRegister80",		/* 180 */
	"FcNoRegister84",		/* 184 */
	"FcNoRegister88",		/* 188 */
	"FcNoRegister8C",		/* 18C */
	"FcNoRegister90",		/* 190 */
	"FcNoRegister94",		/* 194 */
	"FcNoRegister98",		/* 198 */
	"FcNoRegister9C",		/* 19C */
	"FcBufferFilled",		/* 1A0 */
	"FcBufferFilledSet",	/* 1A4 */
	"FcBufferFilledClear",	/* 1A8 */
	"FcNoRegisterAC",		/* 1AC */
	"FcEPIntrStatus",		/* 1B0 */
	"FcEPIntrEnable",		/* 1B4 */
	"FcEPIntrClear",		/* 1B8 */
	"FcEPIntrSet",			/* 1BC */
	"FcSysIntrStatus",		/* 1C0 */
	"FcSysIntrEnable",		/* 1C4 */
	"FcSysIntrClear",		/* 1C8 */
	"FcSysIntrSet",			/* 1CC */
	"FcStartAddr",			/* 1D0 */
	"FcNoRegisterD4",		/* 1D4 */
	"FcNoRegisterD8",		/* 1D8 */
	"FcNoRegisterDC",		/* 1DC */
	"FcChipId",				/* 1E0 */
	"FcFrameNo",			/* 1E4 */
	"FcUnlockRegister",		/* 1E8 */
	"FcNoRegisterEC",		/* 1EC */
};

static char *
dumpreg_GetRegName(int reg)
{
	return regNames[reg >> 2];
}

static UINT32
dumpreg_ReadReg(int reg)
{
	UINT32 w;

#ifdef OTG_16_BIT_DATA_BUS
	w = readw(otg243_map + reg);
	w |= readw(otg243_map + reg) << 16;
#else
	w = readl(otg243_map + reg);
#endif

	return w;
}

static void
dumpreg_WriteReg(int reg, UINT32 val)
{
#ifdef OTG_16_BIT_DATA_BUS
	writew(val, otg243_map + reg);
	writew(val >> 16, otg243_map + reg);
#else
	writel(val, otg243_map + reg);
#endif
}

static int
dumpreg_IsIoReady(void)
{
	UINT32 w;

	w = dumpreg_ReadReg(otg_IOReady_c);
	return !(w & otg_IOReady_NotReady);
}

static void
dumpreg_Dump(void)
{
	int reg;
	int i;

	info("DumpReg: Register and ETD dump");

	for(reg = 0; reg < 0x80; reg += 4)
	{
		printk
		(
			"[0x%02x:%s]=0x%08x\n",
			reg,
			dumpreg_GetRegName(reg),
			dumpreg_ReadReg(reg)
		);
	}

	OS_Wait(2000);

	for(reg = 0x80; reg < 0x100; reg += 4)
	{
		printk
		(
			"[0x%0x:%s]=0x%08x\n",
			reg,
			dumpreg_GetRegName(reg),
			dumpreg_ReadReg(reg)
		);
	}

	OS_Wait(2000);

	for(reg = 0x100; reg < 0x1F0; reg += 4)
	{
		printk
		(
			"[0x%0x:%s]=0x%08x\n",
			reg,
			dumpreg_GetRegName(reg),
			dumpreg_ReadReg(reg)
		);
	}

	OS_Wait(2000);

	dumpreg_WriteReg
	(
		otg_HcRegBankStart_c + otg_hc_AccessStartAddr_c,
		otg_hc_StartEtd_c | otg_hc_StartRead_c
	);

	i = 0;
	while (!dumpreg_IsIoReady())
	{
		i++;
		if (i > 1000)
		{
			err("Chip did not respond");
			return;
		}
	}

	reg = otg_PIO_dataPort_c;
	for(i = 0; i < otg_hc_etd_MaxEtd_c; i++)
	{
		printk("ETD%d: \n", i);

		printk
		(
			"    0x%08x 0x%08x 0x%08x 0x%08x\n",
			dumpreg_ReadReg(reg),
			dumpreg_ReadReg(reg),
			dumpreg_ReadReg(reg),
			dumpreg_ReadReg(reg)
		);

		printk
		(
			"    0x%08x 0x%08x 0x%08x 0x%08x\n",
			dumpreg_ReadReg(reg),
			dumpreg_ReadReg(reg),
			dumpreg_ReadReg(reg),
			dumpreg_ReadReg(reg)
		);
	}
}

#ifdef CONFIG_SA1100_CERF
static void
dumpreg_CS(void)
{
	UINT32 w;

	w = MSC2;
	printk("MSC2=0x%08x MCS2=%p\n", w, (volatile UINT32 *)io_p2v(_MSC2));
}
#elif CONFIG_COLDFIRE
#include <asm/coldfire.h>
#include <asm/m5272sim.h>
static void
dumpreg_CS(void)
{
	UINT32 csbr;
	UINT32 csor;

	printk("CSBR0=0x%08x CSOR0=0x%08x\n",
			*(volatile UINT32 *)(MCFSIM_CSBR0 + MCF_MBAR), 
			*(volatile UINT32 *)(MCFSIM_CSOR0 + MCF_MBAR));
	printk("CSBR1=0x%08x CSOR1=0x%08x\n", 
			*(volatile UINT32 *)(MCFSIM_CSBR1 + MCF_MBAR), 
			*(volatile UINT32 *)(MCFSIM_CSOR1 + MCF_MBAR));

	csbr = *(volatile UINT32 *)(MCFSIM_CSBR2 + MCF_MBAR); 
	csor = *(volatile UINT32 *)(MCFSIM_CSOR2 + MCF_MBAR);
	printk("CSBR2=0x%08x CSOR2=0x%08x\n", csbr, csor);

	printk("CSBR3=0x%08x CSOR3=0x%08x\n", 
			*(volatile UINT32 *)(MCFSIM_CSBR3 + MCF_MBAR), 
			*(volatile UINT32 *)(MCFSIM_CSOR3 + MCF_MBAR));
	printk("CSBR4=0x%08x CSOR4=0x%08x\n", 
			*(volatile UINT32 *)(MCFSIM_CSBR4 + MCF_MBAR), 
			*(volatile UINT32 *)(MCFSIM_CSOR4 + MCF_MBAR));
	printk("CSBR5=0x%08x CSOR5=0x%08x\n", 
			*(volatile UINT32 *)(MCFSIM_CSBR5 + MCF_MBAR), 
			*(volatile UINT32 *)(MCFSIM_CSOR5 + MCF_MBAR));
	printk("CSBR6=0x%08x CSOR6=0x%08x\n", 
			*(volatile UINT32 *)(MCFSIM_CSBR6 + MCF_MBAR), 
			*(volatile UINT32 *)(MCFSIM_CSOR6 + MCF_MBAR));
	printk("CSBR7=0x%08x CSOR7=0x%08x\n", 
			*(volatile UINT32 *)(MCFSIM_CSBR7 + MCF_MBAR), 
			*(volatile UINT32 *)(MCFSIM_CSOR7 + MCF_MBAR));

	csbr |= 0x00000200; /* Set to 32 bits */
	*(volatile UINT32 *)(MCFSIM_CSBR2 + MCF_MBAR); 
	csor |= 0xE00; /* Set ASET, WRAH and RDAH */
	*(volatile UINT32 *)(MCFSIM_CSOR2 + MCF_MBAR) = csor;
	printk("CSBR2=0x%08x CSOR2=0x%08x\n", csbr, csor);

	printk("ICR1=0x%08x ICR2=0x%08x\n", 
			*(volatile UINT32 *)(MCFSIM_ICR1 + MCF_MBAR), 
			*(volatile UINT32 *)(MCFSIM_ICR2 + MCF_MBAR));

	printk("ICR=0x%08x PITR=0x%08x\n", 
			*(volatile UINT32 *)(MCFSIM_ISR + MCF_MBAR), 
			*(volatile UINT32 *)(MCFSIM_PITR + MCF_MBAR));
}
#else
static void
dumpreg_CS(void)
{
}
#endif

static int __init dumpreg_init (void) 
{
	otg243_map = ioremap(OTG243_MEM_START, otg_MemMapSize_c);
	printk("memory start=0x%x\n", OTG243_MEM_START);
	if (!otg243_map)
	{
		err("remap of registers failed");
		return -1;
	}

	dumpreg_CS();
	dumpreg_Dump();
	iounmap(otg243_map);

	return 0;
}

void __exit dumpreg_cleanup (void) 
{      
}

module_init (dumpreg_init);
module_exit (dumpreg_cleanup);
