/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: keymain.c
 * Author: Biao Jia		1/21/02
 * Description:
 *    An application to read from standard in and send to keyboard
 *    driver. This is used to send characters through keyboard driver
 *    so that this device can behave as a keyboard.
 *
 * When used without command line parameter, it will send everything over.
 * When used with anything on command line, it will not send '\n'.
 */
#include <stdio.h>
#include <unistd.h>

int main(int argc, char *argv[])
{
	int command;
	void test_kbd(int);

	command = 0;
	if (argc > 1)
	{
		command = 1;
	}

	test_kbd(command);

	return 0;
}

void
test_kbd(int command)
{
	FILE *fp;

	fp = fopen("/dev/test_kbd", "w");
	if (!fp)
	{
		printf("Can't open device /dev/test_kbd to write\n");
		return;
	}

	while (1)
	{
		int ch;
		int len;

		ch = getchar();
		if (ch == EOF)
		{
			break;
		}
		else if (ch == '\n')
		{
			if (command)
			{
				/* do not send return. it will cause endless loop */
				continue;
			}
		}

		len = fputc(ch, fp);
		if (fflush(fp) == EOF)
		{
			printf("failed to flush fp\n");
		}

		if (isprint(len))
		{
			printf("'%c' written\n", len);
		}
		else
		{
			printf("0x%02x written\n", len & 0xFF);
		}
	}

	fclose(fp);
}
