/*
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg.c
 * Author: Biao Jia		1/21/02
 * Description:
 *    OTG243 chip level object.
 */
#include "os.h"
#include "stack_urb.h"
#include "otg.h"

#ifdef DEBUG_DONEQUEUE
UINT32 hc_intr_status = 0;
UINT32 hc_etd_intr_status = 0;
UINT32 hc_etd_enable_status = 0;
UINT32 hc_etd_filled_status = 0;
UINT32 hc_done_queue = 0;
#endif

static otg_t *otgDevice = 0;
static int otgRefCount = 0;

#ifdef DO_PROFILING
long otgTimeElapsed = 0; /* in ticks */
int otgTimeSpend = 0;	/* in micro-seconds */
int otgTimeStarted = 0;
int otgTimeStartFrame;
int otgTimeStopFrame;
int otgTimeStartFrameRemain;
int otgTimeStopFrameRemain;
#endif

#ifdef DEBUG_OTG_CHIP
static char* regNames[] =
{
	"HardwareMode",			/* 0 */
	"MNPControlStatus",		/* 4 */
	"OTGTimer1",			/* 8 */
	"OTGTimer2",			/* C */
	"ClockControl",			/* 10 */
	"DMA0Config",			/* 14 */
	"DMA0ETD",				/* 18 */
	"DMA1Config",			/* 1C */
	"DMA1ETD",				/* 20 */
	"DMA0DataPort",			/* 24 */
	"DMA1DataPort",			/* 28 */
	"PIODataPort",			/* 2C */
	"FuncRamStart",			/* 30 */
	"ChipIntrStatus",		/* 34 */
	"ChipIntrEnable",		/* 38 */
	"ChipIntrClear",		/* 3C */
	"ChipIntrSet",			/* 40 */
	"IOReady",				/* 44 */
	"IOConfig1",			/* 48 */
	"IOConfig2",			/* 4C */
	"ShortFrame",			/* 50 */
	"NoRegister54",			/* 54 */
	"NoRegister58",			/* 58 */
	"NoRegister5C",			/* 5C */
	"NoRegister60",			/* 60 */
	"NoRegister64",			/* 64 */
	"NoRegister68",			/* 68 */
	"NoRegister6C",			/* 6C */
	"NoRegister70",			/* 70 */
	"NoRegister74",			/* 74 */
	"NoRegister78",			/* 78 */
	"NoRegister7C",			/* 7C */
	"HcRevision",			/* 80 */
	"HcControl",			/* 84 */
	"HcCommandStatus",		/* 88 */
	"HcIntrStatus",			/* 8C */
	"HcIntrEnbale",			/* 90 */
	"HcIntrDisable",		/* 94 */
	"HcETDIntrStatus",		/* 98 */
	"HcETDIntrEnable",		/* 9C */
	"HcETDIntrDisable",		/* A0 */
	"HcStartAddr",			/* A4 */
	"HcXYBufferStatus",		/* A8 */
	"HcXYBufferSet",		/* AC */
	"HcXYBufferClear",		/* B0 */
	"HcNoRegister34",		/* B4 */
	"HcETDEnableSet",		/* B8 */
	"HcETDEnableClear",		/* BC */
	"HcETDEnableStatus",	/* C0 */
	"HcNoRegister44",		/* C4 */
	"HcNoRegister48",		/* C8 */
	"HcDirectRouting",		/* CC */
	"HcImmediateIntr",		/* D0 */
	"HcDoneQueue",			/* D4 */
	"HcFmInterval",			/* D8 */
	"HcFmRemaining",		/* DC */
	"HcFmNumber",			/* E0 */
	"HcLSThreshold",		/* E4 */
	"HcRhDescriptorA",		/* E8 */
	"HcRhDescriptorB",		/* EC */
	"HcRhStatus",			/* F0 */
	"HcRhPortStatus0",		/* F4 */
	"HcRhPortStatus1",		/* F8 */
	"HcRhPortStatus2",		/* FC */
	"FcDeviceAddress",		/* 100 */
	"FcCommand",			/* 104 */
	"FcEP0Control",			/* 108 */
	"FcEP0PacketSize",		/* 10C */
	"FcEP1Config",			/* 110 */
	"FcEP1BufferStart",		/* 114 */
	"FcEP1PacketControl",	/* 118 */
	"FcNoRegister1C",		/* 11C */
	"FcEP2Config",			/* 120 */
	"FcEP2BufferStart",		/* 124 */
	"FcEP2PacketControl",	/* 128 */
	"FcNoRegister2C",		/* 12C */
	"FcEP3Config",			/* 130 */
	"FcEP3BufferStart",		/* 134 */
	"FcEP3PacketControl",	/* 138 */
	"FcNoRegister3C",		/* 13C */
	"FcEP4Config",			/* 140 */
	"FcEP4BufferStart",		/* 144 */
	"FcEP4PacketControl",	/* 148 */
	"FcNoRegister4C",		/* 14C */
	"FcEP5Config",			/* 150 */
	"FcEP5BufferStart",		/* 154 */
	"FcEP5PacketControl",	/* 158 */
	"FcNoRegister5C",		/* 15C */
	"FcEP6Config",			/* 160 */
	"FcEP6BufferStart",		/* 164 */
	"FcEP6PacketControl",	/* 168 */
	"FcFcNoRegister6C",		/* 16C */
	"FcEP7Config",			/* 170 */
	"FcEP7BufferStart",		/* 174 */
	"FcEP7PacketControl",	/* 178 */
	"FcNoRegister7C",		/* 17C */
	"FcNoRegister80",		/* 180 */
	"FcNoRegister84",		/* 184 */
	"FcNoRegister88",		/* 188 */
	"FcNoRegister8C",		/* 18C */
	"FcNoRegister90",		/* 190 */
	"FcNoRegister94",		/* 194 */
	"FcNoRegister98",		/* 198 */
	"FcNoRegister9C",		/* 19C */
	"FcBufferFilled",		/* 1A0 */
	"FcBufferFilledSet",	/* 1A4 */
	"FcBufferFilledClear",	/* 1A8 */
	"FcNoRegisterAC",		/* 1AC */
	"FcEPIntrStatus",		/* 1B0 */
	"FcEPIntrEnable",		/* 1B4 */
	"FcEPIntrClear",		/* 1B8 */
	"FcEPIntrSet",			/* 1BC */
	"FcSysIntrStatus",		/* 1C0 */
	"FcSysIntrEnable",		/* 1C4 */
	"FcSysIntrClear",		/* 1C8 */
	"FcSysIntrSet",			/* 1CC */
	"FcStartAddr",			/* 1D0 */
	"FcNoRegisterD4",		/* 1D4 */
	"FcNoRegisterD8",		/* 1D8 */
	"FcNoRegisterDC",		/* 1DC */
	"FcChipId",				/* 1E0 */
	"FcFrameNo",			/* 1E4 */
	"FcUnlockRegister",		/* 1E8 */
	"FcNoRegisterEC",		/* 1EC */
};

char *
otg_GetRegName(int reg)
{
	return regNames[reg >> 2];
}

void
otg_DumpReg(otg_t *otg)
{
	int reg;

	info("DumpReg: ");
	for(reg = 0; reg < 0x80; reg += 4)
	{
		otg_ReadChipReg(otg, reg);
	}

	for(reg = 0x00; reg < 0x80; reg += 4)
	{
		otg_ReadHostReg(otg, reg);
	}

/*	for(reg = 0x00; reg < 0xF0; reg += 4)
	{
		otg_ReadFunctionReg(otg, reg);
	}
	OS_Wait(2000);*/
}

#endif

#ifdef DEBUG_ETD_ENABLE
void *
otg_GetMemMap(void)
{
	return otgDevice->mem_map;
}
#endif

OS_INTERRUPT_HANDLER_PROTOTYPE(otg_DoIntr, intr_num)
{
	otg_intr_t *intr;
	otg_t *otg;
	unsigned long flags;

	OS_ENTER_ISR()

	otg = otgDevice;
	if (!otgDevice)
	{
		return IRQ_NONE;
	}

#if (defined DEBUG_OTG_CHIP) | (defined DEBUG_OTG_OTHERS)
info("DoIntr: otg=%p", otg);
#endif
	intr = &otg->intr;
	OS_DisableInterrupts(flags)

#ifdef OTG_CLEAR_INTR
	ClearInterrupt();
#endif

#ifdef OTG_CORRECT_READ_CYCLE
	/* ChipId may changed because of read allignment */
	if (otg_IsRevisionMatch(readw(otg->mem_map + otg_Revision)))
	{
		readw(otg->mem_map + otg_Revision);
	}
	else
	{
		err("DoIntr: read unalligned corrected");
	}
#endif
#ifdef DEBUG_DONEQUEUE
hc_intr_status = otg_ReadHostReg(otg, otg_hc_intr_Status_c);
hc_etd_intr_status = otg_ReadHostReg(otg, otg_hc_etd_IntrStatus_c);
hc_etd_enable_status = otg_ReadHostReg(otg, otg_hc_etd_EnableStatus_c);
hc_etd_filled_status = otg_ReadHostReg(otg, otg_hc_etd_FilledStatus_c);
hc_done_queue = otg_ReadHostReg(otg, otg_hc_etd_DoneQueue_c);
#endif
#ifdef DO_PROFILING
	otg_TimeStart(otg);
#endif
	otg_intr_DisableAll(intr);

	while (otg_intr_Check(&(otg->intr), otg_intr_All_c))
	{
		if (otg_intr_IsTriggered(intr, otg_intr_HNPAll_c))
		{
#ifndef OTG_FUNCTION_ONLY
#ifndef OTG_HOST_ONLY
			otg_device_DoIntr(&otg->device, intr);
#endif
#endif
			otg_intr_ClearIntr(intr, otg_intr_HNPAll_c);
		}

		if (otg_intr_IsTriggered(intr, otg_intr_F_c))
		{
#ifndef OTG_HOST_ONLY
			otg_fc_DoIntr(&otg->fc);
#endif
			otg_intr_ClearIntr(intr, otg_intr_F_c);
		}
	
		if (otg_intr_IsTriggered(intr, otg_intr_H_c))
		{
			/* Host controller interrupt */
#ifndef OTG_FUNCTION_ONLY
			otg_hc_DoIntr(otg->hc);
#endif
			otg_intr_ClearIntr(intr, otg_intr_H_c);
		}

		if (otg_intr_IsTriggered(intr, otg_intr_EOT0_c))
		{
			otg_intr_ClearIntr(intr, otg_intr_EOT0_c);
		}

		if (otg_intr_IsTriggered(intr, otg_intr_EOT1_c))
		{
			otg_intr_ClearIntr(intr, otg_intr_EOT1_c);
		}

		otg_intr_Clear(intr);
	}
	otg_intr_EnableSelected(intr);
#ifdef DO_PROFILING
	otg_TimeStop(otg);
#endif
	OS_EnableInterrupts(flags);

	OS_EXIT_ISR()

	return IRQ_HANDLED;
}

void
otg_Reset(otg_t *otg)
{
	otg_mode_Reset(&otg->mode);
	otg_dma_Reset(&otg->dma[0]);
	otg_dma_Reset(&otg->dma[1]);
	otg_intr_Reset(&otg->intr);

#ifndef OTG_FUNCTION_ONLY
	otg_hc_ResetUnlocked(otg->hc);
#endif

#ifndef OTG_HOST_ONLY
	otg_fc_ResetUnlocked(&otg->fc);
#endif

#ifndef OTG_FUNCTION_ONLY
#ifndef OTG_HOST_ONLY
	otg_device_Reset(&otg->device);
#endif
#endif
}

/* Will be called from alloc_uhci. otg will be allocated by uhci driver */
int
otg_Init(otg_t *otg, unsigned long address)
{
	unsigned long flags;
	int fc_mem_size;
	int hc_mem_size;
	UINT32 w;
	int rc;

#ifdef CONFIG_XSCALE_KOREBOT
	HWReset();
#endif /* CONFIG_XSCALE_KOREBOT */

#ifdef OTG_ADJUST_TIMING
	ModifySpeed();
#endif

#ifdef SIM_OTG
	otg->mem_map = (void *)address;
	info("Init: otg=%p hc=%p fc=%p", otg, &otg->hc, &otg->fc);
#else
	otg->mem_map = ioremap(address, otg_MemMapSize_c);
#ifdef DEBUG_OTG_CHIP
#ifdef OTG_16_BIT_DATA_BUS
info("Init: mem_map=%p 16 bit mode", otg->mem_map);
#else
info("Init: mem_map=%p", otg->mem_map);
#endif
#endif
#endif /* #ifdef SIM_OTG */
	if (otg->mem_map == 0)
	{
		info("Remap of register to memory failed!");
		return -1;
	}

	if (!otg_IsRevisionMatch(otg_ReadChipReg(otg, otg_Revision)))
	{
		err("OTG243 revision not match");
		return -1;
	}
	printk("Revision is %x\n", otg_ReadChipReg(otg, otg_Revision));

	spin_lock_init(&otg->chip_lock);

	otg->sofCount = 0;
	otg_LockChipIntr(otg, flags);

	otg_mode_Init(&otg->mode, otg);
	otg_dma_Init(&otg->dma[0], otg, 0);
	otg_dma_Init(&otg->dma[1], otg, 1);
	otg_intr_Init(&otg->intr, otg);
	otg_io_config_Init(&otg->ioConfig[0], otg, 0);
	otg_io_config_Init(&otg->ioConfig[1], otg, 1);

#ifdef OTG_HOST_ONLY
	w = 0x5;
#else
#ifdef OTG_FUNCTION_ONLY
	w = 0x3;
#else
	w = 0x7;
#endif
#endif

	otg_WriteChipReg(otg, otg_ClockCtl_c, w);

#ifdef OTG_HOST_ONLY
	fc_mem_size = otg_ep0_GetChipBufferSize();
#else
#ifdef OTG_FUNCTION_ONLY
	fc_mem_size = otg_mem_Size_c;
#else
	fc_mem_size = 1024;
#endif /* OTG_FUNCTION_ONLY */
#endif /* OTG_HOST_ONLY */

	hc_mem_size = otg_mem_Size_c - fc_mem_size;
#ifndef OTG_FUNCTION_ONLY
	otg_hc_Init(otg->hc, otg, otg_mem_Start_c, hc_mem_size);
#endif

#ifndef OTG_HOST_ONLY
	otg_WriteChipReg(otg, otg_FncRamStart_c, hc_mem_size >> 2);
	otg_fc_Init(&otg->fc, otg, hc_mem_size, fc_mem_size);
#endif

#ifndef OTG_FUNCTION_ONLY
#ifndef OTG_HOST_ONLY
	otg_device_Init(&otg->device, otg);
#endif
#endif

	otg_Reset(otg);

	rc = otg_mem_Test(otg);
	if (rc)
	{
		err("Init: memory test failed with error %d", rc);
		otg_UnlockChipIntr(otg, flags);
		return rc;
	}

#ifdef DEBUG_OTG_CHIP
info("Init: memory test successful");
#endif

	otg_UnlockChipIntr(otg, flags);

	return 0;
}

/* otg_New and otg_Delete can not be called from inside interrupt */
otg_t *
otg_New(otg_hc_t* otg_hc)
{
#ifdef DEBUG_OTG_CHIP
info("New: otgRefCount=%d", otgRefCount);
#endif
	if (otgRefCount)
	{
		otgRefCount++;
		if (otg_hc)
			otgDevice->hc = otg_hc;
		return otgDevice;
	}

	otgDevice = OS_Malloc(sizeof(otg_t));
	if (!otgDevice)
	{
		return 0;
	}

	OS_MemSet(otgDevice, 0, sizeof(otg_t));
	otgDevice->hc = otg_hc;

	if (otg_Init(otgDevice, OTG243_MEM_START) < 0)
	{
		OS_Free(otgDevice);
		otgDevice = 0;
		return 0;
	}

	if (RequestIrq(OTG243_INT_LVL, otg_DoIntr, otgDevice))
	{
		err("New: request_irq %d failed", OTG243_INT_LVL);
		otg_Exit(otgDevice);
		OS_Free(otgDevice);
		otgDevice = 0;
		return 0;
	}

	otg_SetIntrNo(otgDevice, OTG243_INT_LVL);

	otgRefCount++;
	return otgDevice;
}

void
otg_Delete(otg_t *otg)
{
#ifdef DEBUG_OTG_CHIP
info("Delete: otgRefCount=%d otgDevice=%p", otgRefCount, otgDevice);
#endif
	otgRefCount--;
	if (!otgRefCount)
	{
#ifdef DEBUG_OTG_CHIP
info("Delete: intr_no=%d", otg_GetIntrNo(otgDevice));
#endif
		OS_UnhookNonPCIInterruptVector(otg_GetIntrNo(otgDevice), otgDevice);
		otg_Exit(otgDevice);
		OS_Free(otgDevice);
		otgDevice = 0;
	}
	else if (otgRefCount < 0)
	{
		err("Delete: reference count negative");
		return;
	}
}

/* At this point, all ETDs should be freed and interrupt number freed */
void
otg_Exit(otg_t *otg)
{
#ifdef DEBUG_OTG_CHIP
	info("Exit: otg=%p", otg);
#endif /* #ifdef DEBUG_OTG_CHIP */

#ifndef OTG_FUNCTION_ONLY
#ifndef OTG_HOST_ONLY
	otg_device_Exit(&otg->device);
#endif
#endif

#ifndef OTG_FUNCTION_ONLY
	otg_hc_Exit(otg->hc);
#endif

#ifndef OTG_HOST_ONLY
	otg_fc_Exit(&otg->fc);
#endif

#ifndef SIM_OTG
	iounmap(otg->mem_map);
#endif /* #ifndef SIM_OTG */

#ifdef DEBUG_OTG_CHIP
	info("Exit: returning");
#endif
}

#ifdef DO_PROFILING
void
otg_TimeAdd(int time)
{
	if (!otgTimeElapsed)
	{
		/* First time, start */
		otgTimeElapsed = OS_ReadTimeMS();
		otgTimeSpend = time;
		return;
	}

	otgTimeSpend += time;
	if (OS_ReadTimeMS() - otgTimeElapsed > 10000)
	{
		/* Print every 10 seconds */
		printk("Time elapsed: %ld (ms)  Time spent: %d (us)\n",
			(OS_ReadTimeMS() - otgTimeElapsed), otgTimeSpend);

		otgTimeElapsed = OS_ReadTimeMS();
		otgTimeSpend = time;
	}
}

void
otg_TimeStart(otg_t *otg)
{
	if (otgTimeStarted)
	{
		printk("Time not stopped\n");
	}

	otgTimeStarted = 1;
	otgTimeStartFrame = 
		otg_ReadHostReg(otg, otg_hc_FmNumber_c) & otg_hc_FmMask_c;
	otgTimeStartFrameRemain =
		otg_ReadHostReg(otg, otg_hc_FmRemaining_c) & otg_hc_FmRemainingMask_c;
}

void
otg_TimeStop(otg_t *otg)
{
	int frames;
	int bits;

	if (!otgTimeStarted)
	{
		printk("Time not started\n");
	}

	otgTimeStopFrame =
		otg_ReadHostReg(otg, otg_hc_FmNumber_c) & otg_hc_FmMask_c;
	otgTimeStopFrameRemain =
		otg_ReadHostReg(otg, otg_hc_FmRemaining_c) & otg_hc_FmRemainingMask_c;
	
	otgTimeStarted = 0;
	frames = (otgTimeStopFrame - otgTimeStartFrame) & otg_hc_FmMask_c;
	if (frames)
	{
		bits = 11999 - otgTimeStopFrameRemain;
	}
	else
	{
		bits = otgTimeStartFrameRemain - otgTimeStopFrameRemain;
	}

	otg_TimeAdd(frames * 1000 + bits / 12);
}
#endif
