/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_adevice.c
 * Author: Biao Jia		1/21/02
 * Description:
 *    A-device object.
 */
#include "os.h"
#include "stack_urb.h"
#include "otg.h"

void
otg_adevice_DoIntr(otg_device_t *dev)
{
	switch (dev->state)
	{
		case otg_adevice_StateIdle:
			otg_adevice_DoIdle(dev);
			break;
		case otg_adevice_StateHost:
			otg_adevice_DoHost(dev);
			break;
		case otg_adevice_StateDevice:
			otg_adevice_DoDevice(dev);
			break;
		default:
			err("DoIntr: Unknown state 0x%0x", dev->state);
	}
}

void
otg_adevice_DoIdle(otg_device_t *dev)
{
#ifdef OTG_HNP_DEBUG
		info("DoIdle: otg=%p status=0x%0x", dev->otg, dev->status);
#endif
	if (!otg_device_IsStatus(dev, otg_device_HNPStatusIsADevice))
	{
		otg_adevice_ToBDevice(dev);
		otg_bdevice_ToBDevice(dev);
		return;
	}

	if (otg_device_IsStatus(dev, otg_device_HNPStatusMaster))
	{
#ifdef OTG_HNP_DEBUG
		info("DoIdle:  Master State");
#endif
		dev->state = otg_adevice_StateHost;
		otg_device_EnableBus(dev);
return;
		otg_adevice_DoHost(dev);
	}
	else if (otg_device_IsStatus(dev, otg_device_HNPStatusSlave))
	{
#ifdef OTG_HNP_DEBUG
		info("DoIdle:  Slave State");
#endif
		/* Enable function on port, nothing to do here */
		dev->state = otg_adevice_StateDevice;
		otg_fc_StartTimer(otg_GetFunction(dev->otg));
		otg_adevice_DoDevice(dev);
	}
}

void
otg_adevice_DoHost(otg_device_t *dev)
{
#ifdef OTG_HNP_DEBUG
		info("DoHost: otg=%p status=0x%0x", dev->otg, dev->status);
#endif
	if (!otg_device_IsStatus(dev, otg_device_HNPStatusIsADevice))
	{
		otg_device_DisableBus(dev);
		otg_adevice_ToBDevice(dev);
		otg_bdevice_ToBDevice(dev);
		return;
	}

	if (otg_device_IsStatus(dev, otg_device_HNPStatusSlave))
	{
#ifdef OTG_HNP_DEBUG
		info("DoHost:  Slave State");
#endif
		otg_device_DisableBus(dev);
		dev->state = otg_adevice_StateDevice;
		otg_fc_StartTimer(otg_GetFunction(dev->otg));
		otg_adevice_DoDevice(dev);
	}
	else if (!otg_device_IsStatus(dev, otg_device_HNPStatusMaster))
	{
#ifdef OTG_HNP_DEBUG
		info("DoHost:  non-Master State");
#endif
		otg_device_DisableBus(dev);
		dev->state = otg_adevice_StateIdle;
		otg_adevice_DoIdle(dev);
	}
}

void
otg_adevice_DoDevice(otg_device_t *dev)
{
#ifdef OTG_HNP_DEBUG
		info("DoDevice: otg=%p status=0x%0x", dev->otg, dev->status);
#endif

	if (!otg_device_IsStatus(dev, otg_device_HNPStatusIsADevice))
	{
		otg_fc_Disable(otg_GetFunction(dev->otg));
		otg_device_EnableBus(dev);
		otg_adevice_ToBDevice(dev);
		otg_bdevice_ToBDevice(dev);
		otg_device_RequestBus(dev);
		otg_device_ClearAvBHnpEnable(dev);
		return;
	}

	if (otg_device_IsStatus(dev, otg_device_HNPStatusMaster))
	{
#ifdef OTG_HNP_DEBUG
		info("DoDevice:  Master State");
#endif
		otg_fc_Disable(otg_GetFunction(dev->otg));
		otg_device_EnableBus(dev);
		dev->state = otg_adevice_StateHost;
		otg_device_RequestBus(dev);
		otg_device_ClearAvBHnpEnable(dev);
		otg_adevice_DoHost(dev);
	}
	else if (!otg_device_IsStatus(dev, otg_device_HNPStatusSlave))
	{
#ifdef OTG_HNP_DEBUG
		info("DoDevice:  non-Slave State");
#endif
		dev->state = otg_adevice_StateIdle;
		otg_device_EnableBus(dev);
		otg_fc_Disable(otg_GetFunction(dev->otg));
		otg_device_RequestBus(dev);
		otg_device_ClearAvBHnpEnable(dev);
		otg_adevice_DoIdle(dev);
	}
}

void
otg_adevice_ToADevice(otg_device_t *dev)
{
	dev->state = otg_adevice_StateIdle;
	otg_device_ReadStatus(dev);
	dev->status |= otg_device_HNPStatusBusRequest;
	dev->status &= ~otg_device_HNPStatusBusDrop;
	otg_WriteChipReg(dev->otg, otg_device_HNPStatus_c, dev->status);

	otg_adevice_DoIdle(dev);
}

void
otg_adevice_ToBDevice(otg_device_t *dev)
{
	dev->isADevice = 0;

	otg_device_ReadStatus(dev);
#ifdef OTG_HNP_DEBUG
	info("ToBDevice: status=0x%0x", dev->status);
#endif
	dev->status &= ~otg_device_HNPStatusBusRequest;
	dev->status &= ~otg_device_HNPStatusBusDrop;
	otg_WriteChipReg(dev->otg, otg_device_HNPStatus_c, dev->status);
}

void
otg_adevice_Reset(otg_device_t *dev)
{
#ifdef OTG_HNP_DEBUG
	info("Reset: state=%d", dev->state);
#endif
	if (dev->state == otg_adevice_StateHost)
	{
		otg_device_DisableBus(dev);
	}
	else if (dev->state == otg_adevice_StateDevice)
	{
		otg_fc_Disable(otg_GetFunction(dev->otg));
	}

	dev->state = otg_adevice_StateIdle;
}

