/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_bdevice.c
 * Author: Biao Jia		1/21/02
 * Description:
 *    B-device object handler.
 */
#include "os.h"
#include "stack_urb.h"
#include "otg.h"

void
otg_bdevice_DoSofIntr(otg_device_t *dev)
{
	if (!dev->bdevice.waitSof)
	{
		return;
	}

#ifdef OTG_HNP_DEBUG
info("DoSofIntr: waitSof=%d", dev->bdevice.waitSof);
#endif
	if (dev->bdevice.waitSof == otg_bdevice_WaitAConnMs)
	{
		otg_device_ClearBHardReset(dev);
	}
	
	if (otg_device_IsConnected(dev))
	{
#ifdef OTG_HNP_DEBUG
info("DoSofIntr: a connected");
#endif
		/* Get a-connect, reset the port */
		otg_device_ResetBus(dev);
		otg_DisableSofIntr(dev->otg);
		dev->bdevice.waitSof = 0;
		return;
	}

	if (dev->bdevice.waitSof == 1)
	{
		/* Could not get a-connect, give up */
		otg_DisableSofIntr(dev->otg);
	}
	
	(dev->bdevice.waitSof)--;
}

void
otg_bdevice_Timeout(unsigned long data)
{
	otg_device_t *dev;

	dev = (otg_device_t *)data;
#ifdef OTG_HNP_DEBUG
info("Timeout: dev=%p", dev);
#endif
	dev->bdevice.timerSet = 0;
	dev->bdevice.waitSof = otg_bdevice_WaitAConnMs;
	otg_EnableSofIntr(dev->otg);
}

void
otg_bdevice_StartTimer(otg_device_t *dev)
{
	OS_DeclareTimer(*timer);

	timer = &dev->bdevice.timer;
	OS_StartTimer(timer, otg_bdevice_Timeout, dev, 10);
	dev->bdevice.timerSet = 1;
}

void
otg_bdevice_DoIntr(otg_device_t *dev)
{
	switch (dev->state)
	{
		case otg_bdevice_StateIdle:
			otg_bdevice_DoIdle(dev);
			break;
		case otg_bdevice_StateHost:
			otg_bdevice_DoHost(dev);
			break;
		case otg_bdevice_StateDevice:
			otg_bdevice_DoDevice(dev);
			break;
		default:
			err("DoIntr: Unknown state %d", dev->state);
	}
}

void
otg_bdevice_DoIdle(otg_device_t *dev)
{
#ifdef OTG_HNP_DEBUG
info("DoIdle: otg=%p status=0x%0x", dev->otg, dev->status);
#endif
	if (otg_device_IsStatus(dev, otg_device_HNPStatusIsADevice))
	{
		otg_bdevice_ToADevice(dev);
		otg_adevice_ToADevice(dev);
		return;
	}

	if (otg_device_IsStatus(dev, otg_device_HNPStatusMaster))
	{
#ifdef OTG_HNP_DEBUG
		info("DoIdle: master state");
#endif
		dev->state = otg_bdevice_StateHost;
		otg_device_EnableBus(dev);
		otg_device_ClearBHardReset(dev);
		otg_bdevice_DoHost(dev);
	}
	else if (otg_device_IsStatus(dev, otg_device_HNPStatusSlave))
	{
#ifdef OTG_HNP_DEBUG
		info("DoIdle: SlaveState");
#endif
		dev->state = otg_bdevice_StateDevice;
		otg_fc_StartTimer(otg_GetFunction(dev->otg));
		otg_bdevice_DoDevice(dev);
	}
}

void
otg_bdevice_DoDevice(otg_device_t *dev)
{
#ifdef OTG_HNP_DEBUG
info("DoDevice: otg=%p status=0x%0x", dev->otg, dev->status);
#endif
	if (otg_device_IsStatus(dev, otg_device_HNPStatusIsADevice))
	{
		otg_fc_Disable(otg_GetFunction(dev->otg));
		otg_bdevice_ToADevice(dev);
		otg_adevice_ToADevice(dev);
		return;
	}

	if (otg_device_IsStatus(dev, otg_device_HNPStatusMaster))
	{
#ifdef OTG_HNP_DEBUG
		info("DoDevice: master state");
#endif
		dev->state = otg_bdevice_StateHost;
		otg_fc_Disable(otg_GetFunction(dev->otg));
		otg_bdevice_StartTimer(dev);
		otg_bdevice_DoHost(dev);
	}
	else if (!otg_device_IsStatus(dev, otg_device_HNPStatusSlave))
	{
#ifdef OTG_HNP_DEBUG
		info("DoDevice: non-slave state");
#endif
		dev->state = otg_bdevice_StateIdle;
		otg_fc_Disable(otg_GetFunction(dev->otg));
		otg_bdevice_DoIdle(dev);
	}
}

void
otg_bdevice_DoHost(otg_device_t *dev)
{
#ifdef OTG_HNP_DEBUG
info("DoHost: otg=%p status=0x%0x", dev->otg, dev->status);
#endif
	if (otg_device_IsStatus(dev, otg_device_HNPStatusIsADevice))
	{
		otg_device_DisableBus(dev);
		otg_bdevice_ToADevice(dev);
		otg_adevice_ToADevice(dev);
		return;
	}

	if (otg_device_IsStatus(dev, otg_device_HNPStatusSlave))
	{
#ifdef OTG_HNP_DEBUG
		info("DoHost: slave state");
#endif
		otg_device_DisableBus(dev);
		dev->state = otg_bdevice_StateDevice;
		otg_fc_StartTimer(otg_GetFunction(dev->otg));
		otg_device_SetBHardReset(dev);
		otg_bdevice_DoDevice(dev);
	}
	else if (!otg_device_IsStatus(dev, otg_device_HNPStatusMaster))
	{
#ifdef OTG_HNP_DEBUG
		info("DoHost: non-master state");
#endif
		otg_device_DisableBus(dev);
		dev->state = otg_bdevice_StateIdle;
		otg_device_SetBHardReset(dev);
		otg_bdevice_DoIdle(dev);
	}
}

void
otg_bdevice_ToBDevice(otg_device_t *dev)
{
	dev->state = otg_bdevice_StateIdle;
	otg_device_SetBHardReset(dev);
	otg_bdevice_DoIdle(dev);
}

void
otg_bdevice_ToADevice(otg_device_t *dev)
{
	dev->isADevice = 1;
}

void
otg_bdevice_Reset(otg_device_t *dev)
{
#ifdef OTG_HNP_DEBUG
	info("Reset: state=%d", dev->state);
#endif
	if (dev->state == otg_bdevice_StateHost)
	{
		otg_device_DisableBus(dev);
	}
	else if (dev->state == otg_bdevice_StateDevice)
	{
		otg_fc_Disable(otg_GetFunction(dev->otg));
	}

	otg_device_SetBHardReset(dev);
	if (dev->bdevice.timerSet)
	{
		OS_StopTimer(&dev->bdevice.timer);
		dev->bdevice.timerSet = 0;
	}

	if (dev->bdevice.waitSof)
	{
		dev->bdevice.waitSof = 0;
		otg_DisableSofIntr(dev->otg);
	}

	dev->state = otg_bdevice_StateIdle;
}

