/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_buffer.c
 * Author: Biao Jia		5/5/02
 * Description:
 *    Buffer handler.
 */

#include "os.h"
#include "stack_urb.h"
#include "otg.h"
#include "otg_buffer.h"

void
otg_buffer_Init(otg_buffer_t *buf)
{
	buf->readPos = 0;
	buf->writePos = 0;

	return;
}

otg_buffer_t *
otg_buffer_New(int size)
{
	otg_buffer_t *buf;

	buf = OS_Malloc(sizeof(otg_buffer_t));
	if (!buf)
	{
		return 0;
	}

	buf->data = OS_Malloc(size);
	if (!buf->data)
	{
		otg_buffer_Delete(buf);
		return 0;
	}

	buf->size = size;
	otg_buffer_Init(buf);

	return buf;
}

void
otg_buffer_Delete(otg_buffer_t *buf)
{
	if (buf->data)
	{
		OS_Free(buf->data);
	}

	OS_Free(buf);
}

int
otg_buffer_GetData(otg_buffer_t *buf, UINT8 *toBuf, int max)
{
	int count;
	int len;

#ifdef DEBUG_OTG_CHIP
info("GetData: max=%d", max);
#endif
	count = 0;
	while(!otg_buffer_IsEmpty(buf))
	{
		if (count >= max)
		{
			break;
		}

		if (buf->writePos > buf->readPos)
		{
			len = buf->writePos - buf->readPos;
		}
		else
		{
			len = buf->size - buf->readPos;
		}

		if (len > max - count)
		{
			len = max - count;
		}

		memcpy(&toBuf[count], &buf->data[buf->readPos], len);
		otg_buffer_IncReadPos(buf, len);
		count += len;
#ifdef DEBUG_OTG_CHIP
info("GetData: len=%d read_pos=%d write_pos=%d count=%d", 
	len, buf->readPos, buf->writePos, count);
#endif
	}

#ifdef DEBUG_OTG_CHIP
info("GetData: count=%d", count);
#endif
	return count;
}

int
otg_buffer_PutData(otg_buffer_t *buf, UINT8 *fromBuf, int max)
{
	int count;
	int len;

#ifdef DEBUG_OTG_CHIP
info("PutData: max=%d", max);
#endif
	count = 0;
	while(!otg_buffer_IsFull(buf))
	{
		if (count >= max)
		{
			break;
		}

		if (buf->writePos >= buf->readPos)
		{
			len = buf->size - buf->writePos;
		}
		else
		{
			len = buf->readPos - buf->writePos - 1;
		}

		if (len > max - count)
		{
			len = max - count;
		}

		memcpy(&buf->data[buf->writePos], &fromBuf[count], len);
		otg_buffer_IncWritePos(buf, len);
		count += len;
#ifdef DEBUG_OTG_CHIP
info("PutData: len=%d read_pos=%d write_pos=%d count=%d", 
	len, buf->readPos, buf->writePos, count);
#endif
	}

	return count;
}

int
otg_buffer_GetMaxReadLength(otg_buffer_t *buf)
{
	if (buf->writePos > buf->readPos)
	{
		return buf->writePos - buf->readPos;
	}
	else
	{
		return buf->size - buf->readPos;
	}
}

int
otg_buffer_GetMaxWriteLength(otg_buffer_t *buf)
{
	if (buf->writePos >= buf->readPos)
	{
		return buf->size - buf->writePos;
	}
	else
	{
		return buf->readPos - buf->writePos - 1;
	}
}

