/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_buffer.h
 * Author: Biao Jia		5/5/02
 * Description:
 *    Data buffer function definitions.
 */

#ifndef OTG_BUFFER_H
#define OTG_BUFFER_H

#include "otg_buffer_t.h"

void otg_buffer_Init(otg_buffer_t *buf);
otg_buffer_t *otg_buffer_New(int size);
void otg_buffer_Delete(otg_buffer_t *buf);

int otg_buffer_GetData(otg_buffer_t *buf, UINT8 *toBuf, int max);
int otg_buffer_PutData(otg_buffer_t *buf, UINT8 *fromBuf, int max);

int otg_buffer_GetMaxReadLength(otg_buffer_t *buf);
int otg_buffer_GetMaxWriteLength(otg_buffer_t *buf);

static inline int
otg_buffer_IsEmpty(otg_buffer_t *buf)
{
	return buf->readPos == buf->writePos;
}

static inline int
otg_buffer_IsFull(otg_buffer_t *buf)
{
	if (buf->readPos)
	{
		return buf->writePos == buf->readPos - 1;
	}
	else
	{
		return buf->writePos == buf->size - 1;
	}
}

static inline void
otg_buffer_IncReadPos(otg_buffer_t *buf, int len)
{
	buf->readPos += len;
	if (buf->readPos >= buf->size)
	{
		buf->readPos -= buf->size;
	}
}

static inline void
otg_buffer_IncWritePos(otg_buffer_t *buf, int len)
{
	buf->writePos += len;
	if (buf->writePos >= buf->size)
	{
		buf->writePos -= buf->size;
	}
}

static inline UINT8 *
otg_buffer_GetReadPtr(otg_buffer_t *buf)
{
	return &buf->data[buf->readPos];
}

static inline UINT8 *
otg_buffer_GetWritePtr(otg_buffer_t *buf)
{
	return &buf->data[buf->writePos];
}

#endif /* #ifndef OTG_BUFFER_H */
