/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_config_desc.c
 * Author: Biao Jia		1/21/02
 * Description:
 *    Configuration object handler.
 */
#include "os.h"
#include "stack_urb.h"
#include "otg.h"

otg_interface_desc_t *
otg_config_desc_GetInterface(otg_config_desc_t *config, int index)
{
	int i;
	otg_interface_desc_t *intf;

	intf = config->interfaceDesc;
	for(i = 0; i < config->numInterfaces; i++)
	{
		if (otg_interface_desc_IsInterface(intf, index))
		{
			return intf;
		}

		intf++;
	}

	return 0;
}

int
otg_config_desc_SetDesc(otg_config_desc_t *config, UINT8 *desc, int tot)
{
	UINT8 b;
	int len;
	int count;
	int rc;
	int total;

	len = desc[otg_config_desc_bLength];
	if (len != otg_config_desc_Max || len > tot)
	{
		err("SetDesc: descriptor size %d is incorrect", len);
		return -EINVAL;
	}

	b = desc[otg_config_desc_bDescriptorType];
	if (otg_config_desc_Type != b)
	{
		err("SetDesc: wrong descriptor type %d", b);
		return -EINVAL;
	}

	config->desc = desc;
	config->numInterfaces = 0;
	count = 0;
	while (len < tot)
	{
		if (count >= otg_config_desc_MaxInterface)
		{
			err("SetDesc: too many interfaces");
			return -EINVAL;
		}

		if (!otg_interface_desc_IsInterfaceDesc(&desc[len]))
		{
			break;
		}

		rc = otg_interface_desc_SetDesc(&config->interfaceDesc[count],
				&desc[len], tot - len);
		if (rc < 0)
		{
			return rc;
		}

		len += rc;
		count++;
	}

	total = desc[otg_config_desc_wTotalLength];
	total |= desc[otg_config_desc_wTotalLength + 1] << 8;
	if (len != total)
	{
		err("SetDesc: total length %d incorrect %d", len, total);
		return -EINVAL;
	}

	b = desc[otg_config_desc_bNumInterfaces];
	if (b != count)
	{
		err("SetDesc: number of interfaces %d incorrect", b);
		return -1;
	}

	config->numInterfaces = count;
	config->length = len;

	return len;
}
