/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_device.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    Superclass of a-device and b-device.
 */
#ifndef OTG_FUNCTION_ONLY
#ifndef OTG_HOST_ONLY
#ifndef OTG_DEVICE_H
#define OTG_DEVICE_H

#include "otg_device_t.h"
#include "otg_t.h"

#include "otg.h"

static inline void
otg_device_EnableIntr(otg_device_t *dev)
{
	UINT32 w;

	w = otg_intr_SlaveState_c | otg_intr_MasterState_c |
		otg_intr_VBusError_c | otg_intr_MnpDone_c | otg_intr_IdIsA_c;
	
	otg_intr_Enable(otg_GetIntr(dev->otg), w);
}

static inline int
otg_device_CheckIntr(otg_device_t *dev, otg_intr_t *intr)
{
	UINT32 w;

	w = otg_intr_SlaveState_c | otg_intr_MasterState_c |
		otg_intr_VBusError_c | otg_intr_MnpDone_c | otg_intr_IdIsA_c;
	
	return otg_intr_IsTriggered(intr, w);
}

static inline void
otg_device_ReadStatus(otg_device_t *dev)
{
	dev->status = otg_ReadChipReg(dev->otg, otg_device_HNPStatus_c);
}

static inline int
otg_device_CheckStatus(otg_device_t *dev, int status)
{
	dev->status = otg_ReadChipReg(dev->otg, otg_device_HNPStatus_c);
	return dev->status & status;
}

static inline int
otg_device_IsStatus(otg_device_t *dev, int status)
{
	return dev->status & status;
}

static inline int
otg_device_IsDevice(otg_device_t *dev)
{
	return dev->state == otg_adevice_StateDevice ||
		dev->state == otg_bdevice_StateDevice;
}

static inline void
otg_device_RequestBus(otg_device_t *dev)
{
	otg_device_ReadStatus(dev);
	dev->status |= otg_device_HNPStatusBusRequest;
	otg_WriteChipReg(dev->otg, otg_device_HNPStatus_c, dev->status);
	dev->isBusRequested = 1;
#ifdef OTG_HNP_DEBUG
info("RequestBus: status=0x%0x", dev->status);
#endif
}

static inline void
otg_device_ReleaseBus(otg_device_t *dev)
{
	otg_device_ReadStatus(dev);
	dev->status &= ~otg_device_HNPStatusBusRequest;
	otg_WriteChipReg(dev->otg, otg_device_HNPStatus_c, dev->status);
	dev->isBusRequested = 0;
}

static inline void
otg_device_ClearBHnpEnable(otg_device_t *dev)
{
	otg_device_ReadStatus(dev);
	dev->status &= ~otg_device_HNPStatusBHNPEnable;
	otg_WriteChipReg(dev->otg, otg_device_HNPStatus_c, dev->status);
}

static inline void
otg_device_SetBHnpEnable(otg_device_t *dev)
{
	otg_device_RequestBus(dev);
	otg_device_ReadStatus(dev);
	dev->status |= otg_device_HNPStatusBHNPEnable;
	otg_WriteChipReg(dev->otg, otg_device_HNPStatus_c, dev->status);
#ifdef OTG_HNP_DEBUG
info("SetBHnpEnable: status=0x%0x", dev->status);
#endif
}

static inline void
otg_device_ClearBHnpSupport(otg_device_t *dev)
{
	otg_device_ReadStatus(dev);
	dev->status &= ~otg_device_HNPStatusBHNPSupport;
	otg_WriteChipReg(dev->otg, otg_device_HNPStatus_c, dev->status);
}

static inline void
otg_device_SetBHnpSupport(otg_device_t *dev)
{
	otg_device_ReadStatus(dev);
	dev->status |= otg_device_HNPStatusBHNPSupport;
	otg_WriteChipReg(dev->otg, otg_device_HNPStatus_c, dev->status);
#ifdef OTG_HNP_DEBUG
info("SetBHnpSupport: status=0x%0x", dev->status);
#endif
}

static inline void
otg_device_ClearAvBHnpEnable(otg_device_t *dev)
{
	otg_device_ReadStatus(dev);
	dev->status &= ~otg_device_HNPStatusAvBHNPEnable;
	otg_WriteChipReg(dev->otg, otg_device_HNPStatus_c, dev->status);
}

static inline void
otg_device_SetAvBHnpEnable(otg_device_t *dev)
{
	otg_device_ReadStatus(dev);
	dev->status |= otg_device_HNPStatusAvBHNPEnable;
	otg_WriteChipReg(dev->otg, otg_device_HNPStatus_c, dev->status);
}

static inline void
otg_device_ClearError(otg_device_t *dev)
{
	otg_device_ReadStatus(dev);
	dev->status |= otg_device_HNPStatusClearError;
	otg_WriteChipReg(dev->otg, otg_device_HNPStatus_c, dev->status);
}

static inline void
otg_device_SetBHardReset(otg_device_t *dev)
{
	otg_device_ReadStatus(dev);
	dev->status |= otg_device_HNPStatusBReset;
	otg_WriteChipReg(dev->otg, otg_device_HNPStatus_c, dev->status);
}

static inline void
otg_device_ClearBHardReset(otg_device_t *dev)
{
	otg_device_ReadStatus(dev);
	dev->status &= ~otg_device_HNPStatusBReset;
	otg_WriteChipReg(dev->otg, otg_device_HNPStatus_c, dev->status);
}

static inline void
otg_device_Disconnect(otg_device_t *dev)
{
	otg_device_ReadStatus(dev);
	dev->status &= ~otg_device_HNPStatusSwPullUp;
	dev->status |= otg_device_HNPStatusSwDmPullDown |
			otg_device_HNPStatusSwDpPullDown;
	otg_WriteChipReg(dev->otg, otg_device_HNPStatus_c, dev->status);
}

#include "otg_bdevice.h"
#include "otg_adevice.h"

static inline void
otg_device_BHnpEnabled(otg_device_t *dev)
{
#ifdef OTG_HNP_DEBUG
info("BHnpEnable: dev->isADevice=%d", dev->isADevice);
#endif
	if (dev->isADevice)
	{
		return;
	}

	otg_device_SetBHnpEnable(dev);
}

static inline void
otg_device_Init(otg_device_t *dev, otg_t *otg)
{
#ifndef OTG_FUNCTION_ONLY
#ifndef OTG_HOST_ONLY
	otg_hc_t *hc;

	hc = otg_GetHost(otg);
	dev->port = otg_hc_rh_GetHNPPort(otg_hc_GetRootHub(hc));
#endif
#endif

#ifdef OTG_HNP_DEBUG
	info("Init: dev->port=%p", dev->port);
#endif
	dev->otg = otg;
	otg_adevice_Init(dev);
	otg_bdevice_Init(dev);

	dev->isADevice = 0;
	dev->state = otg_bdevice_StateIdle;
	dev->isBusRequested = 0;
}

static inline void
otg_device_Reset(otg_device_t *dev)
{
	dev->status = otg_ReadChipReg(dev->otg, otg_device_HNPStatus_c);
	if (dev->isADevice)
	{
		otg_adevice_Reset(dev);
	}
	else
	{
		otg_bdevice_Reset(dev);
	}

	dev->status = otg_ReadChipReg(dev->otg, otg_device_HNPStatus_c);
	dev->status &= ~(otg_device_HNPStatusBusDrop |
		otg_device_HNPStatusBHNPEnable | otg_device_HNPStatusBHNPSupport);
	dev->status |= (otg_device_HNPStatusBusRequest |
		otg_device_HNPStatusBandgap | otg_device_HNPStatusComparator);
	otg_WriteChipReg(dev->otg, otg_device_HNPStatus_c, dev->status);
#ifdef OTG_HNP_DEBUG
info("Reset: status=0x%0x", dev->status);
#endif

	otg_device_EnableIntr(dev);
}

static inline int
otg_device_IsConnected(otg_device_t *dev)
{
	return otg_hc_rh_port_IsConnected(dev->port);
}

static inline void
otg_device_ResetBus(otg_device_t *dev)
{
	otg_hc_rh_port_ResetPort(dev->port);
}

static inline void
otg_device_EnableBus(otg_device_t *dev)
{
	otg_hc_rh_port_EnableDownStream(dev->port);
}

static inline void
otg_device_DisableBus(otg_device_t *dev)
{
	otg_hc_rh_port_DisableDownStream(dev->port);
}

static inline void
otg_device_Exit(otg_device_t *dev)
{
#ifdef OTG_HNP_DEBUG
	info("Exit: status=%d", dev->status);
#endif
	if
	(
		dev->state == otg_adevice_StateHost ||
		dev->state == otg_bdevice_StateHost
	)
	{
		otg_device_DisableBus(dev);
	}
	else if 
	(
		dev->state == otg_adevice_StateDevice ||
		dev->state == otg_bdevice_StateDevice
	)
	{
		otg_fc_Disable(otg_GetFunction(dev->otg));
	}

	otg_device_ReadStatus(dev);
	dev->status &= ~(otg_device_HNPStatusBusRequest |
		otg_device_HNPStatusBandgap | otg_device_HNPStatusComparator);
	dev->status |= otg_device_HNPStatusBusDrop;
	otg_WriteChipReg(dev->otg, otg_device_HNPStatus_c, dev->status);
	dev->state = otg_bdevice_StateIdle;
}

static inline void
otg_device_DoIntr(otg_device_t *dev, otg_intr_t *intr)
{
	dev->status = otg_ReadChipReg(dev->otg, otg_device_HNPStatus_c);
	if (otg_intr_IsTriggered(otg_GetIntr(dev->otg), otg_intr_VBusError_c))
	{
		dev->status |= otg_device_HNPStatusClearError;
		otg_WriteChipReg(dev->otg, otg_device_HNPStatus_c, dev->status);
#ifdef OTG_HNP_DEBUG
info("DoIntr: VBus error detected. Status=0x%0x", dev->status);
#endif
	}

	if (dev->isADevice)
	{
		otg_adevice_DoIntr(dev);
	}
	else
	{
		otg_bdevice_DoIntr(dev);
	}
}

static inline void
otg_device_ClearIntr(otg_device_t *dev, otg_intr_t *intr)
{
#ifdef OTG_HNP_DEBUG
	if (otg_intr_IsTriggered(intr, otg_intr_BSRPSuccess_c))
	{
		info("ClearIntr: otg_intr_BSRPSuccess_c");
	}
	if (otg_intr_IsTriggered(intr, otg_intr_SlaveState_c))
	{
		info("ClearIntr: otg_intr_SlaveState_c");
	}
	if (otg_intr_IsTriggered(intr, otg_intr_MasterState_c))
	{
		info("ClearIntr: otg_intr_MasterState_c");
	}
	if (otg_intr_IsTriggered(intr, otg_intr_VBusError_c))
	{
		info("ClearIntr: otg_intr_VBusError_c");
	}
	if (otg_intr_IsTriggered(intr, otg_intr_VBusSRPDetect_c))
	{
		info("ClearIntr: otg_intr_VBusSRPDetect_c");
	}
	if (otg_intr_IsTriggered(intr, otg_intr_DPSRPDetect_c))
	{
		info("ClearIntr: DPSRPDetect_c");
	}
	if (otg_intr_IsTriggered(intr, otg_intr_DMSRPDetect_c))
	{
		info("ClearIntr: otg_intr_DMSRPDetect_c");
	}
	if (otg_intr_IsTriggered(intr, otg_intr_BSRPDone_c))
	{
		info("ClearIntr: otg_intr_BSRPDone_c");
	}
	if (otg_intr_IsTriggered(intr, otg_intr_MnpDone_c))
	{
		info("ClearIntr: otg_intr_MnpDone_c");
	}
	if (otg_intr_IsTriggered(intr, otg_intr_IdIsA_c))
	{
		info("ClearIntr: otg_intr_IdIsA_c");
	}
#endif
	if (otg_intr_IsTriggered(intr, otg_intr_BSRPSuccess_c))
	{
		otg_intr_ClearIntr(intr, otg_intr_BSRPSuccess_c);
	}
	if (otg_intr_IsTriggered(intr, otg_intr_SlaveState_c))
	{
		otg_intr_ClearIntr(intr, otg_intr_SlaveState_c);
	}
	if (otg_intr_IsTriggered(intr, otg_intr_MasterState_c))
	{
		otg_intr_ClearIntr(intr, otg_intr_MasterState_c);
	}
	if (otg_intr_IsTriggered(intr, otg_intr_VBusError_c))
	{
		otg_intr_ClearIntr(intr, otg_intr_VBusError_c);
	}
	if (otg_intr_IsTriggered(intr, otg_intr_VBusSRPDetect_c))
	{
		otg_intr_ClearIntr(intr, otg_intr_VBusSRPDetect_c);
	}
	if (otg_intr_IsTriggered(intr, otg_intr_DPSRPDetect_c))
	{
		otg_intr_ClearIntr(intr, otg_intr_DPSRPDetect_c);
	}
	if (otg_intr_IsTriggered(intr, otg_intr_DMSRPDetect_c))
	{
		otg_intr_ClearIntr(intr, otg_intr_DMSRPDetect_c);
	}
	if (otg_intr_IsTriggered(intr, otg_intr_BSRPDone_c))
	{
		otg_intr_ClearIntr(intr, otg_intr_BSRPDone_c);
	}
	if (otg_intr_IsTriggered(intr, otg_intr_MnpDone_c))
	{
		otg_intr_ClearIntr(intr, otg_intr_MnpDone_c);
	}
	if (otg_intr_IsTriggered(intr, otg_intr_IdIsA_c))
	{
		otg_intr_ClearIntr(intr, otg_intr_IdIsA_c);
	}
}

static inline void
otg_device_Start(otg_device_t *dev)
{
	if (dev->isADevice)
	{
		if (dev->state == otg_adevice_StateHost)
		{
			otg_device_DisableBus(dev);
		}
		else if (dev->state == otg_adevice_StateDevice)
		{
			otg_fc_Disable(otg_GetFunction(dev->otg));
		}

		otg_bdevice_ToADevice(dev);
		otg_adevice_ToADevice(dev);
	}
	else
	{
		if (dev->state == otg_bdevice_StateHost)
		{
			otg_device_DisableBus(dev);
		}
		else if (dev->state == otg_bdevice_StateDevice)
		{
			otg_fc_Disable(otg_GetFunction(dev->otg));
		}

		otg_adevice_ToBDevice(dev);
		otg_bdevice_ToBDevice(dev);
	}
}

static inline void
otg_device_DoSofIntr(otg_device_t *dev)
{
	otg_bdevice_DoSofIntr(dev);
}

#endif /* #ifndef OTG_DEVICE_H */
#endif
#endif
