/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_device_desc.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    Device description object handler.
 */
#ifndef OTG_DEVICE_DESC_H
#define OTG_DEVICE_DESC_H

static inline void
otg_device_desc_Init(otg_device_desc_t *dev)
{
	dev->length = 0;
}

static inline int
otg_device_desc_GetNumConfigurations(otg_device_desc_t *dev)
{
	return dev->desc[otg_device_desc_bNumConfigurations];
}

static inline int
otg_device_desc_IsSet(otg_device_desc_t *dev)
{
	return dev->length > 0;
}

static inline int
otg_device_desc_GetLength(otg_device_desc_t *dev)
{
	return dev->length;
}

static inline UINT8 *
otg_device_desc_GetDescPtr(otg_device_desc_t *dev)
{
	return dev->desc;
}

static inline int
otg_device_desc_SetDesc(otg_device_desc_t *dev, UINT8 *desc, int len)
{
	UINT8 b;

	if (len > otg_device_desc_Max)
	{
		err("SetDesc: descriptor size %d too large", len);
		return -1;
	}

	b = desc[otg_device_desc_bDescriptorType];
	if (otg_device_desc_Type != b)
	{
		err("SetDesc: wrong descriptor type %d", b);
		return -1;
	}

	b = desc[otg_device_desc_bNumConfigurations];
	if (b > otg_fc_MaxConfig)
	{
		err("SetDesc: trying to setup too many configurations %d", b);
		return -1;
	}

	memcpy(dev->desc, desc, len);

	/* Right now, OTG chip only support 8 bytes packet for ep 0 */
	desc[otg_device_desc_bMaxPacketSize0] = 8;
	desc[otg_device_desc_bLength] = len;

	dev->length = len;

	return 0;
}

#endif /* #ifndef OTG_DEVICE_DESC_H */
