/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_device_t.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    Both OTG device data and macro definitions.
 */
#ifndef OTG_DEVICE_T_H
#define OTG_DEVICE_T_H

#define otg_device_HNPStatus_c	0x04 /* HNP control and status */
#define otg_device_HNPTimer1_c	0x08 /* timer 1 */
#define otg_device_HNPTimer2_c	0x0c /* timer2 */

#define otg_device_HNPStatusIDCheck			0x00000001
#define otg_device_HNPStatusBusRequest		0x00000002
#define otg_device_HNPStatusBusDrop			0x00000004
#define otg_device_HNPStatusClearError		0x00000008
#define otg_device_HNPStatusState			0x000000F0
#define otg_device_HNPStatusSwDmPullDown	0x00000100
#define otg_device_HNPStatusSwDpPullDown	0x00000200
#define otg_device_HNPStatusSwPullUp		0x00000400
#define otg_device_HNPStatusBReset			0x00000800
#define otg_device_HNPStatusIsADevice		0x00010000
#define otg_device_HNPStatusIsBDevice		0x00020000
#define otg_device_HNPStatusComparator		0x00040000
#define otg_device_HNPStatusBandgap			0x00080000
#define otg_device_HNPStatusMaster			0x00100000
#define otg_device_HNPStatusSlave			0x00200000
#define otg_device_HNPStatusBHNPSupport		0x00400000
#define otg_device_HNPStatusBHNPEnable		0x00800000
#define otg_device_HNPStatusAvBHNPEnable	0x01000000
#define otg_device_HNPStatusRemoteSOF		0x02000000
#define otg_device_HNPStatusSRPDetected		0x04000000
#define otg_device_HNPStatusVBusValid		0x08000000
#define otg_device_HNPStatusSessionValid	0x10000000
#define otg_device_HNPStatusBSessionEnd		0x20000000

#include "otg_adevice_t.h"
#include "otg_bdevice_t.h"

/* Both A and B device has:
 *	isADevice - if the device is an A device.
 *		Changed by IDisA interrupt, isADev status bit.
 *	isBusRequested - if application issued a bus request.
 *		Changed by calls made by application.
 *
 * A-Device states:
 *	AIdle - When A-Device is working on anything.
 *	AHost - When A-Device is a host.
 *	ADevice - When A-Device is a function.
 *
 * B-Device states:
 *	BIdle - When B-Device is doing nothing or waiting to become either master
 *			or slave.
 *	BHost - When B-Device is a host.
 *	BDevice - When B-Device is a function.
 */

struct otg;

struct otg_device
{
	UINT32 status;

	otg_bdevice_t bdevice;

	int isADevice;
	int isBusRequested;

	int state;

	struct otg *otg;
	struct otg_hc_rh_port *port;
};

typedef struct otg_device otg_device_t;

#endif /* #ifndef OTG_DEVICE_T_H */
