/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_dma_t.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    OTG243 DMA object data and macro definitions.
 */
#ifndef OTG_DMA_T_H
#define OTG_DMA_T_H

/* DMA configuration (Channel 0 - Address: 0x14, 0x1C) */

#define otg_dma_EN_c            0x00008000
#define otg_dma_AutoLD_c        0x00004000
#define otg_dma_BurstLength_c   0x00003FE0
#define otg_dma_ShortP_c        0x00000200
#define otg_dma_CounterEOT_c    0x00000100
#define otg_dma_ZP_EOP_c        0x00000080
#define otg_dma_EpIndex_c       0x0000001E
#define otg_dma_HF_c            0x00000001

#define otg_dma_ENLoc_c          15
#define otg_dma_AutoLDLoc_c      14
#define otg_dma_BurstLengthLoc_c 5
#define otg_dma_EpIndexLoc_c     1
#define otg_dma_HFLoc_c          0

/* constants for the fields in dam channel configuration register */
#define otg_dma_Ch0BurstLen_c  8
#define otg_dma_Ch1BurstLen_c  4
#define otg_dma_AutoLoad_c     1
#define otg_dma_NoAutoLoad_c   0
#define otg_dma_Hc_c           0
#define otg_dma_Fc_c           1
#define otg_dma_maxBurstLen_c  512

/* macros to manipulate fields in dma channel configuration register */
#define otg_dma_GetENField(y)((y << otg_dma_ENLoc_c) & otg_dma_EN_c)
#define otg_dma_GetAutoLoadField(y)((y << otg_dma_AutoLDLoc_c) & otg_dma_AutoLD_c)
#define otg_dma_GetBurstLengthField(y) \
        ((y << otg_dma_BurstLengthLoc_c) & otg_dma_BurstLength_c)
#define otg_dma_GetEpIndexField(y) \
        ((y << otg_dma_EpIndexLoc_c) & otg_dma_EpIndex_c)
#define otg_dma_GetHFField(y)((y << otg_dma_HFLoc_c) & otg_dma_HF_c)
#define otg_dma_SetEN(x,y)(x | otg_dma_GetENField(y))
#define otg_dma_SetAutoLoad(x,y)(x | otg_dma_GetAutoLoadField(y))
#define otg_dma_SetBurstLength(x,y)(x | otg_dma_GetBurstLengthField(y))
#define otg_dma_SetEpIndex(x,y)(x | otg_dma_GetEpIndexField(y))
#define otg_dma_SetHF(x,y)(x | otg_dma_GetHFField(y))

/* data modes */
#define otg_dma_PIO_c              0
#define otg_dma_Channel0_c         1
#define otg_dma_Channel1_c         2

/* DMA channels ETD assignment (Address: 0x18, 0x20) */

#define otg_dma_ETD_W_c            0x0000FFFF

#define otg_dma_Config0_c	0x14	/* dma channel 0 configuration */
#define otg_dma_Etd0_c		0x18	/* dma channel 0 etd assignment */
#define otg_dma_Config1_c	0x1c	/* dma channel 1 configuration */
#define otg_dma_Etd1_c		0x20	/* dma channel 0 etd assignment */
#define otg_dma_DataPort0_c	0x24	/* dma channel 0 data port */
#define otg_dma_DataPort1_c	0x28	/* dma channel 1 data port */

#include "uhcTypes.h"

struct otg;

struct otg_dma
{
	UINT32 config;
	UINT32 etd;

	int channel; /* 0,1 for channel */
	struct otg *otg;
};

typedef struct otg_dma otg_dma_t;

#endif /* #ifndef OTG_DMA_T_H */
