/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_ep.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    Function side endpoint object function definitions.
 */
#ifndef OTG_EP_H
#define OTG_EP_H

#include "uhcTypes.h"
#include "otg_fc_t.h"

void otg_ep_Init(otg_ep_t *ep, otg_fc_t *fc);
void otg_ep_Reset(otg_ep_t *ep);

void otg_ep_DoIntr(otg_ep_t *ep, otg_ep_intr_t *intr, UINT32 filled);
void otg_ep_DoTCIntr(otg_ep_t *ep);
void otg_ep_Release(otg_ep_t *ep);
int otg_ep_Set(otg_ep_t *ep, otg_ep_desc_t *ep_desc);
int otg_ep_Submit(otg_ep_t *ep, otg_urb_t *urb);
int otg_ep_FillConfig(otg_ep_t *ep);
int otg_ep_FillCtrl(otg_ep_t *ep);
void otg_ep_SendData(otg_ep_t *ep);
void otg_ep_RecvData(otg_ep_t *ep, int size, int is_last_pkg);

static inline void
otg_ep_SetHalted(otg_ep_t *ep, int set)
{
	UINT32 w;

	ep->halted = set;
	w = otg_fc_ReadReg(ep->fc, otg_ep_Config(ep->index));
	if (set)
	{
		w |= otg_ep_ConfigStall;
	}
	else
	{
		w &= ~otg_ep_ConfigStall;
	}

	otg_fc_WriteReg(ep->fc, otg_ep_Config(ep->index), w);
}

static inline void
otg_ep_Enable(otg_ep_t *ep)
{
	UINT32 w;

	w = otg_fc_ReadReg(ep->fc, otg_ep_Config(ep->index));
	w |= otg_ep_ConfigEnable;

	otg_fc_WriteReg(ep->fc, otg_ep_Config(ep->index), w);
}


static inline void
otg_ep_Disable(otg_ep_t *ep)
{
	UINT32 w;

	w = otg_fc_ReadReg(ep->fc, otg_ep_Config(ep->index));
	w &= ~otg_ep_ConfigEnable;

	otg_fc_WriteReg(ep->fc, otg_ep_Config(ep->index), w);
}

static inline int
otg_ep_IsEpMatch(otg_ep_t *ep, int ep_num)
{
	if (ep->desc && otg_ep_desc_IsEndpoint(ep->desc, ep_num))
	{
		return 1;
	}

	return 0;
}

static inline int
otg_ep_IsDirectionIn(otg_ep_t *ep)
{
	return otg_ep_desc_IsDirectionIn(ep->desc);
}

static inline int
otg_ep_GetMaxPacketSize(otg_ep_t *ep)
{
	return otg_ep_desc_GetMaxPacketSize(ep->desc);
}

static inline int
otg_ep_IsFixedSize(otg_ep_t *ep)
{
	return !otg_ep_desc_IsIso1(ep->desc);
}

#endif /* #ifndef OTG_EP_H */
