/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_ep0.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    Function side endpoint 0 object function definitions.
 */
#ifndef OTG_EP0_H
#define OTG_EP0_H

#include "uhcTypes.h"
#include "otg_fc_t.h"

void otg_ep0_Init(otg_ep0_t *ep0, otg_fc_t *fc);
void otg_ep0_Reset(otg_ep0_t *ep0);

void otg_ep0_CallStandardSetup(otg_ep0_t *ep0);

void otg_ep0_DoSetupOverWritten(otg_ep0_t *ep0);
void otg_ep0_DoSetup(otg_ep0_t *ep0);
void otg_ep0_DoSetupDone(otg_ep0_t *ep0);
void otg_ep0_DoOut(otg_ep0_t *ep0);
void otg_ep0_DoIn(otg_ep0_t *ep0);
void otg_ep0_SendData(otg_ep0_t *ep0);
void otg_ep0_RecvData(otg_ep0_t *ep0, int bytes);
void otg_ep0_Feature(otg_ep0_t *ep0, int set);
void otg_ep0_ClassRequest(otg_ep0_t *ep0);
void otg_ep0_ClassRequestDone(otg_ep0_t *ep0);

static inline int
otg_ep0_IsStageIn(otg_ep0_t *ep0)
{
	return ep0->stage == otg_ep0_StageIn;
}

static inline int
otg_ep0_GetChipBufferSize(void)
{
	return otg_ep0_BufferSize * 3; /* Setup, In, Out */
}

static inline void
otg_ep0_SendAck(otg_ep0_t *ep0)
{
	otg_fc_WriteReg(ep0->fc, otg_ep0_Ctrl, otg_ep0_CtrlAck);
}

static inline void
otg_ep0_SendStall(otg_ep0_t *ep0)
{
	otg_fc_WriteReg(ep0->fc, otg_ep0_Ctrl, otg_ep0_CtrlStall);
}

static inline void
otg_ep0_EndOut(otg_ep0_t *ep0)
{
	ep0->stage = otg_ep0_StageOut;
	otg_fc_WriteReg(ep0->fc, otg_ep0_Size, 0);
	otg_fc_WriteReg(ep0->fc, otg_ep0_Ctrl, otg_ep0_CtrlIn);
}

static inline void
otg_ep0_Cleanup(otg_ep0_t *ep0)
{
	ep0->standardSetup = 0;
	ep0->classSetup = 0;
	ep0->vendorSetup = 0;
}

static inline void
otg_ep0_GetStatus(otg_ep0_t *ep0)
{
	UINT8 b;

	switch (otg_fc_GetState(ep0->fc))
	{
		case otg_fc_StateDefault:
			err("GetStatus: in Default state, stall");
			otg_ep0_SendStall(ep0);
			return;
		case otg_fc_StateAddress:
			if (ep0->wIndex != 0)
			{
				err("GetStatus: address not zero, stall");
				otg_ep0_SendStall(ep0);
				return;
			}
			break;
		case otg_fc_StateConfigured:
			break;
	}

	b = 0;
	if (otg_ep0_IsDeviceRequest(ep0->bRequestType))
	{
		if (ep0->remoteWakeupEnable)
		{
			b = 2;
		}

		if (ep0->selfPowered)
		{
			b |= 1;
		}
	}
	else if (otg_ep0_IsEndpointRequest(ep0->bRequestType))
	{
		if (!otg_fc_IsValidEndpoint(ep0->fc, ep0->wIndex))
		{
			err("GetStatus: invalid ep %d, stall", ep0->wIndex);
			otg_ep0_SendStall(ep0);
			return;
		}

		if (ep0->halted)
		{
			b = 1;
		}
	}
	else if (otg_ep0_IsInterfaceRequest(ep0->bRequestType))
	{
		if (!otg_fc_IsValidInterface(ep0->fc, ep0->wIndex))
		{
			err("GetStatus: invalid interface %d, stall", ep0->wIndex);
			otg_ep0_SendStall(ep0);
			return;
		}
	}
	else
	{
		err("GetStatus: bad request %d, stall", ep0->bRequestType);
		otg_ep0_SendStall(ep0);
		return; /* no response */
	}


	ep0->dataBuffer[0] = b;
	ep0->dataBuffer[1] = 0;
	ep0->dataLen = otg_ep0_BufferSize;
	ep0->dataPos = 0;
	ep0->dataPtr = ep0->dataBuffer;

	otg_ep0_SendData(ep0);
	ep0->stage = otg_ep0_StageIn;
}

static inline void
otg_ep0_GetDescriptor(otg_ep0_t *ep0)
{
	UINT8 device;
	UINT8 index;

	device = (ep0->wValue >> 8) & 0xFF;
	index = ep0->wValue & 0xFF;

	ep0->dataPos = 0;
	if (!otg_ep0_IsDeviceRequest(ep0->bRequestType))
	{
		otg_ep0_ClassRequest(ep0);
		return;
	}
	else
	{
		switch (device)
		{
			case otg_device_desc_Type:
				ep0->dataPtr = otg_fc_GetDeviceDesc(ep0->fc);
				ep0->dataLen = otg_fc_GetDeviceDescLength(ep0->fc);
				break;
			case otg_config_desc_Type:
				ep0->dataPtr = otg_fc_GetConfigDesc(ep0->fc, index);
				if (!ep0->dataPtr)
				{
					err("GetDescriptor: unknown config %d", index);
					otg_ep0_SendStall(ep0);
					return;
				}

				ep0->dataLen = otg_fc_GetConfigDescLength(ep0->fc, index);
				break;
			case otg_str_desc_Type:
				ep0->dataPtr = otg_fc_GetStrDesc(ep0->fc, index);
				if (!ep0->dataPtr)
				{
					err("GetDescriptor: bad index %d", index);
					otg_ep0_SendStall(ep0);
					return;
				}
				ep0->dataLen = otg_fc_GetStrDescLength(ep0->fc, index);
				break;
			default:
				otg_ep0_CallStandardSetup(ep0);
				ep0->dataLen = ep0->wLength;
				if (!ep0->wLength)
				{
					err("GetDescriptor: unknown type %d", device);
					otg_ep0_SendStall(ep0);
					return;
				}
		}
	}

	otg_ep0_SendData(ep0);
	ep0->stage = otg_ep0_StageIn;
}

static inline void
otg_ep0_SetDescriptor(otg_ep0_t *ep0)
{
	UINT8 device;
	UINT8 index;
	int rc;

	switch (otg_fc_GetState(ep0->fc))
	{
		case otg_fc_StateDefault:
			err("SetDescriptor: in Default state, stall");
			otg_ep0_SendStall(ep0);
			return; /* No response */
		case otg_fc_StateAddress:
		case otg_fc_StateConfigured:
			break;
	}

	device = (ep0->wValue >> 8) & 0xFF;
	index = ep0->wValue & 0xFF;

	if (!otg_ep0_IsDeviceRequest(ep0->bRequestType))
	{
		otg_ep0_ClassRequestDone(ep0);
		return;
	}

	switch (device)
	{
		case otg_device_desc_Type:
			rc = otg_fc_SetDeviceDesc(ep0->fc, ep0->data, ep0->wLength);
			break;
		case otg_config_desc_Type:
			rc = otg_fc_SetConfigDesc(ep0->fc, ep0->data, ep0->wLength);
			break;
		case otg_str_desc_Type:
			rc = otg_fc_SetStrDesc(ep0->fc, ep0->data, ep0->wLength);
			break;
		default:
			err("GetDescriptor: unknown type %d", device);
			otg_ep0_SendStall(ep0);
			ep0->dataPtr = 0; /* Let Out process know stalled */
			return;
	}

	if (rc)
	{
		otg_ep0_SendStall(ep0);
		return;
	}

	otg_fc_WriteReg(ep0->fc, otg_ep0_Size, 0);
	otg_fc_WriteReg(ep0->fc, otg_ep0_Ctrl, otg_ep0_CtrlIn);
#ifdef DEBUG_OTG_CHIP
info("SetDescriptor: dataPos=%d wLength=%d", ep0->dataPos, ep0->wLength);
#endif
}

static inline void
otg_ep0_SetAddress(otg_ep0_t *ep0)
{
	if (ep0->wValue > 127)
	{
		return;
	}

	switch (otg_fc_GetState(ep0->fc))
	{
		case otg_fc_StateDefault:
			if (ep0->wValue)
			{
				otg_fc_ChangeState(ep0->fc, otg_fc_StateAddress);
				otg_fc_WriteReg(ep0->fc, otg_ep0_DeviceAddr, ep0->wValue);
			}
			break;
		case otg_fc_StateAddress:
			if (!ep0->wValue)
			{
				otg_fc_ChangeState(ep0->fc, otg_fc_StateDefault);
			}

			otg_fc_WriteReg(ep0->fc, otg_ep0_DeviceAddr, ep0->wValue);
			break;
		case otg_fc_StateConfigured:
			err("SetAddress: in Configured state, stall");
			otg_ep0_SendStall(ep0);
			return; /* stall */
	}

	otg_ep0_EndOut(ep0);
}

static inline void
otg_ep0_GetConfiguration(otg_ep0_t *ep0)
{
	UINT8 b;
	otg_config_desc_t *config;

	b = 0;
	switch (otg_fc_GetState(ep0->fc))
	{
		case otg_fc_StateDefault:
			err("Configuration: in Default state, stall");
			otg_ep0_SendStall(ep0);
			return; /* No response */
		case otg_fc_StateAddress:
			break;
		case otg_fc_StateConfigured:
			config = otg_fc_GetConfig(ep0->fc);
			b = otg_config_desc_GetConfiguration(config);
			break;
	}

	ep0->dataBuffer[0] = b;
	ep0->dataLen = otg_ep0_BufferSize;
	ep0->dataPos = 0;
	ep0->dataPtr = ep0->dataBuffer;

	otg_ep0_SendData(ep0);
	ep0->stage = otg_ep0_StageIn;
}

static inline void
otg_ep0_SetConfiguration(otg_ep0_t *ep0)
{
	UINT8 val;

	val = ep0->wValue & 0xFF;
	switch (otg_fc_GetState(ep0->fc))
	{
		case otg_fc_StateDefault:
			err("Configuration: in Default state, stall");
			otg_ep0_SendStall(ep0);
			return; /* stall */
		case otg_fc_StateAddress:
			if (val)
			{
				if (otg_fc_SetConfig(ep0->fc, val))
				{
					otg_ep0_SendStall(ep0);
					return;
				}

				otg_fc_SetState(ep0->fc, otg_fc_StateConfigured);
			}

			break;
		case otg_fc_StateConfigured:
			if (!val)
			{
				otg_fc_SetConfig(ep0->fc, 0);
				otg_fc_SetState(ep0->fc, otg_fc_StateAddress);
			}
			else
			{
				if (otg_fc_SetConfig(ep0->fc, val))
				{
					otg_ep0_SendStall(ep0);
					return;
				}
			}

			break;
	}

	otg_ep0_CallStandardSetup(ep0);

	otg_ep0_EndOut(ep0);
}

static inline void
otg_ep0_GetInterface(otg_ep0_t *ep0)
{
	UINT8 b;
	otg_config_desc_t *config;
	otg_interface_desc_t *intf;

	b = 0;
	switch (otg_fc_GetState(ep0->fc))
	{
		case otg_fc_StateDefault:
			err("GetInterface: in Default state, stall");
			otg_ep0_SendStall(ep0);
			return; /* stall */
		case otg_fc_StateAddress:
			err("GetInterface: in Address state, stall");
			otg_ep0_SendStall(ep0);
			return; /* stall */
		case otg_fc_StateConfigured:
			config = otg_fc_GetConfig(ep0->fc);
			intf = otg_config_desc_GetInterface(config, ep0->wIndex);
			if (!intf)
			{
				err("GetInterface: bad interface %d, stall", ep0->wIndex);
				otg_ep0_SendStall(ep0);
				return; /* stall */
			}

			b = otg_interface_desc_GetAltSetting(intf);
			break;
	}

	ep0->dataBuffer[0] = b;
	ep0->dataLen = otg_ep0_BufferSize;
	ep0->dataPos = 0;
	ep0->dataPtr = ep0->dataBuffer;

	otg_ep0_SendData(ep0);
	ep0->stage = otg_ep0_StageIn;
}

static inline void
otg_ep0_SetInterface(otg_ep0_t *ep0)
{
	UINT8 val;
	otg_config_desc_t *config;
	otg_interface_desc_t *intf;

	val = ep0->wValue & 0xFF;
	switch (otg_fc_GetState(ep0->fc))
	{
		case otg_fc_StateDefault:
			err("SetInterface: in Default state, stall");
			otg_ep0_SendStall(ep0);
			return;
		case otg_fc_StateAddress:
			err("SetInterface: in Address state, stall");
			otg_ep0_SendStall(ep0);
			return;
		case otg_fc_StateConfigured:
			config = otg_fc_GetConfig(ep0->fc);
			intf = otg_config_desc_GetInterface(config, ep0->wIndex);
			if (!intf)
			{
				err("SetInterface: bad interface %d, stall", ep0->wIndex);
				otg_ep0_SendStall(ep0);
				return;
			}

			otg_interface_desc_SetAltSetting(intf, val);
			break;
	}

	otg_ep0_SendAck(ep0);
}

static inline void
otg_ep0_SyncFrame(otg_ep0_t *ep0)
{
	switch (otg_fc_GetState(ep0->fc))
	{
		case otg_fc_StateDefault:
			err("SyncFrame: in Default state, stall");
			otg_ep0_SendStall(ep0);
			return;
		case otg_fc_StateAddress:
			err("SyncFrame: in Address state, stall");
			otg_ep0_SendStall(ep0);
			return;
		case otg_fc_StateConfigured:
			break;
	}

	/* Need code to sync frame */
	ep0->dataBuffer[0] = 0;
	ep0->dataBuffer[1] = 0;
	ep0->dataLen = otg_ep0_BufferSize;
	ep0->dataPos = 0;
	ep0->dataPtr = ep0->dataBuffer;

	otg_ep0_SendData(ep0);
	ep0->stage = otg_ep0_StageIn;
}

static inline int
otg_ep0_SetCallBack
(
	otg_ep0_t *ep0,
	otg_device_request_CallBack_t standardRequest,
	otg_device_request_CallBack_t classRequest,
	otg_device_request_CallBack_t vendorRequest
)
{
	ep0->standardSetup = standardRequest;
	ep0->classSetup = classRequest;
	ep0->vendorSetup = vendorRequest;
	return 0;
}

static inline int
otg_ep0_GetAHnpSupport(otg_ep0_t *ep0)
{
	return ep0->aHnpSupport;
}

static inline int
otg_ep0_GetAAltHnpSupport(otg_ep0_t *ep0)
{
	return ep0->aAltHnpSupport;
}

#endif /* #ifndef OTG_EP0_H */
