/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_ep0_t.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    Function side endpoint 0 object data and macro definitions.
 */
#ifndef OTG_EP0_T
#define OTG_EP0_T

#define otg_ep0_DeviceAddr	0x00
#define otg_ep0_Ctrl		0x08
#define otg_ep0_Size		0x0C

#define otg_ep0_CtrlStall	0x00000010
#define otg_ep0_CtrlAck		0x00000020
#define otg_ep0_CtrlIn		0x00010000
#define otg_ep0_CtrlOut		0x00020000
#define otg_ep0_CtrlSetup	0x00040000

#define otg_ep0_SizeOut		0x0000000F
#define otg_ep0_SizeIn		0x000003C0
#define otg_ep0_SizeInLoc	6

#define otg_ep0_BufferSize	0x08
#define otg_ep0_SetupBuffer	0x00
#define otg_ep0_OutBuffer	(otg_ep0_SetupBuffer + otg_ep0_BufferSize)
#define otg_ep0_InBuffer	(otg_ep0_SetupBuffer + otg_ep0_BufferSize * 2)

#define otg_ep0_StageIdle	0
#define otg_ep0_StageIn 	1
#define otg_ep0_StageOut	2

#define otg_ep0_bRequestType	0
#define otg_ep0_bRequest		1
#define otg_ep0_wValue			2
#define otg_ep0_wIndex			4
#define otg_ep0_wLength			6

#define otg_ep0_RequestGetStatus		0
#define otg_ep0_RequestClearFeature		1
#define otg_ep0_RequestSetFeature		3
#define otg_ep0_RequestSetAddress		5
#define otg_ep0_RequestGetDescriptor	6
#define otg_ep0_RequestSetDescriptor	7
#define otg_ep0_RequestGetConfiguration	8
#define otg_ep0_RequestSetConfiguration	9
#define otg_ep0_RequestGetInterface		10
#define otg_ep0_RequestSetInterface		11
#define otg_ep0_RequestSyncFrame		12

#define otg_ep0_IsStandardRequest(type)	((type & 0x60) == 0x00)
#define otg_ep0_IsClassRequest(type)	((type & 0x60) == 0x20)
#define otg_ep0_IsVendorRequest(type)	((type & 0x60) == 0x40)
#define otg_ep0_IsDeviceRequest(type)	((type & 0x1F) == 0x00)
#define otg_ep0_IsInterfaceRequest(type)	((type & 0x1F) == 0x01)
#define otg_ep0_IsEndpointRequest(type)	((type & 0x1F) == 0x02)
#define otg_ep0_IsOtherRequest(type)	((type & 0x1F) == 0x03)
#define otg_ep0_IsDeviceToHostRequest(type)	(type & 0x80)

#define otg_ep0_DataMax	1024

#define otg_ep0_FEATURE_ENDPOINT_HALT		0
#define otg_ep0_FEATURE_REMOTE_WAKEUP		1
#define otg_ep0_FEATURE_B_HNP_ENABLE		3
#define otg_ep0_FEATURE_A_HNP_SUPPORT		4
#define otg_ep0_FEATURE_A_ALT_HNP_SUPPORT	5

#include "otg_device_request_t.h"

struct otg_fc;

struct otg_ep0
{
	UINT8 stage;
	UINT8 dataBuffer[otg_ep0_BufferSize];
	UINT8 data[otg_ep0_DataMax];
	int dataLen; /* Length of the buffer pointed to by dataPtr */
	int dataPos; /* number of bytes already sent */
	void *dataPtr;

	int setupMem;
	int outMem;
	int inMem;

	UINT8 bRequestType;
	UINT8 bRequest;
	UINT16 wValue;
	UINT16 wIndex;
	UINT16 wLength; /* Total data length */

	UINT8 remoteWakeupEnable;
	UINT8 selfPowered;
	UINT8 halted;
	UINT8 bHnpEnable;
	UINT8 aHnpSupport;
	UINT8 aAltHnpSupport;

	void *privateData;
	otg_device_request_CallBack_t standardSetup;
	otg_device_request_CallBack_t classSetup;
	otg_device_request_CallBack_t vendorSetup;

	struct otg_fc *fc;
};

typedef struct otg_ep0 otg_ep0_t;

#endif /* #ifndef OTG_EP0_T */
