/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_ep_desc.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    Endpoint descriptor object handler.
 */
#ifndef OTG_EP_DESC_H
#define OTG_EP_DESC_H

static inline int
otg_ep_desc_IsEndpoint(otg_ep_desc_t *ep, int ep_num)
{
	return ep->desc[otg_ep_desc_bEndpointAddress] == ep_num;
}

static inline int
otg_ep_desc_IsEndpointDesc(UINT8 *ep)
{
	return ep[otg_ep_desc_bDescriptorType] == otg_ep_desc_Type;
}

static inline int
otg_ep_desc_GetMaxPacketSize(otg_ep_desc_t *ep)
{
	int size;

	size = ep->desc[otg_ep_desc_wMaxPacketSize];
	size |= ep->desc[otg_ep_desc_wMaxPacketSize + 1] << 8;
	return size;
}

static inline int
otg_ep_desc_SetDesc(otg_ep_desc_t *ep, UINT8 *desc, int len)
{
	UINT8 b;

	b = desc[otg_ep_desc_bLength];
	if (b != otg_ep_desc_Max || b > len)
	{
		err("SetDesc: endpoint descriptor size %d incorrect", b);
		return -EINVAL;
	}

	b = desc[otg_ep_desc_bEndpointAddress] & 0x7F;
	if (b < 1 || b >= otg_fc_MaxEP)
	{
		err("SetDesc: endpoint address %d out of range", b);
		return -EINVAL;
	}

	b = desc[otg_ep_desc_bDescriptorType];
	if (b != otg_ep_desc_Type)
	{
		err("SetDesc: incorrect descriptor type %d", b);
		return -EINVAL;
	}

	ep->desc = desc;

	return desc[otg_ep_desc_bLength];
}

static inline UINT8
otg_ep_desc_GetAddr(otg_ep_desc_t *ep)
{
	return ep->desc[otg_ep_desc_bEndpointAddress];
}

static inline int
otg_ep_desc_IsDirectionIn(otg_ep_desc_t *ep)
{
	return ep->desc[otg_ep_desc_bEndpointAddress] & otg_ep_desc_DirectionIn;
}

static inline int
otg_ep_desc_IsIso1(otg_ep_desc_t *ep)
{
/* Do not handle iso1 for now */
/*	return ep->desc[otg_ep_desc_bmAttributes] == otg_ep_desc_TypeIso; */
	return 0;
}

static inline int
otg_ep_desc_Set(otg_ep_desc_t *ep, otg_fc_t *fc)
{
	return otg_fc_SetEp(fc, ep->desc[otg_ep_desc_bEndpointAddress], ep);
}

#endif /* #ifndef OTG_EP_DESC_H */
