/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_ep_intr.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    Endpoint interrupt object function definitions.
 */
#ifndef OTG_EP_INTR_H
#define OTG_EP_INTR_H

#include "otg_fc_t.h"

static inline void
otg_ep_intr_Init(otg_ep_intr_t *intr, otg_fc_t *fc)
{
	intr->fc = fc;
	intr->status = 0;
	intr->enabled = 0;

	otg_fc_WriteReg(intr->fc, otg_ep_intr_Enable, 0);
	otg_fc_WriteReg(intr->fc, otg_ep_intr_Clear, otg_ep_intr_All);
}

static inline void
otg_ep_intr_Reset(otg_ep_intr_t *intr)
{
	intr->status = 0;
	otg_fc_WriteReg(intr->fc, otg_ep_intr_Enable, intr->enabled);
}

static inline int
otg_ep_intr_IsTriggered(otg_ep_intr_t *intr, UINT32 no)
{
	return intr->status & no;
}

static inline int
otg_ep_intr_Check(otg_ep_intr_t *intr, UINT32 no)
{
	UINT32 w;

	w = otg_fc_ReadReg(intr->fc, otg_ep_intr_Status);
	intr->status |= w;
	otg_fc_WriteReg(intr->fc, otg_ep_intr_Clear, w);

	return otg_ep_intr_IsTriggered(intr, no);
}

static inline void
otg_ep_intr_ClearIntr(otg_ep_intr_t *intr, UINT32 no)
{
	intr->status &= ~no;
}

static inline void
otg_ep_intr_EnableIntr(otg_ep_intr_t *intr, UINT32 no)
{
#ifdef DEBUG_OTG_CHIP
	otg_fc_ReadReg(intr->fc, otg_ep_intr_Enable);
#endif
	intr->enabled |= no;
	otg_fc_WriteReg(intr->fc, otg_ep_intr_Enable, intr->enabled);
}

static inline void
otg_ep_intr_DisableIntr(otg_ep_intr_t *intr, UINT32 no)
{
#ifdef DEBUG_OTG_CHIP
	otg_fc_ReadReg(intr->fc, otg_ep_intr_Enable);
#endif
	intr->enabled &= ~no;
	otg_fc_WriteReg(intr->fc, otg_ep_intr_Enable, intr->enabled);
}

#endif /* #ifndef OTG_EP_INTR_H */
