/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_etd.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    ETD object function definitions.
 */
#ifndef OTG_ETD_H
#define OTG_ETD_H

#include "otg_etd_t.h"

int otg_etd_Fill(otg_etd_t *etd, otg_hc_t *hc, stack_urb_t *urb);
void otg_etd_SetPipe(otg_etd_t *etd, stack_urb_t *urb);
int otg_etd_DoDoneEtd(otg_etd_t *etd, otg_hc_etd_t *hc_etd);
int otg_etd_CopyHcInfo(otg_etd_t *etd, otg_etd_t *from);
void otg_etd_Delete(otg_etd_t *etd);
otg_etd_t * otg_etd_GetNext(otg_etd_t *etd);
otg_etd_t * otg_etd_GetPrev(otg_etd_t *etd);

#ifdef DEBUG_OTG_ETD
void otg_etd_PrintEtd(otg_etd_t *etd);
#endif

#ifdef DEBUG_OTG_SUBMIT
void otg_etd_PrintErrorCode(stack_urb_t *urb);
static inline void
otg_etd_PrintEtdWord(otg_etd_t *etd)
{
	info("ETD: %p w0=0x%0x w1=0x%0x w2=0x%0x w3=0x%0x",
	etd, etd->endpoint,etd->buffer,etd->td.ctrl.ctrl,etd->td.ctrl.info);
}
#endif

static inline int
otg_etd_GetBufferSize(otg_etd_t *etd)
{
	return etd->bufferSize;
}

static inline void
otg_etd_SetEtdIndex(otg_etd_t *etd, int hc_etd)
{
	etd->etdIndex = hc_etd;
}

static inline int
otg_etd_GetEtdIndex(otg_etd_t *etd)
{
	return etd->etdIndex;
}

static inline void
otg_etd_ClearEtdIndex(otg_etd_t *etd)
{
	etd->etdIndex = -1;
}

static inline int
otg_etd_IsTransferType(otg_etd_t *etd, int type)
{
	return (etd->endpoint & otg_etd_Format_c) == type;
}

static inline int
otg_etd_GetTransferType(otg_etd_t *etd)
{
	return etd->endpoint & otg_etd_Format_c;
}

static inline stack_urb_t *
otg_etd_GetUrb(otg_etd_t *etd)
{
	return etd->urb;
}

static inline UINT32
otg_etd_GetDWord(otg_etd_t *etd, int index)
{
	UINT32 *dp;

	dp = (UINT32 *)etd;
	return *(dp + index);
}

static inline int
otg_etd_IsInHc(otg_etd_t *etd)
{
	return etd->inHc;
}

static inline void
otg_etd_SetInHc(otg_etd_t *etd)
{
	etd->inHc = 1;
}

static inline void
otg_etd_ClearInHc(otg_etd_t *etd)
{
	etd->inHc = 0;
}

static inline void
otg_etd_MarkUnlink(otg_etd_t *etd)
{
	etd->unlinked = 1;
}

static inline int
otg_etd_IsUnlink(otg_etd_t *etd)
{
	return etd->unlinked;
}

static inline int
otg_etd_IsInDoneQueue(otg_etd_t *etd)
{
	return etd->inDoneQueue;
}

static inline void
otg_etd_SetInDoneQueue(otg_etd_t *etd)
{
	etd->inDoneQueue = 1;
}

static inline void
otg_etd_ClearInDoneQueue(otg_etd_t *etd)
{
	etd->inDoneQueue = 0;
}

static inline void
otg_etd_Init(otg_etd_t *etd)
{
	OS_ListInit(&etd->pipeEtdList);
	OS_ListInit(&etd->formatEtdList);
	etd->endpoint = 0;
	etd->buffer = 0;

	etd->urb = 0;

	etd->etdIndex = -1;
	etd->addrX = -1;
	etd->addrY = -1;
	etd->unlinked = 0;
	etd->inHc = 0;
	etd->inDoneQueue = 0;
#ifdef OTG_ISO_SHARE_MEMORY
	OS_ListInit(&etd->sofList);
#endif

	etd->completed = 0;
	etd->lastCompleted = 0;
	etd->bufferSize = 0;
	etd->dataLength = 0;
	etd->data = 0;
	etd->setupStatus = 0;
}

static inline otg_etd_t *
otg_etd_New(otg_hc_t *hc)
{
	otg_etd_t *etd;

	etd = OS_Malloc(sizeof(otg_etd_t));
	if (!etd)
	{
		return 0;
	}

	etd->hc = hc;

	otg_etd_Init(etd);

	return etd;
}

static inline void
otg_etd_LinkEtd(otg_etd_t *etd, otg_etd_t *prev_etd)
{
	/* add to the tail of the queue. etd_list must be the first etd */
	OS_ListAddTail(&etd->pipeEtdList, &prev_etd->pipeEtdList);
}

static inline int
otg_etd_GetBuffer(otg_etd_t *etd, int yBuffer)
{
	if (yBuffer)
	{
		return etd->addrY;
	}
	else
	{
		return etd->addrX;
	}
}

static inline int
otg_etd_HasMorePacket(otg_etd_t *etd)
{
	return etd->completed < etd->dataLength;
}

#include "otg_td_ctrl.h"
#include "otg_td_iso.h"

static inline void
otg_etd_SetEtdFromHc(otg_etd_t *etd, int index, UINT32 word)
{
	UINT32 w;

	switch (index)
	{
		case 0: /* DWord0 is the ED */
			w = otg_etd_Halted_c | otg_etd_ToggleCarry_c | otg_etd_StopOnNak_c;
			word &= w;
			etd->endpoint &= ~w;
			etd->endpoint |= word;
			break;
		case 1: /* DWord1 is the buffer address */
			etd->buffer = word;
			break;
		case 2: /* DWord2 is the packet info */
		case 3: /* DWord3 is iso buffer 0,1 */
		case 4: /* DWord2 is iso buffer 2,3 */
		case 5: /* DWord2 is iso buffer 4,5 */
		case 6: /* DWord2 is iso buffer 6,7 */
			if (otg_etd_IsTransferType(etd, otg_etd_Format_Iso_c))
			{
				otg_td_iso_SetEtdFromHc(etd, index, word);
			}
			else
			{
				otg_td_ctrl_SetEtdFromHc(etd, index, word);
			}
			break;
	}
}

static inline void
otg_etd_FillPacketWord(otg_etd_t *etd)
{
	if (otg_etd_IsTransferType(etd, otg_etd_Format_Iso_c))
	{
		otg_td_iso_FillPacketWord(etd);
	}
	else
	{
		otg_td_ctrl_FillPacketWord(etd);
	}
}

static inline int
otg_etd_HasBuffer(otg_etd_t *etd, int y)
{
	if (!y)
	{
		/* If there is no x, there is no y */
		return etd->addrX >= 0;
	}

	return etd->addrY >= 0;
}

static inline void
otg_etd_SetBufferX(otg_etd_t *etd, int addr)
{
	etd->buffer &= ~otg_etd_BufferField_c;
	etd->buffer |= addr >> 2;
	etd->addrX = addr;
}

static inline void
otg_etd_SetBufferY(otg_etd_t *etd, int addr)
{
	etd->buffer &= ~(otg_etd_BufferField_c << otg_etd_YBufferLoc_c);
	etd->buffer |= (addr >> 2) << otg_etd_YBufferLoc_c;
	etd->addrY = addr;
}


static inline void
otg_etd_ClearBuffer(otg_etd_t *etd, int y)
{
	if (y)
	{
		etd->buffer &= ~(otg_etd_BufferField_c << otg_etd_YBufferLoc_c);
		etd->addrY = -1;
	}
	else
	{
		etd->buffer &= ~otg_etd_BufferField_c;
		etd->addrX = -1;
	}
}

static inline int
otg_etd_NeedYBuffer(otg_etd_t *etd)
{
	if (otg_etd_IsTransferType(etd, otg_etd_Format_Iso_c))
	{
		return otg_td_iso_NeedYBuffer(etd);
	}
	else
	{
		return otg_td_ctrl_NeedYBuffer(etd);
	}
}

static inline int
otg_etd_NeedEtd(otg_etd_t *etd)
{
	return etd->etdIndex < 0;
}

static inline void
otg_etd_NextPacket(otg_etd_t *etd, otg_hc_etd_t *hc_etd)
{
	if (otg_etd_IsTransferType(etd, otg_etd_Format_Iso_c))
	{
		otg_td_iso_NextPacket(etd, hc_etd);
	}
	else
	{
		otg_td_ctrl_NextPacket(etd, hc_etd);
	}
}

static inline void
otg_etd_SetCtrlPipe(otg_etd_t *etd, stack_urb_t *urb)
{
	etd->endpoint = otg_etd_Format_Ctrl_c;
	otg_etd_SetPipe(etd, urb);

	/* Get direction from TD */
	etd->endpoint &= ~otg_etd_Direction_c;
}

static inline void
otg_etd_SetBulkPipe(otg_etd_t *etd, stack_urb_t *urb)
{
	etd->endpoint = otg_etd_Format_Bulk_c;
	otg_etd_SetPipe(etd, urb);
	etd->dataLength = stack_urb_GetBufferLength(urb);
	etd->data = stack_urb_GetBuffer(urb);
}

static inline void
otg_etd_SetIntrPipe(otg_etd_t *etd, stack_urb_t *urb)
{
	etd->endpoint = otg_etd_Format_Intr_c;
	otg_etd_SetPipe(etd, urb);
	etd->dataLength = stack_urb_GetBufferLength(urb);
	etd->data = stack_urb_GetBuffer(urb);
}

static inline void
otg_etd_SetIsoPipe(otg_etd_t *etd, stack_urb_t *urb)
{
	etd->endpoint = otg_etd_Format_Iso_c;
	otg_etd_SetPipe(etd, urb);
	etd->dataLength = stack_urb_GetPacketCount(urb);
	etd->data = stack_urb_GetBuffer(urb);
}

#include "otg_td_ctrl.h"
#include "otg_hc.h"

/* Return 1 if the ETD is for IN pipe.
 */
static inline int
otg_etd_IsInEtd(otg_etd_t *etd)
{
	UINT32 w;

	w = etd->endpoint & otg_etd_Direction_c;
	if (w == otg_etd_Direction_IN_c)
	{
		return 1;
	}
	else if (w == otg_etd_Direction_OUT_c)
	{
		return 0;
	}

	return otg_td_ctrl_IsInTd(etd);
}

static inline otg_etd_t *
otg_etd_GetWaiting(otg_etd_t *etd)
{
	OS_ListDeclare(*list);
	OS_ListDeclare(*self);
	otg_etd_t *tmp;
	
	self = &etd->pipeEtdList;
	for(list = OS_ListNext(self); list != self; list = OS_ListNext(list))
	{
		tmp = OS_ListEntry(list, otg_etd_t, pipeEtdList);
		if (!otg_etd_IsInHc(tmp) && tmp->etdIndex < 0)
		{
			return tmp;
		}
	}

	return 0;
}

#ifdef OTG_ISO_SHARE_MEMORY
static inline void
otg_etd_CopyEtdInfo(otg_etd_t *etd, otg_etd_t *from)
{
	etd->etdIndex = from->etdIndex;
}

static inline void
otg_etd_CopyBufferInfo(otg_etd_t *etd, otg_etd_t *from)
{
	etd->bufferSize = from->bufferSize;
	if (from->totalPackets & 1)
	{
		etd->addrX = from->addrY;
		etd->addrY = from->addrX;
		etd->buffer = (etd->addrY >> 2) << otg_etd_YBufferLoc_c
			| (etd->addrX >> 2);
	}
	else
	{
		etd->buffer = from->buffer;
		etd->addrX = from->addrX;
		etd->addrY = from->addrY;
	}
}

#ifdef OTG_ISO_SHARE_MEMORY
static inline int
otg_etd_IsMemoryUsed(otg_etd_t *etd)
{
	if (otg_etd_IsTransferType(etd, otg_etd_Format_Iso_c))
	{
		return otg_td_iso_IsMemoryUsed(etd);
	}

	return 0;
}
#endif

static inline int
otg_etd_IsSofLinked(otg_etd_t *etd)
{
	return !OS_ListEmpty(&etd->sofList);
}

#endif

#endif /* #ifndef OTG_ETD_H */
