/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_etd_t.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    ETD object data and macro definitions.
 */
#ifndef OTG_ETD_T_H
#define OTG_ETD_T_H

/* ED field definitions - dword 0, use as masks 
 ***********************************************/

#define otg_etd_FunctionAddress_c	0x0000007f /* address of the function*/
#define otg_etd_EndpointNumber_c	0x00000780 /* endpoint number */
#define otg_etd_Direction_c			0x00001800 /* direction of the data flow */
#define otg_etd_Speed_c				0x00002000 /* speed of the endpoint*/
#define otg_etd_Format_c			0x0000C000 /* format of the TD */
#define otg_etd_MaxPacketSize_c		0x07FF0000 /* max number of bytes */
#define otg_etd_Halted_c			0x08000000 /* halted bit of the ED*/
#define otg_etd_ToggleCarry_c		0x10000000
#define otg_etd_StopOnNak_c			0x40000000 /* stop on a nak if set */

/* ED field bit location - use for shifting bits 
 ************************************************/

#define otg_etd_FunctionAddressLoc_c    0     /* address of the function*/
#define otg_etd_EndpointNumberLoc_c     7     /* endpoint number */
#define otg_etd_DirectionLoc_c          11    /* direction of the data flow */
#define otg_etd_SpeedLoc_c              13    /* speed of the endpoint*/
#define otg_etd_FormatLoc_c             14    /* format of the TD */
#define otg_etd_MaxPacketSizeLoc_c      16    /* max number of bytes */
#define otg_etd_HaltedLoc_c             27    /* halted bit of the ED*/
#define otg_etd_ToggleCarryLoc_c        28
#define otg_etd_StopOnNakLoc_c          30
#define otg_etd_NewLoc_c                31    

#define otg_etd_XbufStartAddrLoc_c	0  /* dword 1 */
#define otg_etd_YbufStartAddrLoc_c	16 /* dword 1 */

/* ED Direction - it's best to 'and' ed with ED_Direction_c and compare
 * with the constant below to determine direction
 ***********************************************************************/

#define otg_etd_Direction_OUT_c	0x00000800    /* OUT*/
#define otg_etd_Direction_IN_c	0x00001000    /* IN*/

/* ED Format - it's best to 'and' ed with ED_Format_c and compare
 * with the constant below to determine format of transfer 
 *****************************************************************/

#define otg_etd_Format_Ctrl_c	0x00000000         
#define otg_etd_Format_Iso_c	0x00004000
#define otg_etd_Format_Bulk_c	0x00008000
#define otg_etd_Format_Intr_c	0x0000c000

/* endpoint speed  - it's best to 'and' ed with ED_Speed_c and compare
 * with the constant below to determine format of transfer 
 **********************************************************************/

#define otg_etd_Speed_Low_c		0x00002000	
#define otg_etd_Speed_Full_c	0x00000000

/* new Etd  - it's best to 'and' ed with ED_New_c and compare
 * with the constant below to set the ETD new bit
 *************************************************************/

#define otg_etd_SetNew_c	   0x80000000	
#define otg_etd_UnsetNew_c	   0x00000000

/* Completion Codes for the TDs 
 ******************************/

#define otg_etd_CC_NoError_c				0x00000000
#define otg_etd_CC_CRC_c					0x10000000
#define otg_etd_CC_BitStuffing_c			0x20000000
#define otg_etd_CC_DataToggleMismatch_c		0x30000000
#define otg_etd_CC_Stall_c					0x40000000
#define otg_etd_CC_DeviceNotResponding_c	0x50000000
#define otg_etd_CC_PidFailure_c				0x60000000
#define otg_etd_CC_DataOverRun_c			0x80000000
#define otg_etd_CC_DataUnderRun_c			0x90000000
#define otg_etd_CC_ACK						0xA0000000
#define otg_etd_CC_NAK						0xB0000000
#define otg_etd_CC_BufferOverRun_c			0xC0000000
#define otg_etd_CC_BufferUnderRun_c			0xD0000000
#define otg_etd_CC_NotAccessed_c			0xF0000000

#define otg_etd_BufferField_c		0xFFFF
#define otg_etd_YBufferLoc_c		16

#include "os.h"

#include "otg_td_ctrl_t.h"
#include "otg_td_iso_t.h"

struct otg_etd
{
	UINT32 endpoint;
	UINT32 buffer;

	/* Use FMT in endpoint to tell pipe type */
	union
	{
		struct otg_td_ctrl ctrl;
		struct otg_td_ctrl bulk;
		struct otg_td_ctrl intr;
		struct otg_td_iso iso;
	} td;

	int etdIndex; /* ETD on chip assigned 0 - 15. not assigned -1. */ 
	int addrX;
	int addrY;
	int unlinked;
	int inHc;
	int inDoneQueue;
#ifdef OTG_ISO_SHARE_MEMORY
	OS_ListDeclare(sofList);
#endif

	/* For iso, these are number of packets. For others, these are
	 * number of bytes.
	 */
	int completed;
	int lastCompleted;
	int bufferSize;
	int dataLength;
	int totalPackets;
	void *data;

	int setupStatus;

	OS_ListDeclare(pipeEtdList); /* ETDs of the same pipe */
	OS_ListDeclare(formatEtdList); /* ETDs of the same transfer type */

	stack_urb_t *urb;
	struct otg_hc *hc;
};

typedef struct otg_etd otg_etd_t;


#endif /* #ifndef OTG_ETD_T_H */
