/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_fc.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    OTG243 function controller function definitions.
 */
#ifndef OTG_FC_H
#define OTG_FC_H

#include "otg_fc_t.h"

otg_fc_t * otg_fc_New(void);
void otg_fc_Init(otg_fc_t *fc, otg_t *otg, int start, int size);
void otg_fc_Reset(otg_fc_t *fc);
void otg_fc_Exit(otg_fc_t *fc);
int otg_fc_Start(otg_fc_t *fc);
int otg_fc_Stop(otg_fc_t *fc);
void otg_fc_Delete(otg_fc_t *fc);

void otg_fc_ResetUnlocked(otg_fc_t *fc);
void otg_fc_StartTimer(otg_fc_t *fc);
void otg_fc_Timeout(unsigned long data);

void otg_fc_DoIntr(otg_fc_t *fc);
int otg_fc_Enable(otg_fc_t *fc);
int otg_fc_Disable(otg_fc_t *fc);

int otg_fc_SetConfig(otg_fc_t *fc, int config);
int otg_fc_Submit(otg_fc_t *fc, otg_urb_t *urb);
int otg_fc_SetEp(otg_fc_t *fc, int ep_addr, otg_ep_desc_t *desc);
int otg_fc_SetDeviceDesc(otg_fc_t *fc, UINT8 *desc, int len);
int otg_fc_SetConfigDesc(otg_fc_t *fc, UINT8 *desc, int len);
int otg_fc_SetStrDesc(otg_fc_t *fc, UINT8 *desc, int len);
int otg_fc_SetCallBack
(
	otg_fc_t *fc,
	otg_device_request_CallBack_t standardRequest,
	otg_device_request_CallBack_t classRequest,
	otg_device_request_CallBack_t vendorRequest
);
int otg_fc_GetConfiguration(otg_fc_t *fc);
int otg_fc_WaitStart(otg_fc_t *fc);

int otg_fc_RequestBus(otg_fc_t *fc);
void otg_fc_ReleaseBus(otg_fc_t *fc);
void otg_fc_BHnpEnabled(otg_fc_t *fc);

void otg_fc_Stall(otg_fc_t *fc, int ep);
void otg_fc_ClearStall(otg_fc_t *fc, int ep);

void otg_fc_Wakeup(otg_fc_t *fc);

static inline void
otg_fc_WriteReg(otg_fc_t *fc, int reg, UINT32 val)
{
	otg_WriteFunctionReg(fc->otg, reg, val);
}

static inline UINT32
otg_fc_ReadReg(otg_fc_t *fc, int reg)
{
	return otg_ReadFunctionReg(fc->otg, reg);
}

static inline void
otg_fc_WriteStart(otg_fc_t *fc, int start)
{
	otg_WriteFunctionReg(fc->otg, otg_fc_StartAddr, start);
}

static inline int
otg_fc_GetState(otg_fc_t *fc)
{
	return fc->state;
}

static inline void
otg_fc_ChangeState(otg_fc_t *fc, int state)
{
	fc->state = state;
}

static inline otg_t *
otg_fc_GetOtg(otg_fc_t *fc)
{
	return fc->otg;
}

#include "otg_urb.h"
#include "otg_device_desc.h"
#include "otg_ep_desc.h"
#include "otg_interface_desc.h"
#include "otg_config_desc.h"
#include "otg_str_desc.h"
#include "otg_fc_intr.h"
#include "otg_ep_intr.h"

static inline UINT8 *
otg_fc_GetDeviceDesc(otg_fc_t *fc)
{
	return otg_device_desc_GetDescPtr(&fc->deviceDesc);
}

static inline int
otg_fc_GetDeviceDescLength(otg_fc_t *fc)
{
	return otg_device_desc_GetLength(&fc->deviceDesc);
}

static inline UINT8 *
otg_fc_GetConfigDesc(otg_fc_t *fc, int config)
{
	if (config < 0 || config >= fc->numConfigs)
	{
		return 0;
	}

	return otg_config_desc_GetDesc(&fc->configDesc[config]);
}

static inline int
otg_fc_GetConfigDescLength(otg_fc_t *fc, int config)
{
	if (config < 0 || config >= fc->numConfigs)
	{
		return 0;
	}

	return otg_config_desc_GetLength(&fc->configDesc[config]);
}

static inline UINT8 *
otg_fc_GetStrDesc(otg_fc_t *fc, int index)
{
	return otg_str_desc_GetDesc(&fc->strDesc, index);
}

static inline int
otg_fc_GetStrDescLength(otg_fc_t *fc, int index)
{
	return otg_str_desc_GetLength(&fc->strDesc, index);
}

static inline otg_config_desc_t *
otg_fc_GetConfig(otg_fc_t *fc)
{
	return fc->selectedConfig;
}

static inline int
otg_fc_IsValidEndpoint(otg_fc_t *fc, int ep)
{
	if (!ep)
	{
		return 1;
	}

	return otg_config_desc_GetEndpoint(fc->selectedConfig, ep) != 0;
}

static inline int
otg_fc_IsValidInterface(otg_fc_t *fc, int interface)
{
	return otg_config_desc_GetInterface(fc->selectedConfig, interface) != 0;
}

static inline void
otg_fc_SetState(otg_fc_t *fc, int state)
{
	fc->state = state;
}

#include "otg_fc_intr.h"
#include "otg_ep_intr.h"
#include "otg_ep0.h"
#include "otg_ep.h"
#include "otg_device.h"

static inline otg_config_desc_t *
otg_fc_FindConfig(otg_fc_t *fc, int config)
{
	int i;

	for(i = 0; i < fc->numConfigs; i++)
	{
#ifdef DEBUG_OTG_CHIP
info("FindConfig: i=%d configDesc[i]=%p", i, &fc->configDesc[i]);
#endif
		if (config == otg_config_desc_GetConfiguration(&fc->configDesc[i]))
		{
			return &fc->configDesc[i];
		}
	}

	return 0;
}

static inline int
otg_fc_AllocateMem(otg_fc_t *fc, int size)
{
	return otg_mem_Allocate(&fc->mem, size);
}

static inline void
otg_fc_ReleaseMem(otg_fc_t *fc, int start)
{
	otg_mem_Release(&fc->mem, start);
}

static inline otg_ep_t *
otg_fc_GetEp(otg_fc_t *fc, int ep_num)
{
	otg_ep_t *ep;
	int i;

	ep = fc->ep;
	for(i = 0; i < otg_fc_MaxEP; i++)
	{
		if (otg_ep_IsEpMatch(ep, ep_num))
		{
			return ep;
		}

		ep++;
	}

	return 0;
}

static inline int
otg_fc_GetMemStart(otg_fc_t *fc)
{
	return fc->start;
}

#endif /* #ifndef OTG_FC_H */
