/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_fc_intr.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    OTG243 function interrupt object function definitions.
 */
#ifndef OTG_FC_INTR_H
#define OTG_FC_INTR_H

#include "otg_fc_t.h"

static inline void
otg_fc_intr_Init(otg_fc_intr_t *intr, otg_fc_t *fc)
{
	intr->fc = fc;
	intr->status = 0;
	intr->enabled = 0;

	otg_fc_WriteReg(intr->fc, otg_fc_intr_Enable, 0);
	otg_fc_WriteReg(intr->fc, otg_fc_intr_Clear, otg_fc_intr_All);
}

static inline void
otg_fc_intr_Reset(otg_fc_intr_t *intr)
{
	intr->status = 0;
	otg_fc_WriteReg(intr->fc, otg_fc_intr_Enable, intr->enabled);
#ifdef DEBUG_OTG_CHIP
	otg_fc_ReadReg(intr->fc, otg_fc_intr_Enable);
#endif
}

static inline int
otg_fc_intr_IsTriggered(otg_fc_intr_t *intr, UINT32 no)
{
	return intr->status & no;
}

static inline int
otg_fc_intr_Check(otg_fc_intr_t *intr, UINT32 no)
{
	UINT32 w;

	w = otg_fc_ReadReg(intr->fc, otg_fc_intr_Status);
	if (w)
	{
		intr->status |= w;
		otg_fc_WriteReg(intr->fc, otg_fc_intr_Clear, w);
	}

	return otg_fc_intr_IsTriggered(intr, no);
}

static inline void
otg_fc_intr_ClearIntr(otg_fc_intr_t *intr, UINT32 no)
{
	intr->status &= ~no;
}

static inline void
otg_fc_intr_EnableIntr(otg_fc_intr_t *intr, int val)
{
#ifdef DEBUG_OTG_CHIP
otg_fc_ReadReg(intr->fc, otg_fc_intr_Enable);
#endif
	intr->enabled |= val;
	otg_fc_WriteReg(intr->fc, otg_fc_intr_Enable, intr->enabled);
}

static inline void
otg_fc_intr_DisableIntr(otg_fc_intr_t *intr, int val)
{
#ifdef DEBUG_OTG_CHIP
otg_fc_ReadReg(intr->fc, otg_fc_intr_Enable);
#endif
	intr->enabled &= ~val;
	otg_fc_WriteReg(intr->fc, otg_fc_intr_Enable, intr->enabled);
}

#endif /* #ifndef OTG_FC_INTR_H */
