/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_function.c
 * Author: Biao Jia		1/21/02
 * Description:
 *    Function side OTG device driver.
 */
#include "os.h"
#include "stack_urb.h"
#include "otg.h"
#include "otg_fd.h"

#define	otg_function_DeviceDescSize		18
#define	otg_function_ConfigDescSize		9
#define	otg_function_InterfaceDescSize	9
#define	otg_function_OtgDescSize		3
#define otg_function_EndpointDescSize	7

static UINT8 otgDeviceDesc[] =
{
	otg_function_DeviceDescSize,
	0x01,	/* bDescriptorType, device */
	0x10,	/* bcdUSB, lower byte */
	0x01,	/* bcdUSB, high byte */
	0x00,	/* bDeviceClass */
	0x00,	/* bDeviceSubClass */
	0x00,	/* bDeviceProtocol */
	0x08,	/* bMaxPacketSize0 */
	0x48,	/* idVendor, TDI */
	0x28,	/* idVendor */
	0x43,	/* idProduct */
	0x02,	/* idProduct */
	0x01,	/* bcdDevice */
	0x00,	/* bcdDevice */
	0x00,	/* iManufacturer */
	0x00,	/* iProduct */
	0x00,	/* iSerialNumber */
	0x01	/* bNumConfigurations */
};

static UINT8 otgConfigDesc[] =
{
	/* Configuration descriptior */
	otg_function_ConfigDescSize,
	0x02,	/* bDescriptorType, configuration */
	otg_function_ConfigDescSize + otg_function_InterfaceDescSize +
	otg_function_OtgDescSize + otg_function_EndpointDescSize,
	0x00,
	0x01,	/* bNumInterfaces */
	0x01,	/* bConfigurationValue */
	0x00,	/* iConfiguration */
	0xA0,	/* Bus powered, Remote Wakeup */
	0x00,	/* MaxPower */

	/* Interface Descriptor */
	otg_function_InterfaceDescSize, /* bLength */
	0x04,	/* bDescriptorType, Interface */
	0x00,	/* bInterfaceNumber */
	0x00,	/* bAlternateSetting */
	0x01,	/* bNumEndpoints */
	0xFF,	/* bInterfaceClass */
	0xFF,	/* bInterfaceSubClass */
	0x00,	/* bInterfaceProtocol */
	0x00,	/* iInteraface */

	/* OTG Descriptor */
	otg_function_OtgDescSize, /* bLength */
	0x09,	/* bDescriptorType, OTG */
	0x03,	/* bmAttributes, HNP and SRP suport */

	/* Endpoint descriptor */
	otg_function_EndpointDescSize,
	0x05,	/* bDescriptorType, endpoint */
	0x81,	/* bEndpointAddress, IN */
	0x03,	/* bmAttributes, interrupt */
	0x02,	/* wMaxPacketSize */
	0x00,
	0x80,	/* bInterval */
};

static char status[2] = { 0, 0 };
static otg_urb_t statusUrb;
static char otg_desc[3] = { 3, 9, 3};

static int otg_function_RequestBus(void)
{
/*
	int rc;

	rc = otg_fd_RequestBus();
	if (rc)
	{
		return rc;
	}
*/

	status[0] |= 1;
#ifdef OTG_HNP_DEBUG
info("RequestBus: status[0]=0x%0x", status[0]);
#endif
	otg_fd_Submit(&statusUrb);

	return 0;
}

static void otg_function_ReleaseBus(void)
{
	otg_fd_ReleaseBus();
	if (status[0] & 1)
	{
		status[0] = 0;
		otg_fd_Submit(&statusUrb);
	}
}

static int
otg_function_StandardRequest
(
	otg_device_request_t *request,
	void *private
)
{
#ifdef OTG_HNP_DEBUG
	info("StandardRequest: bmReguestType=0x%0x bRequest=0x%0x",
		request->bmRequestType, request->bRequest);
#endif
	if (request->bmRequestType == 0x80 && request->bRequest == 0x06)
	{
		request->wLength = otg_desc[0];
		request->data = otg_desc;
#ifdef OTG_HNP_DEBUG
	info("StandardRequest: wLength=%d", request->wLength);
	info("StandardRequest: desc[0]=0x%0x desc[0]=0x%0x desc[0]=0x%0x",
		otg_desc[0], otg_desc[1], otg_desc[2]);
#endif
	}
	else
	{
		request->wLength = 0;
	}


	return 0;
}

static int
otg_function_ClassRequest
(
	otg_device_request_t *request,
	void *private
)
{
	info("ClassRequest: ");
	request->wLength = 0;

	return 0;
}

static int
otg_function_VendorRequest
(
	otg_device_request_t *request,
	void *private
)
{
	info("VendorRequest: ");

	request->wLength = 0;

	return 0;
}

static void
otg_function_Complete(otg_urb_t *urb)
{
	if (status[0] & 1)
	{
		status[0] = 0;
		otg_fd_Submit(&statusUrb);
	}
}

static otg_fdevice_t otgDevice =
{
	deviceDesc: otgDeviceDesc,
	configDesc: otgConfigDesc,
	deviceDescLength: otg_function_DeviceDescSize,
	configDescLength: otg_function_ConfigDescSize +
		otg_function_InterfaceDescSize + otg_function_OtgDescSize +
		otg_function_EndpointDescSize,
	standardSetup: otg_function_StandardRequest,
	classSetup: otg_function_ClassRequest,
	vendorSetup: otg_function_VendorRequest,
};

static int __init otg_function_init(void)
{
	if (otg_fd_RegisterOtg(&otgDevice) < 0)
	{
		return -1;
	}

	otg_urb_Init(&statusUrb, 0x81, status, 2, otg_function_Complete);
	otg_urb_SetDevice(&statusUrb, &otgDevice);

	otg_fd_Submit(&statusUrb);

	return 0;
}

static void __exit otg_function_exit(void)
{
	otg_fd_UnregisterOtg(&otgDevice);
}

module_init(otg_function_init);
module_exit(otg_function_exit);

EXPORT_SYMBOL(otg_function_RequestBus);
EXPORT_SYMBOL(otg_function_ReleaseBus);
