/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_hc.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    Host controller object function definitions.
 */
#ifndef OTG_HC_H
#define OTG_HC_H

#include "uhcTypes.h"
#include "otg_hc_t.h"
#include "otg_hc_etd_t.h"
#include "otg_hc_intr_t.h"

int otg_hc_New(otg_hc_t* otg_hc);
int otg_hc_Init(otg_hc_t *hc, otg_t *otg, int start, int size);
void otg_hc_Reset(otg_hc_t *hc);
void otg_hc_ResetUnlocked(otg_hc_t *hc);
int otg_hc_Start(otg_hc_t *hc);
void otg_hc_Delete(otg_hc_t *hc);

void otg_hc_GoState(otg_hc_t *hc, int state);
void otg_hc_Exit(otg_hc_t *hc);
void otg_hc_DoIntr(otg_hc_t *hc);
int otg_hc_Submit(otg_hc_t *hc, stack_urb_t *urb);
int otg_hc_SubmitEtd(otg_hc_t *hc, otg_etd_t *etd, stack_urb_t *urb);
int otg_hc_UnlinkUrb(otg_hc_t *hc, stack_urb_t *urb);
int otg_hc_UnlinkList(otg_hc_t *hc, stack_urb_t *urb, OS_ListDeclare(*list));
void otg_hc_UnlinkAll(otg_hc_t *hc, stack_device_t *dev);
void otg_hc_UnlinkAllUnlocked(otg_hc_t *hc, stack_device_t *dev);
int otg_hc_UnlinkEtd(otg_hc_t *hc, otg_etd_t *etd);
void otg_hc_UnlinkAllEtd(otg_hc_t *hc, stack_device_t *dev, OS_ListDeclare(*l));

int otg_hc_DoWaitingEtd(otg_hc_t *hc);
void otg_hc_EtdDone(otg_hc_t *hc, otg_etd_t *etd);
int otg_hc_Allocate(otg_hc_t *hc, otg_etd_t *etd, int length);
void otg_hc_Release(otg_hc_t *hc, otg_etd_t *etd);
void otg_hc_StartIntrTimer(otg_hc_t *hc, int interval);
void otg_hc_IntrTimeout(unsigned long data);
int otg_hc_WaitStart(otg_hc_t *hc);

int otg_hc_LinkIso(otg_hc_t *hc, otg_etd_t *etd, stack_urb_t *urb);
void otg_hc_DeleteEtdForUrb(otg_hc_t *hc, otg_etd_t *etd);

otg_etd_t *otg_hc_GetFirstInPipe(otg_hc_t *hc, otg_etd_t *etd);

#ifdef USE_KERNEL_THREAD
void otg_hc_DoDoneQueue(void);
#endif

#ifdef USE_TASKLET
void otg_hc_Tasklet(unsigned long nothing);
#endif

#ifdef OTG_ISO_SHARE_MEMORY
void otg_hc_DoSofIntr(otg_hc_t *hc);
#endif

#ifdef DEBUG_MEMORY
void otg_hc_Stop(void);
#endif

#ifdef CONFIG_PM
void otg_hc_GoSuspend(otg_hc_t *hc);
void otg_hc_GoResume(otg_hc_t *hc);
void otg_hc_GoStandby(otg_hc_t *hc);
#endif

static inline void
otg_hc_WriteReg(otg_hc_t *hc, int reg, UINT32 val)
{
	otg_WriteHostReg(hc->otg, reg, val);
}

static inline UINT32
otg_hc_ReadReg(otg_hc_t *hc, int reg)
{
	return otg_ReadHostReg(hc->otg, reg);
}

static inline otg_hc_rh_t *
otg_hc_GetRootHub(otg_hc_t *hc)
{
	return &hc->rh;
}

static inline otg_t *
otg_hc_GetOtg(otg_hc_t *hc)
{
	return hc->otg;
}

/* This test will read from host controller the latest state */
static inline int
otg_hc_IsInState(otg_hc_t *hc, int state)
{
	UINT32 s;

	s = otg_ReadHostReg(hc->otg, otg_hc_Control_c) & otg_hc_HostState_c;
	return state == s;
}

static inline int
otg_hc_IsIOReady(otg_hc_t *hc)
{
	UINT32 w;

	w = otg_ReadChipReg(hc->otg, otg_IOReady_c);
	return !(w & otg_IOReady_NotReady);
}

static inline void
otg_hc_WriteStart(otg_hc_t *hc, int start)
{
	otg_WriteHostReg(hc->otg, otg_hc_AccessStartAddr_c, start);
}

static inline void
otg_hc_WriteData(otg_hc_t *hc, UINT32 w)
{
	otg_WriteChipReg(hc->otg, otg_PIO_dataPort_c, w);
}

static inline UINT32
otg_hc_ReadData(otg_hc_t *hc)
{
	return otg_ReadChipReg(hc->otg, otg_PIO_dataPort_c);
}

static inline int
otg_hc_GetFrameNo(otg_hc_t *hc)
{
	return hc->frameNo;
}

static inline int
otg_hc_ReadFrameNo(otg_hc_t *hc)
{
	hc->frameNo = otg_ReadHostReg(hc->otg, otg_hc_FmNumber_c);
	hc->frameNo &= otg_hc_FmMask_c;
	return hc->frameNo;
}

static inline int
otg_hc_LinkIntr(otg_hc_t *hc, otg_etd_t *etd, stack_urb_t *urb)
{
	OS_ListDeclare(*h);
	OS_ListDeclare(*list);
	int index;
	int i;
	
	if (!stack_urb_IsValidInterval(urb))
	{
		return -EINVAL;
	}

#ifdef DEBUG_OTG_SUBMIT
info("LinkIntr: original interval=%d", stack_urb_GetIntrInterval(urb));
#endif
	/* round down to power of 2. urb->interval=0 is 1ms */
	stack_urb_SetIntrInterval(urb, stack_urb_GetIntrInterval(urb) + 1);
	for (index = 7, i = 128; index > 0; index--, i >>= 1)
	{
		if (stack_urb_GetIntrInterval(urb) >= i)
		{
			stack_urb_SetIntrInterval(urb, i);
			break;
		}
	}

#ifdef DEBUG_OTG_SUBMIT
info("LinkIntr: index=%d interval=%d", index, stack_urb_GetIntrInterval(urb));
#endif
	list = &hc->intrList[index];
	for(h = OS_ListNext(list); h != list; h = OS_ListNext(h))
	{
		otg_etd_t *tmp;

		tmp = OS_ListEntry(h, otg_etd_t, formatEtdList);
		if (otg_etd_IsUnlink(tmp))
		{
			continue;
		}

		if  (tmp->urb == urb ||
				(
					stack_urb_HasSameDevice(tmp->urb, urb) &&
					stack_urb_HasSamePipe(tmp->urb, urb)
				)
			)
		{
			return -ENXIO;
		}
	}

	OS_ListAddTail(&etd->formatEtdList, list);

	return 0;
}

static inline int
otg_hc_LinkCtrl(otg_hc_t *hc, otg_etd_t *etd, stack_urb_t *urb)
{
	OS_ListDeclare(*h);
	OS_ListDeclare(*list);
	stack_pipe_t pipe;
	
	pipe = stack_urb_GetPipe(urb);
	if (!stack_urb_GetMaxPacketSize(urb, pipe))
	{
		return -EMSGSIZE;
	}

	list = &hc->ctrlList;
	for(h = OS_ListNext(list); h != list; h = OS_ListNext(h))
	{
		otg_etd_t *tmp;

		tmp = OS_ListEntry(h, otg_etd_t, formatEtdList);
		if (otg_etd_IsUnlink(tmp))
		{
			continue;
		}

		if  (tmp->urb == urb || 
				(
					stack_urb_HasSameDevice(tmp->urb, urb) && 
					!stack_urb_HasSameAddress(tmp->urb, urb)
				)
			)
		{
			return -ENXIO;
		}
	}

	OS_ListAddTail(&etd->formatEtdList, list);

	return 0;
}

static inline int
otg_hc_LinkBulk(otg_hc_t *hc, otg_etd_t *etd, stack_urb_t *urb)
{
	OS_ListDeclare(*h);
	OS_ListDeclare(*list);
	stack_pipe_t pipe;
	
	pipe = stack_urb_GetPipe(urb);
	if (stack_urb_IsHalted(urb, pipe))
	{
		return -EPIPE;
	}

	if (!stack_urb_GetBuffer(urb) || !stack_urb_GetBufferLength(urb))
	{
		return -EINVAL;
	}

	if (!stack_urb_GetMaxPacketSize(urb, pipe))
	{
		return -EMSGSIZE;
	}

	list = &hc->bulkList;
	for(h = OS_ListNext(list); h != list; h = OS_ListNext(h))
	{
		otg_etd_t *tmp;

		tmp = OS_ListEntry(h, otg_etd_t, formatEtdList);
		if (otg_etd_IsUnlink(tmp))
		{
			continue;
		}

		if 	(
				tmp->urb == urb ||
				(
					stack_urb_HasSameDevice(tmp->urb, urb) &&
					stack_urb_HasSamePipe(tmp->urb, urb)
				)
			)
		{
			if (!stack_urb_IsBulkQueued(tmp->urb) ||
				!stack_urb_IsBulkQueued(urb))
			{
				return -ENXIO;
			}
			else
			{
				otg_etd_LinkEtd(etd, tmp);
				return 0;
			}
		}
	}

	OS_ListAddTail(&etd->formatEtdList, list);

	return 0;
}

static inline int
otg_hc_GetStartFrameNo(otg_hc_t *hc, int interval)
{
	int start;

	/* Hareware needs at least 2ms to be ready. The actual start frame
	 * are 2^n - 1 + 2.
	 */
	start = hc->frameNo + interval - 1;
	if (interval < 4)
	{
		return start + 2;
	}

	start &= ~(interval - 1);

	start |= (interval >> 1) - 1;
	start += 2;

	return start;
}

#include "otg_hc_etd.h"
#include "otg_hc_intr.h"
#include "otg_hc_rh.h"

static inline int
otg_hc_GetIsoPosition(otg_hc_t *hc, otg_etd_t *etd)
{
	OS_ListDeclare(*h);
	otg_etd_t *tmp;
	int pos;

	h = &hc->isoList;
	pos = 0;
	for(h = OS_ListNext(h); h != &hc->isoList; h = OS_ListNext(h))
	{
		tmp = OS_ListEntry(h, otg_etd_t, formatEtdList);
		if (tmp == etd)
		{
			return pos;
		}

		if (otg_etd_IsUnlink(tmp))
		{
			continue;
		}

		if (stack_urb_HasSameDevice(tmp->urb, etd->urb) &&
			stack_urb_HasSamePipe(tmp->urb, etd->urb))
		{
			pos++;
		}
	}

#ifdef DEBUG_OTG_SUBMIT
info("GetIsoPosition: should not get here");
#endif
	return -1;
}

static inline void
otg_hc_SetOtgPortSuspend(otg_hc_t *hc)
{
	otg_hc_rh_SetOtgPortSuspend(&hc->rh);
}

static inline void
otg_hc_EnableSofIntr(otg_hc_t *hc)
{
	otg_hc_intr_Enable(&hc->intr, otg_hc_intr_SOF_c);
}

static inline void
otg_hc_DisableSofIntr(otg_hc_t *hc)
{
	otg_hc_intr_Disable(&hc->intr, otg_hc_intr_SOF_c);
}

static inline void
otg_hc_ClearDoneQueueIntr(otg_hc_t *hc)
{
	otg_hc_intr_t *intr;

	intr = &hc->intr;
	otg_hc_intr_ClearIntr(intr, otg_hc_intr_RDQ_c);
	otg_hc_intr_Clear(intr);
}

#endif /* #ifndef OTG_HC_H */
