/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_hc_etd.c
 * Author: Biao Jia		1/21/02
 * Description:
 *    Host ETD bank object handler.
 */
#include "os.h"
#include "stack_urb.h"
#include "otg.h"

#ifdef DEBUG_DONEQUEUE
extern UINT32 hc_intr_status;
extern UINT32 hc_etd_intr_status;
extern UINT32 hc_etd_enable_status;
extern UINT32 hc_etd_filled_status;
extern UINT32 hc_done_queue;
int done_queue_found = 0;
UINT32 dqs[5];
UINT32 dq_etd_enabls[5];
UINT32 dq_etd_int_enables[5];
UINT32 dq_frames[5];
UINT32 dq_start_frames[5];
UINT32 dq_dw2s[5];
int dq_buffers[5];
int dq_packets[5];
int dq_last_completed[5];
int current_ptr = 0;
#endif

void
otg_hc_etd_Init(otg_hc_etd_t *etd_bank, otg_hc_t *hc)
{
	int i;

	etd_bank->hc = hc;
	for(i=0; i<otg_hc_etd_MaxEtd_c; i++)
	{
		/* initially no ETD buffer is used */
		etd_bank->etd[i] = 0;
	}
}

void
otg_hc_etd_Reset(otg_hc_etd_t *etd_bank)
{
	int i;

	etd_bank->intrStatus = 0;
	etd_bank->bufferFilledStatus = 0;
	etd_bank->enableStatus = 0;

	otg_hc_etd_DisableIntr(etd_bank, otg_hc_etd_All_c);
	otg_hc_WriteReg
	(
		etd_bank->hc,
		otg_hc_etd_IntrStatus_c,
		otg_hc_etd_AllIntr_c
	);

	otg_hc_WriteReg(etd_bank->hc, otg_hc_etd_EnableClear_c, otg_hc_etd_All_c);

	for(i=0; i<otg_hc_etd_MaxEtd_c; i++)
	{
		otg_etd_t *etd;

		etd = etd_bank->etd[i];
		if (etd)
		{
#ifdef DEBUG_OTG_CHIP
info("Reset: delete etd %p", etd);
#endif
			otg_hc_Release(etd_bank->hc, etd);
			otg_etd_Delete(etd);
		}

		etd_bank->etd[i] = 0;
	}
}

void
otg_hc_etd_DoIntr(otg_hc_etd_t *etd_bank)
{
	int i;
	UINT32 w;
	UINT32 w1;
	otg_etd_t *etd;
#ifdef DEBUG_OTG_SUBMIT
	info("otg_hc_etd_DoIntr");
#endif

	if (otg_hc_etd_CheckIntr(etd_bank, otg_hc_etd_AllIntr_c))
	{
		i = 0;
		w = 0x1;
		w1 = 0x10000;
		while (otg_hc_etd_IsTriggered(etd_bank, otg_hc_etd_AllIntr_c))
		{
			etd = etd_bank->etd[i];
			if (otg_hc_etd_IsTriggered(etd_bank, w))
			{
				otg_etd_NextPacket(etd, etd_bank);
				otg_hc_etd_ClearStatus(etd_bank, w);
			}

			if (otg_hc_etd_IsTriggered(etd_bank, w1))
			{
				otg_etd_NextPacket(etd, etd_bank);
				otg_hc_etd_ClearStatus(etd_bank, w1);
			}

			i++;
			w <<= 1;
			w1 <<= 1;
			if (i >= otg_hc_etd_MaxEtd_c)
			{
				break;
			}
		}
	}
}

void
otg_hc_etd_Release(otg_hc_etd_t *etd_bank, int num)
{
	if (num < 0 || num >= otg_hc_etd_MaxEtd_c)
	{
		err("Illegal ETD index %d", num);
		return;
	}

	etd_bank->etd[num] = 0;
}

int
otg_hc_etd_Allocate(otg_hc_etd_t *etd_bank, otg_etd_t *etd)
{
	int i;
	otg_etd_t **ep;

	ep = etd_bank->etd;

	if (otg_etd_IsTransferType(etd, otg_etd_Format_Iso_c))
	{
		/* Iso packet will take whatever is available */
		for(i=0; i<otg_hc_etd_MaxEtd_c; i++)
		{
			if (!*ep)
			{
				*ep = etd;
				return i;
			}

			ep++;
		}
	}
	else
	{
		int num_iso;
		int num_ctrl;
		int num_bulk;
		int num_intr;
		int total;
		int empty;

		/* Count how many ETD has been allocated and to whom */
		num_iso = 0;
		num_ctrl = 0;
		num_bulk = 0;
		num_intr = 0;
		empty = -1;

		for(i=0; i<otg_hc_etd_MaxEtd_c; i++)
		{
			if (*ep)
			{
				switch (otg_etd_GetTransferType(*ep))
				{
					case otg_etd_Format_Ctrl_c:
						num_ctrl++;
						break;
					case otg_etd_Format_Iso_c:
						num_iso++;
						break;
					case otg_etd_Format_Bulk_c:
						num_bulk++;
						break;
					case otg_etd_Format_Intr_c:
						num_intr++;
						break;
				}
			}
			else if (empty < 0)
			{
				empty = i; /* Found a free ETD */
			}

			ep++;
		}

		if (empty < 0)
		{
			return -1; /* No ETD available */
		}

		total = num_iso + num_ctrl + num_bulk + num_intr;

		/* The algorithm:
		 *     if there is more than 4 free ETD left, allocate one
		 *     for ctrl or bulk pipe, if ctrl + bulk <= 2, allocate one
		 *          otherwise reject.
		 *     for intr pipe, if intr <= 2, allocate one
		 *          otherwise, if iso > 2 allocate one
		 *          otherwise regject.
		 */

		if ((otg_hc_etd_MaxEtd_c - total) > 4)
		{
		 	etd_bank->etd[empty] = etd;
		 	return empty;
		}

		switch (otg_etd_GetTransferType(*ep))
		{
			case otg_etd_Format_Ctrl_c:
			case otg_etd_Format_Bulk_c:
				if ((num_bulk + num_ctrl) <= 2)
				{
					etd_bank->etd[empty] = etd;
					return empty;
				}

				return -1;

			case otg_etd_Format_Intr_c:
				if (num_intr <= 2 || num_iso > 2)
				{
					etd_bank->etd[empty] = etd;
					return empty;
				}
				return -1;
		}

	}

	return -1;
}

int
otg_hc_etd_Enable(otg_hc_etd_t *etd, int index)
{
	UINT32 w;
	int i;

#ifdef DEBUG_OTG_SUBMIT
info("Enable: index=%d enableStatus=0x%0x etd=%p", index, etd->enableStatus, etd);
#endif
	w = 1 << index;

	otg_hc_etd_EnableIntr(etd, w);
	if (etd->enableStatus & w)
	{
		return 0;
	}

	/* Will try three times */
	for (i=0; i<3; i++)
	{
		otg_hc_WriteReg(etd->hc, otg_hc_etd_EnableSet_c, w);
		etd->enableStatus = otg_hc_ReadReg(etd->hc, otg_hc_etd_EnableStatus_c);
	
		/* Enable interrupt on this ETD */
		if (etd->enableStatus & w)
		{
			return 0;
		}
	}
	
	err("Can't enable ETD%d", index);
#ifdef DEBUG_OTG_CHIP
otg_DumpReg(etd->hc->otg);
#endif
	return -1;
}

void
otg_hc_etd_UploadDoneQueue(otg_hc_etd_t *hc_etd)
{
	UINT32 w;
	int index;
	UINT32 pos;
	otg_etd_t *etd;
	UINT32 dq;
#ifdef DEBUG_DONEQUEUE
	UINT32 ww2;
	UINT32 ww3;
	UINT32 ww4;
	int ptr;
	int n;
	otg_etd_t *e;
#endif

	w = otg_hc_ReadReg(hc_etd->hc, otg_hc_etd_DoneQueue_c) & 0xFFFF;
	hc_etd->enableStatus = otg_hc_ReadReg(hc_etd->hc,otg_hc_etd_EnableStatus_c);
       	
#ifdef DEBUG_DONEQUEUE
	ww2 = otg_hc_ReadReg(hc_etd->hc, otg_hc_etd_IntrEnable_c) & 0xFFFF;
	ptr = current_ptr;
	current_ptr++;
	if (current_ptr >= 5)
	{
		current_ptr = 0;
	}

	dqs[ptr] = w;
	dq_etd_enabls[ptr] = otg_hc_ReadReg(hc_etd->hc, otg_hc_etd_EnableStatus_c);
	dq_etd_int_enables[ptr] = ww2;
	dq_frames[ptr] = hc_etd->hc->frameNo;
#endif
	index = 0;
	pos = 1;
	while (w)
	{
		if (w & pos)
		{
			etd = hc_etd->etd[index];
			if (etd)
			{
#ifdef DEBUG_DONEQUEUE
	dq_start_frames[ptr] = stack_urb_GetStartFrame(etd->urb);
	dq_dw2s[ptr] = etd->td.iso.ctrl;
	e = OS_ListEntry(OS_ListPrev(&etd->pipeEtdList), otg_etd_t, pipeEtdList);
	dq_buffers[ptr] = e->buffer;
	dq_packets[ptr] = e->totalPackets;
	dq_last_completed[ptr] = e->lastCompleted;
#endif
				otg_hc_etd_DisableIntr(hc_etd, pos);
				if (hc_etd->enableStatus & pos)
				{
					/* ETD enable may not be cleared yet */
					otg_hc_WriteReg(etd->hc, otg_hc_etd_EnableClear_c, pos);
					hc_etd->enableStatus &= ~pos;
				}

				otg_etd_SetInDoneQueue(etd);

				if (!otg_etd_IsUnlink(etd))
				{
					otg_hc_etd_UploadEtd(hc_etd, etd);
				}
#ifdef DEBUG_OTG_CHIP
				else
				{
info("UploadDoneQueue: ETD %p 0x%0x has been deleted", etd, pos);
				}
#endif
			}
			else
			{
#ifdef DEBUG_MEMORY
otg_hc_Stop();
#endif
				err("UploadDoneQueue: etd%d is empty", index);
			}

			w &= ~pos;
		}

		index++;
		pos <<= 1;
	}

	/* Need to workaround a hardware bug. The bug will not put
	 * the done ETD in done queue but will disable it.
	 */
	otg_hc_ClearDoneQueueIntr(hc_etd->hc);
	w = otg_hc_ReadReg(hc_etd->hc, otg_hc_etd_IntrEnable_c) & 0xFFFF;
	dq = otg_hc_ReadReg(hc_etd->hc, otg_hc_etd_DoneQueue_c) & 0xFFFF;
#ifdef DEBUG_DONEQUEUE
	ww3 = w;
	ww4 = dq;
#endif
	index = 0;
	pos = 1;
	w ^= hc_etd->enableStatus & 0xFFFF;
	while (w)
	{
		if (w & pos & dq)
		{
			w &= ~pos;
		}
		else if (w & pos)
		{
			etd = hc_etd->etd[index];
			if (etd)
			{
#ifdef DEBUG_MEMORY
info("    ETD%d=%p", index, etd);
otg_hc_Stop();
#endif
#if 0
//FB: This is really an info that the bug was hit.
				err("UploadDoneQueue: ETD%d is not in done queue", index);
#endif
#ifdef DEBUG_DONEQUEUE
done_queue_found = 1;

ptr = current_ptr;
for(n = 0; n < 5; n++)
{
printk("%d dq=0x%0x etd_enable=0x%0x etd_int_enable=0x%0x buffer=0x%0x packets=%d lcom=%d\n",
	n, dqs[ptr], dq_etd_enabls[ptr], dq_etd_int_enables[ptr],
	dq_buffers[ptr], dq_packets[ptr], dq_last_completed[ptr]);
printk("   frame=0x%0x start_frame=0x%0x dw2=0x%0x\n",
	dq_frames[ptr], dq_start_frames[ptr], dq_dw2s[ptr]);
	ptr++;
	if (ptr >= 5)
	{
		ptr = 0;
	}
}
info("after dq=0x%0x etd int enable=0x%0x etd enable=0x%0x",
	ww4, ww3, hc_etd->enableStatus);
#endif
				otg_hc_etd_DisableIntr(hc_etd, pos);
				otg_etd_SetInDoneQueue(etd);

				if (!otg_etd_IsUnlink(etd))
				{
					otg_hc_etd_UploadEtd(hc_etd, etd);
				}
			}
			else
			{
#ifdef DEBUG_MEMORY
otg_hc_Stop();
#endif
				err("UploadDoneQueue: ETD%d is empty", index);
			}

			w &= ~pos;
		}

		index++;
		pos <<= 1;
	}
}

void
otg_hc_etd_DoDoneQueue(otg_hc_etd_t *hc_etd)
{
	int index;
	otg_etd_t *etd;

	for (index = 0; index < otg_hc_etd_MaxEtd_c; index++)
	{
		etd = hc_etd->etd[index];
		if (etd)
		{
			if (!otg_etd_IsInDoneQueue(etd))
			{
				continue;
			}
#ifdef DEBUG_OTG_CHIP
info("DoDoneQueue: index=%d etd=%p", index, etd);
#endif
#ifdef DEBUG_DONEQUEUE
if (done_queue_found)
{
printk("packets=%d completd=%d etd enable=0x%0x frame=0x%0x urb=%p\n",
		etd->totalPackets, etd->completed,
		otg_hc_ReadReg(hc_etd->hc, otg_hc_etd_EnableStatus_c),
		hc_etd->hc->frameNo, etd->urb);
otg_hc_WriteStart(hc_etd->hc, (otg_hc_etd_GetAddr(etd->etdIndex) + 7) | otg_hc_StartEtd_c | otg_hc_StartRead_c);
	printk("dw0=0x%0x dw1=0x%0x dw2=0x%0x dw3=0x%0x dw4=0x%0x\n",
		etd->endpoint, etd->buffer, etd->td.iso.ctrl, etd->td.iso.data[0],
		etd->td.iso.data[1]);
	otg_hc_WaitStart(hc_etd->hc);
	printk("dw5=0x%0x dw6=0x%0x dw7=0x%0x\n",
		etd->td.iso.data[2], etd->td.iso.data[3], otg_hc_ReadData(hc_etd->hc));
}
#endif

			otg_hc_etd_Disable(hc_etd, index);
			otg_etd_ClearInDoneQueue(etd);
			if (!otg_etd_IsUnlink(etd))
			{
				otg_etd_DoDoneEtd(etd, hc_etd);
			}

			/* On return, etd may be destroyed */
			otg_hc_EtdDone(hc_etd->hc, etd);
		}
	}
}

int
otg_hc_etd_Transfer(otg_hc_etd_t *hc_etd, otg_etd_t *etd)
{
	int rc;
	int etd_index;

	etd_index = otg_etd_GetEtdIndex(etd);
	hc_etd->etd[etd_index] = etd;
	otg_hc_etd_DownloadEtd(hc_etd, etd);

#ifdef OTG_ISO_SHARE_MEMORY
	if (otg_etd_IsSofLinked(etd))
	{
#ifdef DEBUG_OTG_CHIP
info("Transfer: sof linked");
#endif
		return 0;
	}
#endif
	if (!otg_etd_IsInEtd(etd))
	{
		/* For OUT pipe, we need to download the first two packets data */
		otg_etd_NextPacket(etd, hc_etd);
		if (otg_td_ctrl_GetPacketCount(etd) > 1)
		{
			otg_etd_NextPacket(etd, hc_etd);
		}
	}

	/* Enable the ETD */
	rc = otg_hc_etd_Enable(hc_etd, etd_index);

	return rc;
}

int
otg_hc_etd_TransferIntr(otg_hc_etd_t *hc_etd, otg_etd_t *etd)
{
	int rc;
	int etd_index;

#ifdef DEBUG_OTG_SUBMIT
	info("otg_hc_etd_TransferIntr");
#endif

	if (!otg_etd_IsInEtd(etd))
	{
		otg_etd_NextPacket(etd, hc_etd);
	}

	/* Enable the ETD */
	etd_index = otg_etd_GetEtdIndex(etd);
	rc = otg_hc_etd_Enable(hc_etd, etd_index);

	return 0;
}
