/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_hc_etd.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    Host ETD bank object function definitions.
 */
#ifndef OTG_HC_ETD_H
#define OTG_HC_ETD_H

#include "otg_hc_etd_t.h"
#include "otg_etd_t.h"

void otg_hc_etd_Init(otg_hc_etd_t *etd_bank, otg_hc_t *hc);
void otg_hc_etd_Reset(otg_hc_etd_t *etd_bank);

void otg_hc_etd_DoIntr(otg_hc_etd_t *etd_bank);
int otg_hc_etd_Allocate(otg_hc_etd_t *etd_bank, otg_etd_t *etd);
void otg_hc_etd_Release(otg_hc_etd_t *etd_bank, int num);
void otg_hc_etd_UploadDoneQueue(otg_hc_etd_t *etd);
void otg_hc_etd_DoDoneQueue(otg_hc_etd_t *etd);
void otg_hc_etd_DoDoneEtd(otg_hc_etd_t *etd, int index);
int otg_hc_etd_Enable(otg_hc_etd_t *etd, int index);
int otg_hc_etd_Transfer(otg_hc_etd_t *hc_etd, otg_etd_t *etd);
int otg_hc_etd_TransferIntr(otg_hc_etd_t *hc_etd, otg_etd_t *etd);

static inline int
otg_hc_etd_IsTriggered(otg_hc_etd_t *etd_bank, UINT32 no)
{
	return etd_bank->intrStatus & no;
}

static inline int
otg_hc_etd_EnableIntr(otg_hc_etd_t *etd, int no)
{
	otg_hc_WriteReg(etd->hc, otg_hc_etd_IntrEnable_c, no);
#ifdef DEBUG_OTG_SUBMIT
	otg_hc_ReadReg(etd->hc, otg_hc_etd_IntrEnable_c);
#endif

	return 0;
}

static inline void
otg_hc_etd_DisableIntr(otg_hc_etd_t *etd, int no)
{
	otg_hc_WriteReg(etd->hc, otg_hc_etd_IntrDisable_c, no);
#ifdef DEBUG_OTG_SUBMIT
	otg_hc_ReadReg(etd->hc, otg_hc_etd_IntrEnable_c);
#endif
}

static inline int
otg_hc_etd_CheckIntr(otg_hc_etd_t *etd_bank, int no)
{
	UINT32 w;

	w = otg_hc_ReadReg(etd_bank->hc, otg_hc_etd_IntrStatus_c);
	if (w)
	{
		etd_bank->intrStatus |= w;
		otg_hc_WriteReg(etd_bank->hc, otg_hc_etd_IntrStatus_c, w);
	}

	return otg_hc_etd_IsTriggered(etd_bank, no);
}

static inline void
otg_hc_etd_ClearStatus(otg_hc_etd_t *etd_bank, UINT32 no)
{
	etd_bank->intrStatus &= ~no;
}

static inline int
otg_hc_etd_GetAddr(int num)
{
	return num << 3; /* 32 bytes per ETD starting from zero */
}

static inline void
otg_hc_etd_ClearFillBuffer(otg_hc_t *hc, int etd_index, int yBuffer)
{
	UINT32 w;

	w = 1 << (etd_index + yBuffer * 16);
	otg_hc_WriteReg(hc, otg_hc_etd_FilledClear_c, w);
}

static inline void
otg_hc_etd_ClearFill(otg_hc_etd_t *etd, int etd_index, int yBuffer)
{
	otg_hc_etd_ClearFillBuffer(etd->hc, etd_index, yBuffer);
}

static inline void
otg_hc_etd_SetFill(otg_hc_etd_t *etd, int etd_index, int yBuffer)
{
	UINT32 w;

	w = 1 << (etd_index + yBuffer * 16);
	otg_hc_WriteReg(etd->hc, otg_hc_etd_FilledSet_c, w);
}

static inline void
otg_hc_etd_Disable(otg_hc_etd_t *etd, int index)
{
	etd->etd[index] = 0;
	etd->enableStatus &= ~(1 << index);
}

#include "otg_hc.h"
#include "otg_etd.h"

static inline void
otg_hc_etd_UploadEtd(otg_hc_etd_t *hc_etd, otg_etd_t *etd)
{
	int addr;
	int i;
	int count;
	int rc;

/* Already read. No need to read it again
	if (otg_etd_IsInEtd(etd))
	{
		otg_etd_NextPacket(etd, hc_etd);
	}
*/

	addr = otg_hc_etd_GetAddr(otg_etd_GetEtdIndex(etd));
	addr |= otg_hc_StartEtd_c | otg_hc_StartRead_c;
	otg_hc_WriteStart(hc_etd->hc, addr);

	otg_etd_ClearInHc(etd);
	if (otg_etd_IsTransferType(etd, otg_etd_Format_Iso_c))
	{
		count = 7;
		rc = otg_hc_WaitStart(hc_etd->hc);
	}
	else
	{
		count = 4;
		rc = otg_hc_WaitStart(hc_etd->hc);
	}

	if (rc)
	{
		return;
	}

	for(i = 0; i < count; i++)
	{
		otg_etd_SetEtdFromHc(etd, i, otg_hc_ReadData(hc_etd->hc));
	}
}

static inline void
otg_hc_etd_DownloadEtd(otg_hc_etd_t *etd_bank, otg_etd_t *etd)
{
	int addr;
	int index;
	int i;
	int count;
	int rc;

	index = otg_etd_GetEtdIndex(etd);
	addr = otg_hc_etd_GetAddr(index);
#ifdef DEBUG_OTG_SUBMIT
info("DownloadEtd index=0x%x addr=0x%x", index, addr);
#endif

	/* No DMA */
	addr |= otg_hc_StartEtd_c|otg_hc_StartWrite_c;
	otg_hc_WriteStart(etd_bank->hc, addr);

	otg_etd_SetInHc(etd);
	/* To be parallel with HC start, save time */
	if (otg_etd_IsTransferType(etd, otg_etd_Format_Iso_c))
	{
		count = 7;
		rc = otg_hc_WaitStart(etd_bank->hc);
	}
	else
	{
		count = 4;
		rc = otg_hc_WaitStart(etd_bank->hc);
	}

	if (rc)
	{
		return;
	}

	for(i = 0; i < count; i++)
	{
		otg_hc_WriteData(etd_bank->hc, otg_etd_GetDWord(etd, i));
	}

	/* Software workaround a hardware bug */
	for(i = count; i < 8; i++)
	{
		otg_hc_WriteData(etd_bank->hc, 0);
	}
#ifdef DEBUG_OTG_ETD
	addr &= ~otg_hc_StartWrite_c;
	otg_hc_WriteStart(etd_bank->hc, addr);
	otg_hc_WaitStart(etd_bank->hc);
	for(i = 0; i < 7; i++)
	{
		otg_hc_ReadData(etd_bank->hc);
	}
#endif
}

#endif /* #ifndef OTG_HC_ETD_H */
