/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_hc_intr.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    Host controller interrupt object function definitions.
 */
#ifndef OTG_HC_INTR_H
#define OTG_HC_INTR_H

#include "otg_hc_intr_t.h"

static inline void
otg_hc_intr_Init(otg_hc_intr_t *intr, otg_hc_t *hc)
{
	intr->hc = hc;
}

static inline void
otg_hc_intr_Reset(otg_hc_intr_t *intr)
{
	UINT32 w;

	intr->enabledIntr = 0;
	intr->status = 0;
	intr->completeIntr = 0;

	w = otg_hc_intr_All_c;
	otg_hc_WriteReg(intr->hc, otg_hc_intr_Disable_c, w);
	otg_hc_WriteReg(intr->hc, otg_hc_intr_Status_c, w);
}

static inline void
otg_hc_intr_Enable(otg_hc_intr_t *intr, UINT32 no)
{
	otg_hc_WriteReg(intr->hc, otg_hc_intr_Enable_c, no);
	intr->enabledIntr |= no;
}

static inline void
otg_hc_intr_Disable(otg_hc_intr_t *intr, UINT32 no)
{
	otg_hc_WriteReg(intr->hc, otg_hc_intr_Disable_c, no);
	intr->enabledIntr &= ~no;
}

static inline int
otg_hc_intr_IsTriggered(otg_hc_intr_t *intr, UINT32 no)
{
	return intr->status & no;
}

static inline int
otg_hc_intr_Check(otg_hc_intr_t *intr, UINT32 no)
{
	UINT32 w;

	w = otg_hc_ReadReg(intr->hc, otg_hc_intr_Status_c);
	if (w)
	{
		intr->status |= w;
		otg_hc_WriteReg(intr->hc, otg_hc_intr_Status_c, w & ~otg_hc_intr_RDQ_c);
	}

	return otg_hc_intr_IsTriggered(intr, no);
}

static inline void
otg_hc_intr_ClearIntr(otg_hc_intr_t *intr, UINT32 no)
{
	intr->status &= ~no;
	intr->completeIntr |= no;
}

static inline void
otg_hc_intr_Clear(otg_hc_intr_t *intr)
{
	if (intr->completeIntr & otg_hc_intr_RDQ_c)
	{
		otg_hc_WriteReg(intr->hc, otg_hc_intr_Status_c, otg_hc_intr_RDQ_c);

		intr->completeIntr &= ~otg_hc_intr_RDQ_c;
	}
}

#endif /* #ifndef OTG_HC_INTR_H */
