/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_hc_rh.c
 * Author: Biao Jia		1/21/02
 * Description:
 *    Root hub object handler.
 */
#include "os.h"
#include "stack_urb.h"
#include "otg.h"

char otg_hc_rh_StringDescriptor[] =
{
	66,
	0x03, /* String descriptor */
	0x00, 'T', /* total 42 bytes */
	0x00, 'r',
	0x00, 'a',
	0x00, 'n',
	0x00, 's',
	0x00, 'D',
	0x00, 'i',
	0x00, 'm',
	0x00, 'e',
	0x00, 'n',
	0x00, 's',
	0x00, 'i',
	0x00, 'o',
	0x00, 'n',
	0x00, ',',
	0x00, ' ',
	0x00, 'I',
	0x00, 'n',
	0x00, 'c',
	0x00, '.',
	0x00, 'O', /* offset 21 * 2,  total 14 bytes */
	0x00, 'T',
	0x00, 'G',
	0x00, ' ',
	0x00, '2',
	0x00, '4',
	0x00, '3',
	0x00, '0', /* offset 28 * 2,  total 8 bytes */
	0x00, '1',
	0x00, '0',
	0x00, '0',
};

void
otg_hc_rh_Init(otg_hc_rh_t *rh, otg_hc_t *hc)
{
	int i;

	rh->hc = hc;
	for(i = 0; i < otg_hc_rh_MaxPorts; i++)
	{
		otg_hc_rh_port_Init(&rh->ports[i], rh, i);
	}

	/* Initialize descriptors */
	rh->deviceDescriptor[0] = otg_hc_rh_DeviceDecriptorLength;
	rh->deviceDescriptor[1] = 0x01; /* Device descriptor */
	rh->deviceDescriptor[2] = otg_hc_rh_USBReleaseNo & 0xFF;
	rh->deviceDescriptor[3] = (otg_hc_rh_USBReleaseNo >> 8) & 0xFF;
	rh->deviceDescriptor[4] = 0x09; /* Hub class */
	rh->deviceDescriptor[5] = 0x00;
	rh->deviceDescriptor[6] = 0x00;
	rh->deviceDescriptor[7] = 0x08;
	rh->deviceDescriptor[8] = 0x20; /* TDI's vendor ID */
	rh->deviceDescriptor[9] = 0x0B;
	rh->deviceDescriptor[10] = 0x43; /* Product ID OTG 243 */
	rh->deviceDescriptor[11] = 0x02;
	rh->deviceDescriptor[12] = otg_hc_rh_ReleaseNo & 0xFF;
	rh->deviceDescriptor[13] = (otg_hc_rh_ReleaseNo >> 8) & 0xFF;
	rh->deviceDescriptor[14] = 0x00;
	rh->deviceDescriptor[15] = 42;
	rh->deviceDescriptor[16] = 56;
	rh->deviceDescriptor[17] = 0x01; /* only one configuration */

	/* Configuration descriptor */
	rh->configDescriptor[0] = otg_hc_rh_ConfigDecriptorLength;
	rh->configDescriptor[1] = 0x02; /* Configuration descriptor */
	i = otg_hc_rh_ConfigDecriptorLength + otg_hc_rh_InterfaceDecriptorLength +
		otg_hc_rh_EndpointDecriptorLength;
	rh->configDescriptor[2] = i & 0xFF; /* Total length */
	rh->configDescriptor[3] = (i >> 8) & 0xFF;
	rh->configDescriptor[4] = 0x01; /* Only one interface */
	rh->configDescriptor[5] = 0x01;
	rh->configDescriptor[6] = 0x00;
	rh->configDescriptor[7] = 0xC0; /* Self-powered */
	rh->configDescriptor[8] = 0x00;

	/* Interface descriptor */
	rh->interfaceDescriptor[0] = otg_hc_rh_InterfaceDecriptorLength;
	rh->interfaceDescriptor[1] = 0x04; /* Interface descriptor */
	rh->interfaceDescriptor[2] = 0x00;
	rh->interfaceDescriptor[3] = 0x00;
	rh->interfaceDescriptor[4] = 0x01; /* Number of endpoints */
	rh->interfaceDescriptor[5] = 0x09; /* Hub class */
	rh->interfaceDescriptor[6] = 0x00;
	rh->interfaceDescriptor[7] = 0x00;
	rh->interfaceDescriptor[8] = 0x00;

	/* Endpoint descriptor */
	rh->endpointDescriptor[0] = otg_hc_rh_EndpointDecriptorLength;
	rh->endpointDescriptor[1] = 0x05; /* Endpoint descriptor */
	rh->endpointDescriptor[2] = 0x81; /* IN pipe for status/change */
	rh->endpointDescriptor[3] = 0x03; /* Interrupt */
	rh->endpointDescriptor[4] = 0x08; /* Packet size 8 bytes */
	rh->endpointDescriptor[5] = 0x00;
	rh->endpointDescriptor[6] = 0xFF; /* 255ms interrupt interval */
}

void
otg_hc_rh_StartTimer(otg_hc_rh_t *rh)
{
	OS_DeclareTimer(*timer);

	timer = &rh->timer;
	OS_StartTimer(timer, otg_hc_rh_Timeout, rh, rh->interval);
}

void
otg_hc_rh_Timeout(unsigned long data)
{
	otg_hc_rh_t *rh = (otg_hc_rh_t *)data;

	if (rh->enableSend)
	{
		int i;
		UINT8 w;
		stack_urb_t *urb;

		/* Return status:
		 * bit0: 1 - hub status change detected
		 * bit1: 1 - port 1 status change detected
		 * bit1: 2 - port 2 status change detected
		 * bit1: 3 - port 3 status change detected
		 */
		w = 0;
		for(i = 0; i < rh->num_ports; i++)
		{
			if (otg_hc_rh_port_IsChanged(&rh->ports[i]))
			{
				w |= (1 << (i + 1));
			}
		}

		if (w)
		{
			/* If status change, send it back otherwise ignore (NAK) */
			urb = rh->urb;
			*((UINT8 *)stack_urb_GetBuffer(urb)) = w;
			stack_urb_SetActualLength(urb, 1);
			stack_urb_SetStatus(urb, 0);

			otg_hcd_giveback_urb(rh->hc, urb);
		}
	}

	otg_hc_rh_StartTimer(rh);
}

#if 1 //TODO: NB: RH handled differently, yes, no?
int
otg_hc_rh_Submit(otg_hc_rh_t *rh, stack_urb_t *urb)
{
	int rc;
	stack_setup_t *cmd;

	if (stack_urb_IsIntrPipe(urb))
	{
		if (stack_urb_GetEndpoint(urb) != 1) /* IN EP = 1 */
		{
			return -EPIPE;
		}
		
		if (rh->enableSend)
		{
			return -ENXIO; /* Already sent */
		}

		rh->enableSend = 1;
		rh->urb = urb;
		rh->interval = stack_urb_GetIntrInterval(urb);

		otg_hc_rh_StartTimer(rh);

		return 0;
	}

	if (!stack_urb_IsCtrlPipe(urb) || stack_urb_GetEndpoint(urb))
	{
		return -EPIPE;
	}

	cmd = stack_urb_GetSetup(urb);
	rc = otg_hc_rh_DoCommand(rh,
		stack_urb_GetReqestType(cmd), OS_Le16ToCpu(stack_urb_GetReqestValue(cmd)),
		OS_Le16ToCpu(stack_urb_GetReqestIndex(cmd)), stack_urb_GetBuffer(urb),
		OS_Le16ToCpu(stack_urb_GetReqestLength(cmd)));
	if (rc)
	{
		return rc;
	}

	return 0;
}
#endif

/* Was hubDescriptor variable */
static void
otg_hc_rh_hub_descriptor (
	struct usb_hub_descriptor	*desc
) {
	desc->bDescLength = otg_hc_rh_DecriptorLength;
	desc->bDescriptorType = 0x29; /* Hub descriptor */
	desc->bNbrPorts = otg_hc_rh_MaxPorts; /* Number of ports */
	desc->wHubCharacteristics = (__force __u16)cpu_to_le16(0);
	desc->bPwrOn2PwrGood = 0x00;
	desc->DeviceRemovable[0] = 0xE0;
	desc->PortPwrCtrlMask[0] = 0x07;
	desc->DeviceRemovable[1] = 0xE0;
	desc->PortPwrCtrlMask[1] = 0x07;
	desc->DeviceRemovable[2] = 0xE0;
	desc->PortPwrCtrlMask[2] = 0x07;
}

/*
typeReq=0xa300 wValue=0x0 wIndex=0x1 wLength=0x4
GetPortStatus
typeReq=0x2301 wValue=0x10 wIndex=0x1 wLength=0x0
ClearPortFeature 

typeReq=0x0080 wValue=0x100 wIndex=0x0 wLength=0x40
USB_REQ_GET_STATUS - USB_DIR_IN: 
*/

int
otg_hc_rh_DoCommand(otg_hc_rh_t *rh, u16 typeReq, u16 wValue,
			 u16 wIndex, char *buf, u16 wLength)
{
	int rc = 0;
	unsigned long flags;
	u32 w;

	otg_LockChipIntr(otg_hc_GetOtg(rh->hc), flags);

#ifdef DEBUG_OTG_RH
info("otg_hc_rh_Docommand typeReq=0x%x wValue=0x%x wIndex=0x%x wLength=0x%x",
	typeReq, wValue, wIndex, wLength);
#endif

	switch (typeReq) {
	case GetHubStatus:
		*(__le32 *) buf = cpu_to_le32(0);
#ifdef DEBUG_OTG_RH
		info("otg_hc_rh_Docommand GetHubStatus=0x00");
#endif
		break;
	case ClearHubFeature:
		/* Hub feature, nothing to clear */
#ifdef DEBUG_OTG_RH
		info("otg_hc_rh_Docommand ClearHubFeature");
#endif
		break;
	case SetHubFeature:
		/* Hub feature, nothing to set */
#ifdef DEBUG_OTG_RH
		info("otg_hc_rh_Docommand SetHubFeature");
#endif
		break;
	case GetHubDescriptor:
		otg_hc_rh_hub_descriptor((struct usb_hub_descriptor*)buf);
		break;
/*	case SetHubDescriptor: //NB: Do something with that?
		rc = otg_hc_rh_SetDescriptor(rh, urb, type, wValue, wLength);
		break;*/
	case GetPortStatus:
		if (wIndex > rh->num_ports) {
#ifdef DEBUG_OTG_RH
			info("otg_hc_rh_Docommand GetPortStatus error");
#endif
			rc = -EPIPE;
			break;
		}
		*(__le32 *) buf = OS_CpuToLe32(otg_hc_rh_port_GetStatus(&rh->ports[wIndex - 1]));
#ifdef DEBUG_OTG_RH
		info("otg_hc_rh_Docommand GetPortStatus=0x%x", otg_hc_rh_port_GetStatus(&rh->ports[wIndex - 1]));
#endif
		break;
	case ClearPortFeature:
		if (wIndex > rh->num_ports) {
#ifdef DEBUG_OTG_RH
			info("otg_hc_rh_Docommand ClearPortFeature error: index=0x%x np=0x%x", wIndex, rh->num_ports);
#endif
			rc = -EPIPE;
			break;
		}

		/* Clear port feature */
		w = 0;
		switch (wValue) {
		case otg_hc_rh_port_PORT_ENABLE:
			w = otg_hc_rh_port_Status_CCS_c; /* ClearPortEnable */
			break;
		case otg_hc_rh_port_PORT_SUSPEND:
			w = otg_hc_rh_port_Status_POCI_c; /* ClearPortSuspend */
			break;
		case otg_hc_rh_port_PORT_POWER:
			w = otg_hc_rh_port_Status_LSDA_c; /* ClearPortPower */
			break;
		case otg_hc_rh_port_C_PORT_CONNECTION:
			w = otg_hc_rh_port_Status_CSC_c;
			break;
		case otg_hc_rh_port_C_PORT_RESET:
			w = otg_hc_rh_port_Status_PRSC_c;
			break;
		case otg_hc_rh_port_C_PORT_ENABLE:
			w = otg_hc_rh_port_Status_PESC_c;
			break;
		case otg_hc_rh_port_C_PORT_SUSPEND:
			w = otg_hc_rh_port_Status_PSSC_c;
			break;
		case otg_hc_rh_port_C_PORT_OVER_CURRENT:
			w = otg_hc_rh_port_Status_PORTOCIC_c;
			break;
		default:
			return -EPIPE;
		}

#ifdef DEBUG_OTG_RH
		info("otg_hc_rh_Docommand ClearPortFeature");
#endif

		otg_hc_rh_port_SetStatus(&rh->ports[wIndex - 1], w);
		break;
	case SetPortFeature:
		if (wIndex < 1 || wIndex > rh->num_ports) {
#ifdef DEBUG_OTG_RH
			info("otg_hc_rh_Docommand SetPortFeature error");
#endif
			rc = -EPIPE;
			break;
		}

		/* Clear port feature */
		w = 0;
		switch (wValue)
		{
		case otg_hc_rh_port_PORT_ENABLE:
			w = otg_hc_rh_port_Status_PES_c; /* SetPortEanble */
			break;
		case otg_hc_rh_port_PORT_SUSPEND:
			w = otg_hc_rh_port_Status_PSS_c; /* SetPortSuspend */
			break;
		case otg_hc_rh_port_PORT_RESET:
			w = otg_hc_rh_port_Status_PRS_c;
			break;
		case otg_hc_rh_port_PORT_POWER:
			w = otg_hc_rh_port_Status_PPS_c; /* SetPortPower */
			break;
		default:
			rc = -EPIPE;
		}

#ifdef DEBUG_OTG_RH
		info("otg_hc_rh_Docommand SetPortFeature");
#endif

		otg_hc_rh_port_SetStatus(&rh->ports[wIndex - 1], w);
		break;
	/*case SetPortAddress: //TODO: Not sure...
		rc = otg_hc_rh_SetAddress(rh, type, wValue);
		break;*/
	/*case GetPortDescriptor: //TODO: Not sure...
		rc = otg_hc_rh_GetPortDescriptor(rh, type, wValue, wLength);
		break;
	case SetPortDescriptor: //TODO: Not sure...
		rc = otg_hc_rh_SetPortDescriptor(rh, type, wValue, wLength);
		break;
	case GetPortConfig: //TODO: Not sure...
		rc = otg_hc_rh_GetPortConfig(rh, type, wValue);
		break;
	case SetPortConfig: //TODO: Not sure...
		rc = otg_hc_rh_SetPortConfig(rh, type, wValue);
		break;*/
	default:
#ifdef DEBUG_OTG_RH
		info("otg_hc_rh_Docommand Invalid Request...");
#endif
		rc = -EINVAL;
		break;
	}

	otg_UnlockChipIntr(otg_hc_GetOtg(rh->hc), flags);

	return rc;
}
