/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_hc_rh.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    Root hub object function definitions.
 */
#ifndef OTG_HC_RH_H
#define OTG_HC_RH_H

#include "otg_hc_rh_t.h"
#include "otg_hc_rh_port_t.h"

void otg_hc_rh_Init(otg_hc_rh_t *rh, otg_hc_t *hc);
int otg_hc_rh_Submit(otg_hc_rh_t *rh, stack_urb_t *urb);
void otg_hc_rh_Timeout(unsigned long data);
int otg_hc_rh_DoCommand(otg_hc_rh_t *rh, u16 typeReq, u16 wValue, u16 wIndex, char *buf, u16 wLength);

extern char otg_hc_rh_StringDescriptor[];

static inline otg_hc_t *
otg_hc_rh_GetHost(otg_hc_rh_t *rh)
{
	return rh->hc;
}

static inline otg_hc_rh_port_t *
otg_hc_rh_GetHNPPort(otg_hc_rh_t *rh)
{
	return &rh->ports[otg_hc_rh_port_HNPPort];
}

static inline otg_hc_rh_port_t *
otg_hc_rh_GetPort(otg_hc_rh_t *rh, int index)
{
	return &rh->ports[index];
}

static inline int
otg_hc_rh_GetAddress(otg_hc_rh_t *rh)
{
	return rh->address;
}

#include "otg_hc_rh_port.h"

static inline void
otg_hc_rh_Reset(otg_hc_rh_t *rh)
{
	int i;

	info("otg_hc_rh_Reset");

	rh->descriptorA = otg_hc_ReadReg(rh->hc, otg_hc_rh_DescriptorA_c);
	rh->descriptorB = otg_hc_ReadReg(rh->hc, otg_hc_rh_DescriptorB_c);
	rh->status = otg_hc_ReadReg(rh->hc, otg_hc_rh_Status_c);

	rh->num_ports = otg_hc_rh_MaxPorts;
	rh->enableSend = 0;
	rh->address = 0; /* Default address */
	rh->device_status = otg_hc_rh_DefaultState;

	for(i = 0; i < otg_hc_rh_MaxPorts; i++)
	{
		otg_hc_rh_port_Reset(&rh->ports[i]);
	}
}

static inline void
otg_hc_rh_DoStart(otg_hc_rh_t *rh)
{
	int i;

	otg_hc_WriteReg(rh->hc, otg_hc_rh_Status_c, otg_hc_rh_Status_LPSC_c);

	for(i = 0; i < otg_hc_rh_MaxPorts; i++)
	{
		otg_hc_rh_port_Enable(&rh->ports[i]);
	}
}

static inline void
otg_hc_rh_Stop(otg_hc_rh_t *rh)
{
	int i;

	for(i = 0; i < otg_hc_rh_MaxPorts; i++)
	{
		otg_hc_rh_port_Disable(&rh->ports[i]);
	}

	otg_hc_WriteReg(rh->hc, otg_hc_rh_Status_c, otg_hc_rh_Status_LPS_c);
}

static inline int
otg_hc_rh_GetStatus(otg_hc_rh_t *rh, stack_urb_t *urb, int type, int request)
{
	UINT16 data;

	if (!(type & otg_hc_rh_InputPipe))
	{
		return -EPIPE;
	}

	switch (type & otg_hc_rh_DeviceType)
	{
		case otg_hc_rh_Device:
			data = 0x01; break;
		case otg_hc_rh_Interface:
			data = 0x00; break;
		case otg_hc_rh_Endpoint:
			data = 0x00; break;
		default:
			return -EPIPE;
	}

	if (stack_urb_GetBufferLength(urb) < 2)
	{
		return -EINVAL;
	}

	memcpy(stack_urb_GetBuffer(urb), &data, 2);
	stack_urb_SetActualLength(urb, 2);

	return 0;
}

static inline int
otg_hc_rh_ClearFeature(otg_hc_rh_t *rh, int type, int value)
{
	UINT32 w;

	if (type & otg_hc_rh_InputPipe)
	{
		return -EPIPE;
	}

	switch (type & otg_hc_rh_DeviceType)
	{
		case otg_hc_rh_Device:
			if (value != 1)
			{
				return -EPIPE;
			}
			w = otg_hc_rh_Status_CRWE_c;
			break;
		case otg_hc_rh_Interface:
			return -EPIPE;
		case otg_hc_rh_Endpoint:
			if (value)
			{
				return -EPIPE;
			}
			w = 0;
			break;
		default:
			return -EPIPE;
	}

	otg_hc_WriteReg(rh->hc, otg_hc_rh_Status_c, w);
	return 0;
}

static inline int
otg_hc_rh_SetFeature(otg_hc_rh_t *rh, int type, int value)
{
	UINT32 w;

	if (type & otg_hc_rh_InputPipe)
	{
		return -EPIPE;
	}

	switch (type & otg_hc_rh_DeviceType)
	{
		case otg_hc_rh_Device:
			if (value != 1)
			{
				return -EPIPE;
			}
			w = otg_hc_rh_Status_DRWE_c;
			break;
		case otg_hc_rh_Interface:
			return -EPIPE;
		case otg_hc_rh_Endpoint:
			if (value)
			{
				return -EPIPE;
			}
			w = 0;
			break;
		default:
			return -EPIPE;
	}

	otg_hc_WriteReg(rh->hc, otg_hc_rh_Status_c, w);
	return 0;
}

static inline int
otg_hc_rh_SetAddress(otg_hc_rh_t *rh, int type, int value)
{
	if (type & otg_hc_rh_InputPipe)
	{
		return -EPIPE;
	}

	if ((type & otg_hc_rh_DeviceType) == otg_hc_rh_Device)
	{
		if (value == 0)
		{
			rh->device_status = otg_hc_rh_DefaultState;
		}
		else
		{
			rh->device_status = otg_hc_rh_AddressState;
		}

		rh->address = value;
		return 0;
	}

	return -EPIPE;
}

static inline int
otg_hc_rh_GetDescriptor(otg_hc_rh_t *rh, stack_urb_t *urb, int type, int value, int length)
{
	int len;
	void *dptr;

	if (!(type & otg_hc_rh_InputPipe))
	{
		return -EPIPE;
	}

	len = length;
	switch ((value >> 8) & 0xFF)
	{
		case 1: /* Device */
			if (len > otg_hc_rh_DeviceDecriptorLength)
			{
				len = otg_hc_rh_DeviceDecriptorLength;
			}

			dptr = &rh->deviceDescriptor;
			break;
		case 2: /* Configuration */
			if (len > (otg_hc_rh_ConfigDecriptorLength + 
				otg_hc_rh_InterfaceDecriptorLength +
				otg_hc_rh_EndpointDecriptorLength))
			{
				len = otg_hc_rh_ConfigDecriptorLength + 
					otg_hc_rh_InterfaceDecriptorLength +
					otg_hc_rh_EndpointDecriptorLength;
			}

			dptr = &rh->configDescriptor;
			break;
		case 3: /* String */
			if (len > otg_hc_rh_StringDescriptor[0])
			{
				len = otg_hc_rh_StringDescriptor[0];
			}

			dptr = otg_hc_rh_StringDescriptor;
			break;
		case 0x29: /* Hub descriptor */
/* TODO: Fix or remove */
/*			if (len > rh->hubDescriptor[0])
			{
				len = rh->hubDescriptor[0];
			}

			dptr = &rh->hubDescriptor;*/
			break;
		default:
			return -EPIPE;
	}

	memcpy(stack_urb_GetBuffer(urb), dptr, len);
	stack_urb_SetActualLength(urb, len);
	return 0;
}

static inline int
otg_hc_rh_SetDescriptor(otg_hc_rh_t *rh, stack_urb_t *urb, int type, int value, int length)
{
	int len;
	void *dptr;

	if (type)
	{
		return -EPIPE;
	}

	len = length;
	switch ((value >> 8) & 0xFF)
	{
		case 1: /* Device */
			if (len > otg_hc_rh_DeviceDecriptorLength)
			{
				len = otg_hc_rh_DeviceDecriptorLength;
			}

			dptr = &rh->deviceDescriptor;
			break;
		case 2: /* Configuration */
			if (len > (otg_hc_rh_ConfigDecriptorLength + 
				otg_hc_rh_InterfaceDecriptorLength +
				otg_hc_rh_EndpointDecriptorLength))
			{
				len = otg_hc_rh_ConfigDecriptorLength + 
					otg_hc_rh_InterfaceDecriptorLength +
					otg_hc_rh_EndpointDecriptorLength;
			}

			dptr = &rh->configDescriptor;
			break;
		case 3: /* String */
			if (len > otg_hc_rh_StringDescriptor[0])
			{
				len = otg_hc_rh_StringDescriptor[0];
			}

			dptr = otg_hc_rh_StringDescriptor;
			break;
		case 0x29: /* Hub descriptor */
/* TODO: Fix or remove that
			if (len > rh->hubDescriptor[0])
			{
				len = rh->hubDescriptor[0];
			}

			dptr = &rh->hubDescriptor;*/
			break;
		default:
			return -EPIPE;
	}

	memcpy(dptr, stack_urb_GetBuffer(urb), len);
	stack_urb_SetActualLength(urb, len);
	return 0;
}

static inline int
otg_hc_rh_GetConfig(otg_hc_rh_t *rh, stack_urb_t *urb, int type, int value)
{
	UINT8 data;

	switch (rh->device_status)
	{
		case otg_hc_rh_AddressState:
			data = 0;
			break;
		case otg_hc_rh_CofinguredState:
			data = rh->configDescriptor[5];
			break;
		default:
			data = 0;
			break;
	}

	*((UINT8 *)stack_urb_GetBuffer(urb)) = data;
	stack_urb_SetActualLength(urb, 1);
	return 0;
}

static inline int
otg_hc_rh_SetConfig(otg_hc_rh_t *rh, stack_urb_t *urb, int type, int value)
{
	UINT8 data;

	data = value & 0xFF;
	switch (rh->device_status)
	{
		case otg_hc_rh_AddressState:
		case otg_hc_rh_CofinguredState:
			if (!data)
			{
				rh->device_status = otg_hc_rh_AddressState;
			}
			else if (data == rh->configDescriptor[5])
			{
				rh->device_status = otg_hc_rh_CofinguredState;
			}
			else
			{
				return -EPIPE;
			}
			break;
		default:
			break;
	}

	return 0;
}

static inline int
otg_hc_rh_ClearHubFeature(otg_hc_rh_t *rh, int type, int value, int index)
{
	UINT32 w;

	if (type & otg_hc_rh_InputPipe)
	{
		return -EPIPE;
	}

	if (!(type & 0x03))
	{
		/* Hub feature, nothing to clear */
		return 0;
	}

	if (index < 0 || index > rh->num_ports)
	{
		return -EPIPE;
	}

	/* Clear port feature */
	w = 0;
	switch (value)
	{
		case otg_hc_rh_port_PORT_ENABLE:
			w = otg_hc_rh_port_Status_CCS_c; /* ClearPortEanble */
			break;
		case otg_hc_rh_port_PORT_SUSPEND:
			w = otg_hc_rh_port_Status_POCI_c; /* ClearPortSuspend */
			break;
		case otg_hc_rh_port_PORT_POWER:
			w = otg_hc_rh_port_Status_LSDA_c; /* ClearPortPower */
			break;
		case otg_hc_rh_port_C_PORT_CONNECTION:
			w = otg_hc_rh_port_Status_CSC_c;
			break;
		case otg_hc_rh_port_C_PORT_RESET:
			w = otg_hc_rh_port_Status_PRSC_c;
			break;
		case otg_hc_rh_port_C_PORT_ENABLE:
			w = otg_hc_rh_port_Status_PESC_c;
			break;
		case otg_hc_rh_port_C_PORT_SUSPEND:
			w = otg_hc_rh_port_Status_PSSC_c;
			break;
		case otg_hc_rh_port_C_PORT_OVER_CURRENT:
			w = otg_hc_rh_port_Status_PORTOCIC_c;
			break;
		default:
			return -EPIPE;
	}

	otg_hc_rh_port_SetStatus(&rh->ports[index - 1], w);
	return 0;
}

/* Shall we go somewhere with that???
static inline int
otg_hc_rh_UnlinkUrb(otg_hc_rh_t *rh, stack_urb_t *urb)
{
	if (rh->urb == urb)
	{
		rh->enableSend = 0;
		rh->urb = 0;

		OS_StopTimer(&rh->timer);
	}

	return 0;
}
*/

static inline void
otg_hc_rh_SetOtgPortSuspend(otg_hc_rh_t *rh)
{
	otg_hc_rh_port_SetStatus
	(
		&rh->ports[otg_hc_rh_port_HNPPort],
		otg_hc_rh_port_Status_PSS_c
	);
}

static inline void
otg_hc_rh_SetPortSuspend(otg_hc_rh_t *rh, otg_hc_rh_port_t *port)
{
	otg_hc_rh_port_SetStatus(port, otg_hc_rh_port_Status_PSS_c);
}

static inline void
otg_hc_rh_GoSuspend(otg_hc_rh_t *rh)
{
	int i;

	for(i = 0; i < otg_hc_rh_MaxPorts; i++)
	{
		otg_hc_rh_SetPortSuspend(rh, &rh->ports[i]);
	}
}

static inline void
otg_hc_rh_GoResume(otg_hc_rh_t *rh)
{
	int i;

	for(i = 0; i < otg_hc_rh_MaxPorts; i++)
	{
		otg_hc_rh_port_SetStatus(&rh->ports[i], otg_hc_rh_port_Status_POCI_c);
	}
}

#endif /* #ifndef OTG_HC_RH_H */
