/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_hc_rh_port.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    Root hub port object function definitions.
 */
#ifndef OTG_HC_RH_PORT_H
#define OTG_HC_RH_PORT_H

#include "otg_hc_rh_port_t.h"
#include "otg_hc_rh.h"

static inline int
otg_hc_rh_port_GetPortStatusReg(otg_hc_rh_port_t *port)
{
	int reg;

	reg = 0;
	switch (port->indexNo)
	{
		case 0:
			reg = otg_hc_rh_port_Status0_c;
			break;
		case 1:
			reg = otg_hc_rh_port_Status1_c;
			break;
		case 2:
			reg = otg_hc_rh_port_Status2_c;
			break;
		default:
			err("otg_hc_rh_port_GetPortStatusReg called with port %d", port->indexNo);
			return otg_hc_rh_port_Status2_c;
	}

	return reg;
}

static inline void
otg_hc_rh_port_ResetPort(otg_hc_rh_port_t *port)
{
	int reg;

#ifdef OTG_HNP_DEBUG
	info("ResetPort: ");
#endif
	reg = otg_hc_rh_port_GetPortStatusReg(port);
	otg_hc_WriteReg(port->hc, reg, otg_hc_rh_port_Status_PRS_c);
	port->downStream = 1;
}

static inline void
otg_hc_rh_port_Enable(otg_hc_rh_port_t *port)
{
	UINT32 w;
	int reg;

#ifdef OTG_HNP_DEBUG
	info("Enable: downStream=%d", port->downStream);
#endif
	w = otg_hc_rh_port_Status_PES_c | otg_hc_rh_port_Status_PPS_c;
	reg = otg_hc_rh_port_GetPortStatusReg(port);
	otg_hc_WriteReg(port->hc, reg, w);

	if (!port->downStream)
	{
		if (!(port->hnpPortStatus & otg_hc_rh_port_Status_PES_c))
		{
			port->hnpPortStatus |= otg_hc_rh_port_Status_PESC_c;
		}

		port->hnpPortStatus |= w;

#ifdef OTG_HNP_DEBUG
info("Enable: status=0x%0x", port->hnpPortStatus);
#endif
	}
}

static inline void
otg_hc_rh_port_Disable(otg_hc_rh_port_t *port)
{
	UINT32 w;
	int reg;

	w = otg_hc_rh_port_Status_CCS_c | otg_hc_rh_port_Status_LSDA_c;
	reg = otg_hc_rh_port_GetPortStatusReg(port);
	otg_hc_WriteReg(port->hc, reg, w);

	if (!port->downStream)
	{
		if (port->hnpPortStatus & otg_hc_rh_port_Status_PES_c)
		{
			port->hnpPortStatus |= otg_hc_rh_port_Status_PESC_c;
		}

		port->hnpPortStatus &=
			~(otg_hc_rh_port_Status_PES_c | otg_hc_rh_port_Status_PPS_c);
#ifdef OTG_HNP_DEBUG
info("Disable: status=0x%0x", port->hnpPortStatus);
#endif
	}
}

static inline void
otg_hc_rh_port_Init(otg_hc_rh_port_t *port, otg_hc_rh_t *rh, int no)
{
	port->indexNo = no;
	port->rh = rh;
	port->hc = otg_hc_rh_GetHost(rh);
	port->downStream = 0;
	port->hnpPortStatus = 0;
}

static inline void
otg_hc_rh_port_Reset(otg_hc_rh_port_t *port)
{
	port->downStream = 1;
/* NB: TODO: Check this, looks like it prevents things from working correctly...
#ifndef OTG_HOST_ONLY
	if (port->indexNo == otg_hc_rh_port_HNPPort)
	{
		port->downStream = 0;
		port->hnpPortStatus = 0;
	}
#endif
*/
	info("otg_hc_rh_port_Reset: port->downStream=%d", port->downStream);
}

static inline int
otg_hc_rh_port_IsLowSpeed(otg_hc_rh_port_t *port)
{
	int reg;
	UINT32 w;

	reg = otg_hc_rh_port_GetPortStatusReg(port);
	w = otg_hc_ReadReg(port->hc, reg);

	if (w & otg_hc_rh_port_Status_LSDA_c)
	{
		return 1;
	}

	return 0;
}

static inline int
otg_hc_rh_port_IsConnected(otg_hc_rh_port_t *port)
{
	int reg;
	UINT32 w;

	reg = otg_hc_rh_port_GetPortStatusReg(port);
	w = otg_hc_ReadReg(port->hc, reg);

	if (w & otg_hc_rh_port_Status_CCS_c)
	{
		return 1;
	}

	return 0;
}

static inline int
otg_hc_rh_port_IsChanged(otg_hc_rh_port_t *port)
{
	int reg;
	UINT32 w;

//	info("IsChanged: port->downStream=%d", port->downStream);

	if (!port->downStream)
	{
		w = port->hnpPortStatus;
	}
	else
	{
		reg = otg_hc_rh_port_GetPortStatusReg(port);
		w = otg_hc_ReadReg(port->hc, reg);
	}

	if (w & (otg_hc_rh_port_Status_CSC_c | otg_hc_rh_port_Status_PESC_c))
	{
		return 1;
	}

	return 0;
}

static inline int
otg_hc_rh_port_GetStatus(otg_hc_rh_port_t *port)
{
	int reg;
	//info("GetStatus: port->downStream=%d", port->downStream);

/* TODO: Check that carefully, looks like the !downStream mode is not working well */

	/* If the port is not a dawn stream port */
	if (port->downStream)
	{
		reg = otg_hc_rh_port_GetPortStatusReg(port);
		return otg_hc_ReadReg(port->hc, reg);
	}
	else
	{
		return port->hnpPortStatus;
	}
}

static inline void
otg_hc_rh_port_SetStatus(otg_hc_rh_port_t *port, UINT32 value)
{
	int reg;

	if (port->downStream)
	{
		reg = otg_hc_rh_port_GetPortStatusReg(port);
		otg_hc_WriteReg(port->hc, reg, value);
	}
	else
	{
#ifdef OTG_HNP_DEBUG
info("SetStatus: value=0x%0x status=0x%0x", value, port->hnpPortStatus);
#endif
		if (value & otg_hc_rh_port_Status_CCS_c)
		{
			if (port->hnpPortStatus & otg_hc_rh_port_Status_PES_c)
			{
		 		port->hnpPortStatus &= ~otg_hc_rh_port_Status_PES_c;
		 		port->hnpPortStatus |= otg_hc_rh_port_Status_PESC_c;
			}
		}

		if (value & otg_hc_rh_port_Status_PES_c)
		{
			if (!(port->hnpPortStatus & otg_hc_rh_port_Status_PES_c))
			{
		 		port->hnpPortStatus |= otg_hc_rh_port_Status_PES_c;
		 		port->hnpPortStatus |= otg_hc_rh_port_Status_PESC_c;
			}
		}

		if (value & otg_hc_rh_port_Status_LSDA_c)
		{
			if (port->hnpPortStatus & otg_hc_rh_port_Status_PPS_c)
			{
		 		port->hnpPortStatus &= ~otg_hc_rh_port_Status_PPS_c;
		 		port->hnpPortStatus |= otg_hc_rh_port_Status_PPS_c;
			}
		}

		if (value & otg_hc_rh_port_Status_PPS_c)
		{
			if (!(port->hnpPortStatus & otg_hc_rh_port_Status_PPS_c))
			{
		 		port->hnpPortStatus |= otg_hc_rh_port_Status_PPS_c;
		 		port->hnpPortStatus |= otg_hc_rh_port_Status_PPS_c;
			}
		}

		if (value & otg_hc_rh_port_Status_CSC_c)
		{
			port->hnpPortStatus &= ~otg_hc_rh_port_Status_CSC_c;
		}

		if (value & otg_hc_rh_port_Status_PESC_c)
		{
			port->hnpPortStatus &= ~otg_hc_rh_port_Status_PESC_c;
		}
#ifdef OTG_HNP_DEBUG
info("SetStatus: status=0x%0x", port->hnpPortStatus);
#endif
	}
}

static inline void
otg_hc_rh_port_EnableDownStream(otg_hc_rh_port_t *port)
{
#ifdef OTG_HNP_DEBUG
	info("EnableDownStream: port=%p", port);
#endif
	if (!port->downStream)
	{
		port->downStream = 1;
		port->hnpPortStatus |= otg_hc_rh_port_Status_PESC_c |
			otg_hc_rh_port_Status_PES_c;
	}
}

static inline void
otg_hc_rh_port_DisableDownStream(otg_hc_rh_port_t *port)
{
#ifdef OTG_HNP_DEBUG
	info("DisableDownStream: port=%p", port);
#endif
	if (port->downStream)
	{
		port->downStream = 0;
		port->hnpPortStatus = otg_hc_rh_port_Status_PES_c |
			otg_hc_rh_port_Status_PESC_c |
			otg_hc_rh_port_Status_PPS_c | otg_hc_rh_port_Status_CSC_c;
	}
}

#endif /* #ifndef OTG_HC_RH_PORT_H */
