/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_hc_rh_t.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    Root hub object data and macro definitions.
 */
#ifndef OTG_HC_RH_T_H
#define OTG_HC_RH_T_H

/* HcRhDescriptorA */

#define otg_hc_rh_Da_NDP_c			0x000000ff	/* NumberDownStreamPorts*/
#define otg_hc_rh_Da_PSM_c			0x00000100	/* PowerSwitchingMode*/
#define otg_hc_rh_Da_NPS_c			0x00000200	/* NoPowerSwitching*/
#define otg_hc_rh_Da_DeviceType_c	0x00000400  
#define otg_hc_rh_Da_OCPM_c			0x00000800	/* OverCurrentProtectionMode*/
#define otg_hc_rh_Da_NOCP_c			0x00001000	/* NoOverCurrentProtection */
#define otg_hc_rh_Da_POTPGT_c		0xff000000	/* PowerOnToPowerGoodTime*/

#define otg_hc_rh_Da_NDPLoc_c        0
#define otg_hc_rh_Da_PSMLoc_c        8
#define otg_hc_rh_Da_NPSLoc_c        9
#define otg_hc_rh_Da_DeviceTypeLoc_c 10
#define otg_hc_rh_Da_OCPMLoc_c       11
#define otg_hc_rh_Da_NOCPLoc_c       12
#define otg_hc_rh_Da_POTPGTLoc_c     24

/* HcRhDescriptorB Regster bitmap ( 1 dword) */
#define otg_hc_rh_Db_PPCMLoc_c	16

/* HcRhStatus */

#define otg_hc_rh_Status_LPS_c	0x00000001      /* LocalPowerStatus ; ClearGlobalPower*/
#define otg_hc_rh_Status_OCI_c	0x00000002      /* OverCurrentIndicator */
#define otg_hc_rh_Status_DRWE_c	0x00008000      /* DeviceRemoteWakeupEnable ; SetRemoteWakeupEnable*/
#define otg_hc_rh_Status_LPSC_c	0x00010000      /* LocalPowerStatusChange ; SetGlobalPower*/
#define otg_hc_rh_Status_OCIC_c	0x00020000      /* OverCurrentIndicatorChange*/
#define otg_hc_rh_Status_CRWE_c	0x80000000      /* ClearRemoteWakeupEnable*/

#define otg_hc_rh_Status_HCLPSLoc_c  0
#define otg_hc_rh_Status_OCILoc_c    1
#define otg_hc_rh_Status_DRWELoc_c   15
#define otg_hc_rh_Status_LPSCLoc_c   16
#define otg_hc_rh_Status_OCICLoc_c   17
#define otg_hc_rh_Status_CRWELoc_c   31

#define otg_hc_rh_DescriptorA_c	0x68     
#define otg_hc_rh_DescriptorB_c	0x6c    
#define otg_hc_rh_Status_c		0x70     

#define otg_hc_rh_DeviceDecriptorLength	18
#define otg_hc_rh_ConfigDecriptorLength	9
#define otg_hc_rh_InterfaceDecriptorLength	9
#define otg_hc_rh_EndpointDecriptorLength	7
#define otg_hc_rh_DecriptorLength	9

#define otg_hc_rh_USBReleaseNo	0x0110
#define otg_hc_rh_ReleaseNo	0x0100

#define otg_hc_rh_InputPipe		0x80
#define otg_hc_rh_Device		0x00
#define otg_hc_rh_Interface		0x01
#define otg_hc_rh_Endpoint		0x02
#define otg_hc_rh_DeviceType	0x03
#define otg_hc_rh_ClassRequest	0x20

#define otg_hc_rh_DefaultState		0
#define otg_hc_rh_AddressState		1
#define otg_hc_rh_CofinguredState	2

#define otg_hc_rh_MaxPorts	3

#include "uhcTypes.h"
#include "otg_hc_rh_port_t.h"

struct otg_hc;

struct otg_hc_rh
{
	UINT32 descriptorA;
	UINT32 descriptorB;
	UINT32 status;

	struct otg_hc_rh_port ports[otg_hc_rh_MaxPorts];
	int num_ports;
	int enableSend;
	stack_urb_t *urb;
	int interval;
	int address;
	int device_status;

	UINT8 deviceDescriptor[otg_hc_rh_DeviceDecriptorLength];
	UINT8 configDescriptor[otg_hc_rh_ConfigDecriptorLength];
	UINT8 interfaceDescriptor[otg_hc_rh_InterfaceDecriptorLength];
	UINT8 endpointDescriptor[otg_hc_rh_EndpointDecriptorLength];

	OS_DeclareTimer(timer);

	struct otg_hc *hc;
};

typedef struct otg_hc_rh otg_hc_rh_t;

#include "otg_hc_t.h"

#endif /* #ifndef OTG_HC_RH_T_H */
