/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_hc_t.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    Host controller object data and macro definitions.
 */
#ifndef OTG_HC_T_H
#define OTG_HC_T_H


/* Offsets of host controller registers, from HcRegBankStart_c 
 **************************************************************/

#define otg_hc_Revision_c			0x0      
#define otg_hc_Control_c			0x4     
#define otg_hc_CmdStatus_c			0x8      
#define otg_hc_AccessStartAddr_c	0x24    
#define otg_hc_DirectRouting_c		0x4c    
#define otg_hc_FmInterval_c			0x58     
#define otg_hc_FmRemaining_c		0x5c     
#define otg_hc_FmNumber_c			0x60    
#define otg_hc_LSThreshold_c		0x64     

/* HcRevision Register bitmap 
 *****************************/

#define otg_hc_RevisionMask_c	0x000000ff     /* Revsion number*/


/* HcControl Register bitmap
 ****************************/
#define otg_hc_CtlBulkRatio_c	0x00000003   /* Control Bulk Service Ratio */
#define otg_hc_HostState_c		0x0000000C   /* Host controller state */
#define otg_hc_RemoteWakeup_c	0x00000010   /* Remote Wakeup enable*/
 
/* HcControl Functional states 
 *****************************/
#define otg_hc_USBReset_c		0x00000000   /* Reset */
#define otg_hc_USBResume_c		0x00000004   /* Resume */
#define otg_hc_USBOperational_c	0x00000008   /* Operational */
#define otg_hc_USBSuspend_c		0x0000000C   /* Suspend */

/* HcCommandStatus 
 ******************/

#define otg_hc_CmdStatus_SwReset_c		0x00000001   /* sw initiated HC reset */
#define otg_hc_CmdStatus_SchedOverrun_c	0x000000C0   /* Scheduling Overrun */

/* HcAccessStartAddr register 
 ****************************/
#define otg_hc_StartData_c		0x00008000	/* 1: data phase */
#define otg_hc_StartEtd_c		0x00000000	/* 0: ED phase */
#define otg_hc_StartWrite_c		0x00004000	/* 1: write */
#define otg_hc_StartRead_c		0x00000000	/* 0: read */


/* HcEtdInterrupt Status, Enable and Disable Registers have the same bit assignments.
 * The low order two bytes are used for the X ETDs and the high order two bytes
 * are used for the Y ETDs.  HcXYBufferFill, set, and clear registers also have
 * the same bit definitions.
 ***********************************************************************************/

#define otg_hc_xBuffStart_c             0      
#define otg_hc_yBuffStart_c             16

/* HcDirectRouting register 
 ***************************/

#define otg_hc_RouteSinkEtd_c         0x0000000f
#define otg_hc_RouteSourceEtd_c       0x00000f00
#define otg_hc_RouteEnable_c          0x00008000

/* HcImmediateInterruptETDs and HcDoneQueue
 * - bit0-15 correspond to the etd number 
 *******************************************/

/* HcFmInterval 
 ***************/

#define otg_hc_FmIntervalMask_c		0x00003fff	/* frame interval mask */
#define otg_hc_FmInterval_FSMPS_c	0x7fff0000	/* FSLargestDataPacket */
#define otg_hc_FmInterval_FIT_c		0x80000000	/* FrameIntervalToggle */

/* HcFmRemaining 
 ****************/

#define otg_hc_FmRemainingMask_c	0x00003fff	/* frame remaining mask */
#define otg_hc_FmRemainingFRT_c		0x00008000	/* frame remaining toggle*/

/* HcFmNumber 
 *************/

#define otg_hc_FmMask_c	0x0000ffff         

/* HcLSThreshold */

#define otg_hc_LstMsk_c	0x00000fff        /* low speed threshold*/
#define otg_hc_MaxUrbCount	 128		/* Max urb on urb next list */

#include "uhcTypes.h"
#include "otg_hc_rh_t.h"
#include "otg_hc_intr_t.h"
#include "otg_hc_etd_t.h"

struct otg_hc_etd;
struct otg_mem;
struct otg;

struct otg_hc
{
	int started;
	UINT32 revision;
	UINT32 control;
	UINT32 commandStatus;
	UINT32 startAddress;
	int	frameNo;

	struct otg_hc_intr intr;
	struct otg_hc_rh rh;
	struct otg_hc_etd etd_bank;

	OS_ListDeclare(isoList);
	OS_ListDeclare(ctrlList);
	OS_ListDeclare(bulkList);

	/* Interrupt transfer needs a timer for checking pending
	 * ETDs. The intrInterval is the interval in ms and if
	 * it is zero, there is no timer. The intrPreviousFrame
	 * is used in case the checking point is passed when called.
	 */
	OS_ListDeclare(intrList[8]);
	OS_DeclareTimer(intrTimer);
	int intrPreviousFrame;
	int intrInterval;

#ifdef OTG_ISO_SHARE_MEMORY
	OS_ListDeclare(sofList);
#endif

	struct otg_mem mem;

	struct otg *otg;
};

typedef struct otg_hc otg_hc_t;

#include "otg_mem_t.h"

#endif /* #ifndef OTG_HC_T_H */
