/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_interface_desc.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    Interface descriptor object function definitions.
 */
#ifndef OTG_INTERFACE_DESC_H
#define OTG_INTERFACE_DESC_H

static inline int
otg_interface_desc_IsInterface(otg_interface_desc_t *intf, int index)
{
	return intf->desc[otg_interface_desc_bInterfaceNumber] == index;
}

static inline int
otg_interface_desc_IsInterfaceDesc(UINT8 *intf)
{
	return intf[otg_interface_desc_bDescriptorType] == otg_interface_desc_Type;
}

static inline int
otg_interface_desc_GetAltSetting(otg_interface_desc_t *intf)
{
	return intf->altSetting;
}

static inline UINT8 *
otg_interface_desc_GetDesc(otg_interface_desc_t *intf)
{
	return intf->desc;
}

static inline int
otg_interface_desc_GetLength(otg_interface_desc_t *intf)
{
	return intf->length;
}

static inline UINT8 *
otg_interface_desc_GetClassDesc(otg_interface_desc_t *intf, int type)
{
	int len;

	len = intf->desc[otg_interface_desc_bLength];
	if (type != intf->desc[len + otg_interface_desc_bDescriptorType])
	{
		return 0;
	}

	return &intf->desc[len];
}

static inline int
otg_interface_desc_GetClassDescLength(otg_interface_desc_t *intf, int type)
{
	int len;

	len = intf->desc[otg_interface_desc_bLength];
	if (type != intf->desc[len + otg_interface_desc_bDescriptorType])
	{
		return 0;
	}

	return intf->desc[len + otg_interface_desc_bLength];
}

static inline void
otg_interface_desc_SetAltSetting(otg_interface_desc_t *intf, int alt)
{
	if (alt < 0)
	{
		return;
	}

	if (intf->desc[otg_interface_desc_bAlternateSetting] <= alt)
	{
		return;
	}

	intf->altSetting = alt;
}

static inline int
otg_interface_desc_GetNumEndpoints(otg_interface_desc_t *intf)
{
	return intf->desc[otg_interface_desc_bNumEndpoints];
}

static inline otg_ep_desc_t *
otg_interface_desc_GetEndpoint(otg_interface_desc_t *intf, int index)
{
	int i;

	for(i = 0; i < intf->desc[otg_interface_desc_bNumEndpoints]; i++)
	{
		if (otg_ep_desc_IsEndpoint(&intf->epDesc[i], index))
		{
			return &intf->epDesc[i];
		}
	}

	return 0;
}

static inline int
otg_interface_desc_Set(otg_interface_desc_t *intf, otg_fc_t *fc)
{
	int i;
	int rc;

	for(i = 0; i < intf->desc[otg_interface_desc_bNumEndpoints]; i++)
	{
		rc = otg_ep_desc_Set(&intf->epDesc[i], fc);
		if (rc)
		{
			return rc;
		}
	}

	return 0;
}

static inline int
otg_interface_desc_SetDesc(otg_interface_desc_t *intf, UINT8 *desc, int len)
{
	UINT8 b;
	UINT8 eps;
	UINT8 size;
	int ep_len;

	b = desc[otg_interface_desc_bDescriptorType];
	if (b != otg_interface_desc_Type)
	{
		err("SetDesc: incorrect descriptor type %d", b);
		return -EINVAL;
	}

	size = desc[otg_interface_desc_bLength];
	if (size != otg_interface_desc_Max || size > len)
	{
		err("SetDesc: incorrect descriptor size %d", size);
		return -EINVAL;
	}

	if (!otg_ep_desc_IsEndpointDesc(&desc[size]))
	{
		size += desc[size];
	}

	intf->length = size;
	eps = 0;
	while (size < len)
	{
		if (eps >= otg_interface_desc_MaxEP)
		{
			err("SetDesc: bNumEndpoints %d is too many", eps);
			return -EINVAL;
		}

		if (!otg_ep_desc_IsEndpointDesc(&desc[size]))
		{
			break;
		}

		ep_len = otg_ep_desc_SetDesc(&intf->epDesc[eps],&desc[size],len-size);
		if (ep_len < 0)
		{
			return ep_len;
		}

		eps++;
		size += ep_len;
	}

	b = desc[otg_interface_desc_bNumEndpoints];
	if (eps != b)
	{
		err("SetDesc: bNumEndpoints %d is incorrect", b);
		return -EINVAL;
	}

	intf->desc = desc;

	return size;
}

#endif /* #ifndef OTG_INTERFACE_DESC_H */
