/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_intr.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    OTG243 chip level interrupt object function definitions.
 */
#ifndef OTG_INTR_H
#define OTG_INTR_H

#include "otg_intr_t.h"

static inline int
otg_intr_GetIntrNo(otg_intr_t *intr)
{
	return intr->intrNo;
}

static inline void
otg_intr_SetIntrNo(otg_intr_t *intr, int no)
{
	intr->intrNo = no;
}

static inline int
otg_intr_IsTriggered(otg_intr_t *intr, int num)
{
	return intr->status & num & intr->enable;
}

static inline int
otg_intr_Check(otg_intr_t *intr, int num)
{
	UINT32 w;

	w = otg_ReadChipReg(intr->otg, otg_intr_Status_c);
	if (w)
	{
		intr->status |= w;
		/* Do not clear host and function interrupt bits */
		w &= ~otg_intr_H_c & ~otg_intr_F_c;
		otg_WriteChipReg(intr->otg, otg_intr_Clear_c, w);
	}

	return otg_intr_IsTriggered(intr, num);
}

static inline void
otg_intr_ClearIntr(otg_intr_t *intr, int num)
{
	intr->status &= ~num;
}

static inline void
otg_intr_Clear(otg_intr_t *intr)
{
}

static inline void
otg_intr_Init(otg_intr_t *intr, otg_t *otg)
{
	intr->otg = otg;
	intr->intrNo = -1;
	intr->enable = 0;
}

static inline void
otg_intr_Reset(otg_intr_t *intr)
{
	UINT32 w;

	intr->status = 0x0000;

	/* Disable all interrupts and clear all previous interrupts */
	w = otg_intr_All_c;
	otg_WriteChipReg(intr->otg, otg_intr_Enable_c, 0x0);
	otg_WriteChipReg(intr->otg, otg_intr_Clear_c, w);
}

static inline void
otg_intr_Enable(otg_intr_t *intr, UINT32 bit)
{
#ifdef DEBUG_OTG_CHIP
otg_ReadChipReg(intr->otg, otg_intr_Enable_c);
#endif
	intr->enable |= bit;
	otg_WriteChipReg(intr->otg, otg_intr_Enable_c, intr->enable);
}

static inline void
otg_intr_EnableSelected(otg_intr_t *intr)
{
#ifdef DEBUG_OTG_CHIP
otg_ReadChipReg(intr->otg, otg_intr_Enable_c);
#endif
	otg_WriteChipReg(intr->otg, otg_intr_Enable_c, intr->enable);
}

static inline void
otg_intr_Disable(otg_intr_t *intr, UINT32 bit)
{
	UINT32 w;

	w = otg_ReadChipReg(intr->otg, otg_intr_Enable_c);
	intr->enable &= ~bit;
	w &= ~bit;
	otg_WriteChipReg(intr->otg, otg_intr_Enable_c, w);
}

static inline void
otg_intr_DisableIntr(otg_intr_t *intr, UINT32 bit)
{
	intr->enable &= ~bit;
}

static inline void
otg_intr_DisableAll(otg_intr_t *intr)
{
	otg_WriteChipReg(intr->otg, otg_intr_Enable_c, 0);
#ifdef DEBUG_OTG_CHIP
otg_ReadChipReg(intr->otg, otg_intr_Enable_c);
#endif
}

static inline void
otg_intr_EnableHostController(otg_intr_t *intr)
{
#ifndef OTG_FUNCTION_ONLY
	otg_intr_Enable(intr, otg_intr_H_c);
#endif
}

static inline void
otg_intr_EnableFunctionController(otg_intr_t *intr)
{
#ifndef OTG_HOST_ONLY
	otg_intr_Enable(intr, otg_intr_F_c);
#endif
}

static inline void
otg_intr_DisableFunctionController(otg_intr_t *intr)
{
#ifndef OTG_HOST_ONLY
	otg_intr_Disable(intr, otg_intr_F_c);
#endif
}

#endif /* #ifndef OTG_INTR_H */
