/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_linux.c
 * Author: Biao Jia		1/21/02
 * Description:
 *    OTG243 driver host interface to linux.
 */

#include <linux/platform_device.h>
#include <linux/usb.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/kernel.h>

#include "os.h"
#include "stack_urb.h"
#include "otg.h"
#include "otg_linux.h"

#define VERSTR "Revision: 0.95-2.6 $ time " __TIME__ " " __DATE__

#define DRIVER_AUTHOR "TransDimension Inc. & Nicolas Boichat"
#define DRIVER_DESC "TDI OTG243 Linux Host Controller Driver"

static const char hcd_name[] = "otg243";

static inline otg_hc_t *hcd_to_otg_hc(struct usb_hcd *hcd)
{
	return (otg_hc_t *) (hcd->hcd_priv);
}

static inline struct usb_hcd *otg_hc_to_hcd(otg_hc_t *otg_hc)
{
	return container_of((void *) otg_hc, struct usb_hcd, hcd_priv);
}

void otg_hcd_giveback_urb(otg_hc_t *hc, struct urb* urb) {
#ifdef DEBUG_OTG_OTHERS
	info("otg_hcd_giveback_urb");
#endif
	usb_hcd_giveback_urb(otg_hc_to_hcd(hc), urb);
}

static int otg_hcd_get_frame_number(struct usb_hcd *hcd)
{
#ifdef DEBUG_OTG_OTHERS
	info("otg_hcd_get_frame_number\n");
#endif
	return otg_hc_ReadFrameNo(hcd_to_otg_hc(hcd));
}

/* Was otg_hcd_Start */
static int otg_hcd_start(struct usb_hcd *hcd)
{
	otg_hc_t *otg_hc = hcd_to_otg_hc(hcd);

	hcd->state = HC_STATE_RUNNING;

	info("otg_hcd_start stub\n");
#if 0 // TODO: Do something!
	/* start it up */
	/* connect the virtual root hub */
	stack_device_t *usb_dev;

	usb_dev = usb_alloc_dev(NULL, s->bus);
	if (!usb_dev)
	{
		return -1;
	}

	s->bus->root_hub = usb_dev;
	usb_connect(usb_dev);

	if (usb_new_device(usb_dev) != 0)
	{
		usb_free_dev(usb_dev);
		return -1;
	}
#endif
	return 0;
}

/* Was otg_hcd_Remove */
static void otg_hcd_stop(struct usb_hcd *hcd)
{
	otg_hc_t *otg_hc = hcd_to_otg_hc(hcd);

	info("otg_hcd_stop stub\n");

#if 0 // TODO: Do something!
	stack_device_t *root_hub = s->bus->root_hub;

	if (root_hub)
	{
		usb_disconnect(&root_hub);
	}

	otg_hcd_ResetHc(s);

	usb_deregister_bus(s->bus);

	usb_free_bus(s->bus);
#endif
}

int otg_hcd_urb_enqueue (struct usb_hcd *hcd, struct usb_host_endpoint *ep,
			 struct urb *urb, gfp_t mem_flags) {
	otg_hc_t *otg_hc = hcd_to_otg_hc(hcd);
	int ret = 0;

#ifdef DEBUG_OTG_OTHERS
	info("otg_hcd_enqueue urb=%p ep=%p device_addr=%d", urb, ep, urb->dev->devnum);
#endif

	ret = otg_hc_Submit(otg_hc, urb);

#ifdef DEBUG_OTG_OTHERS
	info("otg_hcd_enqueue ret=%d", ret);
#endif

	return ret;
}

int otg_hcd_urb_dequeue (struct usb_hcd *hcd, struct urb *urb) {
	otg_hc_t *otg_hc = hcd_to_otg_hc(hcd);
	int ret;

#ifdef DEBUG_OTG_OTHERS
	info("otg_hcd_dequeue");
#endif

	ret = otg_hc_UnlinkUrb(otg_hc, urb);

#ifdef DEBUG_OTG_OTHERS
	info("otg_hcd_dequeue ret=%d", ret);
#endif

	return ret;
}

void otg_hcd_endpoint_disable(struct usb_hcd *hcd, struct usb_host_endpoint *ep) {
	info("otg_hcd_endpoint_disable stub");
#ifdef DEBUG_OTG_OTHERS
	WARN_ON(1);
#endif
}

/* NB: Copied from uhci-hub.c:get_hub_status_data */
int otg_hcd_status_data (struct usb_hcd *hcd, char *buf) {
	otg_hc_t *otg_hc = hcd_to_otg_hc(hcd);
	int port;

	/*info("Status ports : 1: 0x%x 2: 0x%x 3: 0x%x",
		otg_hc_rh_port_IsChanged(&otg_hc->rh.ports[0]),
		otg_hc_rh_port_IsChanged(&otg_hc->rh.ports[1]),
		otg_hc_rh_port_IsChanged(&otg_hc->rh.ports[2]));*/

	*buf = 0;
	for (port = 0; port < otg_hc_rh_MaxPorts; ++port) {
		if (otg_hc_rh_port_IsChanged(&otg_hc->rh.ports[port]))
			*buf |= (1 << (port + 1));
	}
	return !!*buf;
}

int otg_hcd_control (struct usb_hcd *hcd, u16 typeReq, u16 wValue,
			 u16 wIndex, char *buf, u16 wLength) {
	otg_hc_t       *otg_hc = hcd_to_otg_hc(hcd);

	return otg_hc_rh_DoCommand(&otg_hc->rh, typeReq, wValue, wIndex, buf, wLength);	
}

int otg_hcd_bus_suspend(struct usb_hcd *hcd) {
	info("otg_hcd_bus_suspend stub");
	return -1;
}

int otg_hcd_bus_resume(struct usb_hcd *hcd) {
	info("otg_hcd_bus_resume stub");
	return -1;
}

//TODO: Check this
static struct hc_driver otg_hc_driver = {
	.description =		"otg243-hcd",
	.product_desc = 	"OTG243",
	.hcd_priv_size =	sizeof(otg_hc_t),

	/*
	 * generic hardware linkage
	 */
	.flags =		HCD_USB11 | HCD_MEMORY,

	/* Basic lifecycle operations */
	.start =		otg_hcd_start,
	.stop =			otg_hcd_stop,

	/*
	 * managing i/o requests and associated device resources
	 */
	.urb_enqueue =		otg_hcd_urb_enqueue,
	.urb_dequeue =		otg_hcd_urb_dequeue,
	.endpoint_disable =	otg_hcd_endpoint_disable,

	/*
	 * periodic schedule support
	 */
	.get_frame_number =	otg_hcd_get_frame_number,

	/*
	 * root hub support
	 */
	.hub_status_data =	otg_hcd_status_data,
	.hub_control =		otg_hcd_control,

	.bus_suspend =		otg_hcd_bus_suspend,
	.bus_resume =		otg_hcd_bus_resume,
};

/* Was: otg_hc_init and otg_hc_Alloc */
static int __devinit
otg_probe(struct platform_device *dev)
{
	struct usb_hcd		*hcd;
	otg_hc_t		*otg_hc;
	int			retval;

	info("otg_probe");

	/* refuse to confuse usbcore */
	if (dev->dev.dma_mask) {
		err("no we won't dma\n");
		return -EINVAL;
	}

#ifdef CONFIG_PPC850_HANWANG
	ppc850_memory_control();
#endif

#ifdef SIM_OTG
	otg_chip_Init();
#endif /* #ifdef SIM_OTG */

#ifndef OTG_FUNCTION_ONLY
	/* allocate and initialize hcd */
	hcd = usb_create_hcd(&otg_hc_driver, &dev->dev, dev->dev.bus_id);
	if (!hcd) {
		err("usb_create_hcd failed");
		retval = -1;
		goto out;
	}
	
	otg_hc = hcd_to_otg_hc(hcd);
	memset(otg_hc, 0, sizeof(otg_hc));
       	
	retval = otg_hc_New(otg_hc);
	if(retval)
	{
		err("otg_hc_New failed");
		retval = -1;
		goto out_remove_hcd;
	}

	otg_hc_Reset(otg_hc);

	retval = otg_hc_Start(otg_hc);
	if (retval)
	{
		err("OTG start failed");
		goto out_delete_hc;
	}

	retval = usb_add_hcd(hcd, -1, 0); /* NB: IRQ are handled another way, maybe it could be integrated here... */
	if (retval) {
		err("usb_add_hcd failed");
		goto out_delete_hc;
	}

/* Done in otg_hcd_Start */
/*	if(otg_hcd_CreateRootHub(otg_hc) < 0)
	{
		retval = -1;
		err("OTG start failed");
		goto out_delete_hc;
	}*/
#endif //ifndef OTG_FUNCTION_ONLY
	
#ifdef CONFIG_PM
	if (!otg_pm_dev)
	{
		otg_pm_dev = pm_register(PM_SYS_DEV, 0, otg_linux_CallBackForPM);
	}
#endif

	info("USB OTG init successul.");
	return retval;

#ifndef OTG_FUNCTION_ONLY
out_delete_hc:
	otg_hc_Delete(otg_hc);
out_remove_hcd:
	usb_put_hcd(hcd);
out:
#ifdef SIM_OTG
	otg_chip_Exit();
#endif
	return retval;
#endif
}

static int
__devexit otg_remove(struct platform_device *dev) 
{
	struct usb_hcd	*hcd = platform_get_drvdata(dev); /* Set by usb_create_hcd */
	otg_hc_t	*otg_hc = hcd_to_otg_hc(hcd);

#ifdef CONFIG_PM
	if (otg_pm_dev)
	{
		pm_unregister(otg_pm_dev);
		otg_pm_dev = 0;
	}
#endif

#ifndef OTG_FUNCTION_ONLY
	otg_hcd_stop(hcd);

	usb_remove_hcd(hcd);

	usb_put_hcd(hcd);

	otg_hc_Delete(otg_hc);
#endif

#ifdef SIM_OTG
	otg_chip_Exit();
#endif
	return 0;
}

struct platform_driver otg_driver = {
	.probe =	otg_probe,
	.remove =	__devexit_p(otg_remove),

/*	.suspend =	otg_suspend,
	.resume =	otg_resume,*/
	.driver = {
		.name =	(char *) hcd_name,
		.owner = THIS_MODULE,
	},
};

/*-------------------------------------------------------------------------*/

static struct platform_device *pdev;

static int __init otg_init(void)
{
	int ret;

	if (usb_disabled())
		return -ENODEV;

	info(VERSTR);
	info(DRIVER_DESC);

	ret = platform_driver_register(&otg_driver);
	if (ret) {
		info("platform_driver_register error...");
		return ret;
	}

	pdev = platform_device_register_simple("otg243", 0, NULL, 0);

	if (IS_ERR(pdev)) {
		info("platform_device_register_simple error...");
		return PTR_ERR(pdev);
	}

	return 0;
}
module_init(otg_init);

static void __exit otg_cleanup(void)
{
	platform_device_unregister(pdev);
	platform_driver_unregister(&otg_driver);
}
module_exit(otg_cleanup);

MODULE_AUTHOR( DRIVER_AUTHOR );
MODULE_DESCRIPTION( DRIVER_DESC );
MODULE_LICENSE("GPL v2"); /* NB: Not sure about the legal implications of this. */
