#include <linux/autoconf.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/smp_lock.h>
#include <linux/errno.h>
#include <linux/unistd.h>
#include <linux/interrupt.h>
#include <linux/init.h>
#include <linux/version.h>
#include <linux/pm.h>

#include <linux/usb.h>
#include "../../core/hcd.h"

#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/system.h>

#ifdef CONFIG_PM
static struct pm_dev *otg_pm_dev = 0;
#endif

static struct platform_device *pdev;

static otg_hcd_t *hcd = NULL;

static int otg_hcd_UnlinkUrb(stack_urb_t *urb)
{
	otg_hcd_t *s;
#ifdef DEBUG_OTG_CHIP
info("otg_hcd_UnlinkUrb called for %p",urb);
#endif
	if (!urb || !urb->dev || !urb->dev->bus)
	{
		return -EINVAL;
	}
	
	s = (otg_hcd_t*)urb->dev->bus->hcpriv;

	return otg_hc_UnlinkUrb(s->hc, urb);
}

static int otg_hcd_AllocDev(stack_device_t *usb_dev)
{
	return 0;
}

static int otg_hcd_FreeDev(stack_device_t *usb_dev)
{
	otg_hcd_t *s;

	if(!usb_dev || !usb_dev->bus || !usb_dev->bus->hcpriv)
	{
		return -EINVAL;
	}
	
	s=(otg_hcd_t*) usb_dev->bus->hcpriv;	
	otg_hc_UnlinkAll(s->hc, usb_dev);

	return 0;
}

static int otg_hcd_SubmitUrb(stack_urb_t *urb)
{
	otg_hcd_t *s;

	if (!urb->dev || !urb->dev->bus)
	{
		return -ENODEV;
	}

	s = (otg_hcd_t *)urb->dev->bus->hcpriv;

	return otg_hc_Submit(s->hc, urb);
}

#if 0
struct usb_operations otg_hcd_DeviceOperations =
{
	otg_hcd_AllocDev,
	otg_hcd_FreeDev,
	otg_hcd_GetFrameNumber,
	otg_hcd_SubmitUrb,
	otg_hcd_UnlinkUrb
};
#endif

#ifdef CONFIG_PM
static int
otg_linux_CallBackForPM(struct pm_dev *pm_dev, pm_request_t req, void *data)
{
	otg_hc_t *hc;

	if (!hcd)
	{
		/* no host controller */
		return 0;
	}

	hc = hcd->hc;
	switch (req)
	{
		case PM_SUSPEND:
			info("OTG243 is going to suspend mode");
			otg_hc_GoSuspend(hc);
			break;
		case PM_RESUME:
			info("OTG243 is resumed");
			otg_hc_GoResume(hc);
			break;
/*
		case PM_STANDBY:
			info("OTG243 standby called");
			otg_hc_GoStandby(hc);
			break;
*/
		default:
			info("OTG243 power management called with %d", req);
			break;
	}

	return 0;
}
#endif

stack_bus_t *
otg_hcd_GetUsbBus(void)
{
	if (!hcd)
	{
		return 0;
	}

	return hcd->bus;
}

otg_hc_t *otg_hcd_GetHostController(void)
{
	if (!hcd)
	{
		return 0;
	}

	return hcd->hc;
}

#ifdef SIM_OTG
otg_t *otg_hcd_GetDevice(void)
{
	return hcd->hc->otg;
}

ISR otg_hcd_GetISR(void)
{
	return otg_DoIntr;
}
#endif
