/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_mem.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    OTG243 on chip memory object function definitions.
 */
#ifndef OTG_MEM_H
#define OTG_MEM_H

#include "otg_mem_t.h"
#include "otg_mem_block.h"

void otg_mem_Init(otg_mem_t *mem, int start, int size);
int otg_mem_Test(otg_t *otg);
int otg_mem_Allocate(otg_mem_t *mem, int size);
void otg_mem_Release(otg_mem_t *mem, int start);
void otg_mem_BulkWrite(otg_hc_t *hc, int addr, void *buffer, int size);
void otg_mem_BulkRead(otg_hc_t *hc, int addr, void *buffer, int size);

static inline int
otg_mem_ByteToBlock(int size)
{
	return (size + otg_mem_BlockSize_c - 1) >> otg_mem_BlockMagnitude_c;
}

static inline int
otg_mem_BlockToAddr(otg_mem_t *mem, int blocks)
{
	return mem->start + (blocks << otg_mem_BlockMagnitude_c);
}

static inline otg_mem_block_t *
otg_mem_AddrToBlock(otg_mem_t *mem, int start)
{
	return mem->mem_blocks +
		((start - mem->start) >> otg_mem_BlockMagnitude_c);
}

static inline int
otg_mem_GetStart(otg_mem_t *mem)
{
	return mem->start;
}

#include "otg_hc.h"
#include "otg_fc.h"

static inline void
otg_mem_FcWrite(otg_fc_t *fc, int addr, void *buffer, int size)
{
	otg_mem_BulkWrite
	(
		otg_GetHost(otg_fc_GetOtg(fc)),
		addr,
		buffer,
		size
	);
}

static inline void
otg_mem_FcRead(otg_fc_t *fc, int addr, void *buffer, int size)
{
	otg_mem_BulkRead
	(
		otg_GetHost(otg_fc_GetOtg(fc)),
		addr,
		buffer,
		size
	);
}
#endif /* #ifndef OTG_MEM_H */
