/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_mode.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    OTG243 mode object function definitions.
 */
#ifndef OTG_MODE_H
#define OTG_MODE_H

#include "otg_mode_t.h"
#include "otg.h"

static inline void
otg_mode_Init(otg_mode_t *mode, otg_t *otg)
{
	mode->otg = otg;
}

static inline void
otg_mode_Reset(otg_mode_t *mode)
{
	UINT32 w;

	w = 0;
#ifdef OTG_DMA_DACK_ONLY
	w |= otg_mode_DAKOLY_c;
#endif
#ifdef OTG_DMA_DACK_ACTIVE_HIGH
	w |= otg_mode_DAKPOL_c;
#endif
#ifdef OTG_DMA_DRQ_LEVEL_SENSITIVE
	w |= otg_mode_DREQT_c;
#endif
#ifdef OTG_DMA_DRQ_ACTIVE_HIGH
	w |= otg_mode_DRQPOL_c;
#endif
#ifdef OTG_DMA_EOT_ACTIVE_HIGH
	w |= otg_mode_EOTPOL_c;
#endif
#ifdef OTG_INT_ACTIVE_HIGH
	w |= otg_mode_INTPOL_c;
#endif
#ifdef OTG_VBUS_SOURCE_EXTERNAL
	w |= otg_mode_VBusExt_c;
#endif
#ifdef OTG_SRP_VBUS_PULSE
	w |= otg_mode_VBusPuls_c;
#else
	w |= otg_mode_DataPuls_c;
#endif
#ifdef OTG_EXTERNAL_PULLUP
	w |= otg_mode_ExtPU_c;
#endif
#ifdef OTG_HOST_ONLY
	w |= otg_mode_HFHost_c;
#else
#ifdef OTG_FUNCTION_ONLY
	w |= otg_mode_HFFunc_c;
#else
	w |= otg_mode_HFSwMNP_c;
#endif /* OTG_FUNCTION_ONLY */
#endif /* OTG_HOST_ONLY */
	mode->mode = w;
	otg_WriteChipReg(mode->otg, otg_mode_Reg_c, w);
}

static inline void
otg_mode_SetHardwareHNP(otg_mode_t *mode)
{
	mode->mode &= ~otg_mode_HF_c;
	mode->mode |= otg_mode_HFMNP_c;
	otg_WriteChipReg(mode->otg, otg_mode_Reg_c, mode->mode);
}

static inline void
otg_mode_SetSoftwareHNP(otg_mode_t *mode)
{
	mode->mode &= ~otg_mode_HF_c;
	mode->mode |= otg_mode_HFSwMNP_c;
	otg_WriteChipReg(mode->otg, otg_mode_Reg_c, mode->mode);
}

#endif /* #ifndef OTG_MODE_H */
