
#include "os.h"
#include "stack_urb.h"
#include "otg_linux.h"

#define PLX_VENDOR_ID 0x10B5
#define PLX_DEVICE_ID 0x9030
#define MAX_DEV 15
#define PCE243_BASE	0xEF100000

static int pce243_started = 0;
static int pce243_int = 0;
static int pce243_base = 0;

void HW_WriteOTG243Reg(unsigned int *BaseAddress,
                       unsigned int OffsetAddress,
                       unsigned int Value)
   {
   unsigned int *Address;

   Address = (unsigned int *)((unsigned char *) BaseAddress + OffsetAddress);
   *Address = Value;
   }


unsigned int HW_ReadOTG243Reg(unsigned int *BaseAddress,
                              unsigned int OffsetAddress)
   {
   unsigned int *Address;

   Address = (unsigned int *)((unsigned char *)BaseAddress + OffsetAddress);
   return(*Address);
   }


void OTG_PciSetup(void)
   {
   unsigned char BusNo;
   unsigned char FuncNo;
   unsigned int ConfigRegBase;
   unsigned int *BaseAddr;
#ifdef DEBUG_MAP1
   unsigned int *Temp;
   unsigned int ConfigTemp;
#endif
   int Status;
   unsigned int Index;

   if (pce243_started)
      {
      return;
      }

#ifdef CONFIG_PCI
   Status = PCIBIOS_DEVICE_NOT_FOUND;
   for (Index = 0; Index < MAX_DEV; Index++)
      {
      Status = OS_FindPciDevice(PLX_VENDOR_ID, PLX_DEVICE_ID,
                  Index, &BusNo, &FuncNo);
      if (Status == PCIBIOS_SUCCESSFUL)
         {
         break;
         }
      }
   
   if (Status != PCIBIOS_SUCCESSFUL)
   {
   	   printk("faile to find pci device\n");
       return;
   }

#ifdef DEBUG_MAP1
   printk("PCI BusNo:0x%x FuncNo:0x%x\n", BusNo, FuncNo);
#endif

   /* Read Base Address of PLX */
   Status = OS_PciConfigInLong(BusNo, FuncNo, 0x10, &ConfigRegBase); 

#ifdef DEBUG_MAP1
   for (Index = 0x0; Index < 0x40; Index += 0x4)
      {
      OS_PciConfigInLong(BusNo, FuncNo, Index,
               &ConfigTemp);
      printk("%x : 0x%08x\n", Index, ConfigTemp);
      } 
#endif

   /* Remap PLX Registers */
   BaseAddr = OS_Ioremap(ConfigRegBase, 0x2000);

   /* Debug: Print PCE243 Registers */
#ifdef DEBUG_MAP1
   for (Index = 0; Index < 0x80; Index += 0x4)
      {
      Temp = (unsigned int *)(((unsigned char *)BaseAddr) + Index);
      printk("0x%x : %p : 0x%08x\n", Index, Temp,  *Temp);
      } 
#endif
   /* Change wait states */
   /*
   HW_WriteOTG243Reg(BaseAddr, 0x2C, 0x00912040);
   HW_WriteOTG243Reg(BaseAddr, 0x18, 0);
   HW_WriteOTG243Reg(BaseAddr, 0x04, 0x0FF00000); 
   HW_WriteOTG243Reg(BaseAddr, 0x18, 1);
   HW_WriteOTG243Reg(BaseAddr, 0x2C, 0x40800000);
   HW_WriteOTG243Reg(BaseAddr, 0x3C, 0x0BFFFFC1);
   HW_WriteOTG243Reg(BaseAddr, 0x40, 0x00001001);
   HW_WriteOTG243Reg(BaseAddr, 0x4C, 0x00000041);
   HW_WriteOTG243Reg(BaseAddr, 0x50, 0x08700000);
   HW_WriteOTG243Reg(BaseAddr, 0x54, 0x00B49924);
   */

   /* Read Base Address of PCE243 */
   Status = OS_PciConfigInLong(BusNo, FuncNo, 0x1C, &ConfigRegBase);
   if (!ConfigRegBase)
   	  {
	  Status = OS_PciConfigOutLong(BusNo, FuncNo, 0x1C, PCE243_BASE);
      Status = OS_PciConfigInLong(BusNo, FuncNo, 0x1C, &ConfigRegBase);
   	  }

   pce243_base = ConfigRegBase;

#ifdef DEBUG_MAP1
   printk("PCE243 pce243_base:0x%x\n", pce243_base);
#endif

   Status = OS_PciConfigInLong(BusNo, FuncNo, 0x3C, &ConfigRegBase);
   pce243_int = ConfigRegBase & 0xFF;
#ifdef DEBUG_MAP1
   printk("PCE243 int:0x%x\n", pce243_int);
#endif

	pce243_started = 1;
#endif /* CONFIG_PCI */
   }

int OTG_PCE243_GetInt(void)
	{
	if (!pce243_started)
		{
		OTG_PciSetup();
		}

		return pce243_int;
	}

int OTG_PCE243_GetBase(void)
	{
	if (!pce243_started)
		{
		OTG_PciSetup();
		}

		return pce243_base;
	}

