/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_str_desc.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    String descriptor object function definitions.
 */
#ifndef OTG_STR_DESC_H
#define OTG_STR_DESC_H

#include "otg_fc_t.h"

static inline void
otg_str_desc_Init(otg_str_desc_t *str)
{
	str->length = 0;
	str->indexCount = 0;
}

static inline UINT8 *
otg_str_desc_GetDesc(otg_str_desc_t *str, int index)
{
	if (index >= str->indexCount)
	{
		return 0;
	}

	return str->descIndex[index];
}

static inline int
otg_str_desc_GetLength(otg_str_desc_t *str, int index)
{
	UINT8 *buf;

	if (index >= str->indexCount)
	{
		return 0;
	}

	buf = str->descIndex[index];
	return buf[otg_str_desc_bLength];
}

static inline int
otg_str_desc_SetDesc(otg_str_desc_t *str, UINT8 *desc, int len)
{
	UINT8 b;
	UINT8 *buf;

	if (!len)
	{
		/* No string descriptor, this is ok. */
		return 0;
	}

	if (len < 4)
	{
		return -EINVAL;
	}

	b = desc[otg_str_desc_bDescriptorType];
	if (otg_str_desc_Type != b)
	{
		err("SetDesc: wrong descriptor type %d", b);
		return -EINVAL;
	}

	buf = OS_Malloc(len);
	if (!buf)
	{
		return -ENOMEM;
	}

	b = desc[otg_str_desc_bLength];
	if (b < 4 || (b & 1) || b > len)
	{
		err("SetDesc: incorrect descriptor size %d", b);
		return -EINVAL;
	}

	str->length = b;
	memcpy(buf, desc, b);
	str->desc = buf;
	str->descIndex[0] = buf;

	str->indexCount = 1;
	while (str->length < len)
	{
		if (str->indexCount >= otg_str_desc_Max)
		{
			err("SetDesc: too many strings");
			goto err;
		}

		buf = &desc[str->length];
		b = buf[otg_str_desc_bDescriptorType];
		if (otg_str_desc_Type != b)
		{
			err("SetDesc: wrong descriptor type %d at location %d",
				b, str->length + otg_str_desc_bDescriptorType);
			goto err;
		}

		b = buf[otg_str_desc_bLength];
		if ((b & 1) || (b + str->length) > len)
		{
			err("SetDesc: incorrect descriptor size %d at location %d",
				b, str->length + otg_str_desc_bLength);
			goto err;
		}

		memcpy(&str->desc[str->length], &desc[str->length], b);
		str->descIndex[str->indexCount] = &str->desc[str->length];
		str->length += b;
		str->indexCount++;
	}

	return 0;

err:
	str->length = 0;
	str->indexCount = 0;
	OS_Free(str->desc);
	return -EINVAL;
}

#endif /* #ifndef OTG_STR_DESC_H */
