/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_t.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    OTG243 chip object data and micro definitions.
 */
#ifndef OTG_T_H
#define OTG_T_H

/*
 * Register offsets within chip level register bank 
 */
#define otg_ClockCtl_c		0x10	/* clock control */
#define otg_PIO_dataPort_c	0x2c	/* pio data port */
#define otg_FncRamStart_c	0x30	/* function controller RAM Start */
#define otg_IOReady_c		0x44	/* chip io ready */
#define otg_ShortFrame_c	0x50	/* chip io ready */
#define otg_Revision		0x7C	/* revision */

#define otg_IsRevisionMatch(r)		(r == 0x87)

#define otg_IOReady_NotReady	0x1

/* Error codes */

#define otg_NO_ERROR                    0
#define otg_CLEAR_FEATURE_FAILED       -1
#define otg_SET_FEATURE_FAILED         -2
#define otg_GET_CONFIGURATION_FAILED   -3
#define otg_SET_CONFIGURATION_FAILED   -4
#define otg_GET_DESCRIPTOR_FAILED      -5
#define otg_GET_STATUS_FAILED          -6
#define otg_SET_ROOT_ADDRESS_FAILED    -7
#define otg_OUT_OF_DM                  -8
#define otg_OUT_OF_ETDS                -9
#define otg_INVALID_UH_ADDR            -10
#define otg_CANNOT_SELECT_TRANSACTION  -11
#define otg_CANNOT_ACTIVATE_BATCH      -12
#define otg_TRANSACTION_NOT_DONE       -13
#define otg_INCORRECT_STATE            -14
#define otg_BATCH_STILL_ON             -15
#define otg_INVALID_TOGGLE_STATUS      -16
#define otg_STALL_STATUS               -17
#define otg_ERROR_STATUS               -18
#define otg_OVERFLOW_STATUS            -19
#define otg_TIMEOUT_STATUS             -20
#define otg_NAK_STATUS                 -21
#define otg_BATCH_NOT_COMPLETE         -22
#define otg_NO_TRANSACTION_SELECTED    -23
#define otg_PORT_ENABLE_FAILED         -24
#define otg_PORT_DISABLE_FAILED        -25
#define otg_PORT_SUSPEND_FAILED        -26
#define otg_PORT_RESUME_FAILED         -27
#define otg_PORT_RESET_FAILED          -28
#define otg_PORT_CONNECTION_FAILED     -29
#define otg_SET_DESCRIPTOR_FAILED      -30
#define otg_PORT_POWER_FAILED          -31
#define otg_PORT_FEATURE_FAILED        -32

#define otg_MAX_ERROR                  33 

#define otg_RegBankStart_c		0x000 /* chip level register bank offset */
#define otg_HcRegBankStart_c	0x080 /* Host controller register bank */
#define otg_FcRegBankStart_c	0x100 /* Function register bank */

#define otg_MemMapSize_c		0x200 /* Size of register and dma mapping */

#include "otg_mode_t.h"
#include "otg_dma_t.h"
#include "otg_intr_t.h"
#include "otg_mem_t.h"
#include "otg_io_config_t.h"
#include "otg_hc_etd_t.h"
#include "otg_hc_t.h"
#include "otg_fc_t.h"
#include "otg_device_t.h"

struct otg
{
	void *mem_map;
	spinlock_t chip_lock;

	otg_mode_t mode;
	otg_dma_t dma[2];
	otg_intr_t intr;
	otg_io_config_t ioConfig[2];

#ifndef OTG_FUNCTION_ONLY
	otg_hc_t* hc; //NB: Set that as pointer, as otg_hc_t is now embedded in Linux hcd structure
#endif

#ifndef OTG_HOST_ONLY
	otg_fc_t fc;
#endif

#ifndef OTG_FUNCTION_ONLY
#ifndef OTG_HOST_ONLY
	otg_device_t device;
#endif
#endif

	int sofCount;
};

typedef struct otg otg_t;

#endif /* #ifndef OTG_T_H */
