/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_td_ctrl.c
 * Author: Biao Jia		1/21/02
 * Description:
 *    OTG243 TD for ctrl/int/bulk object handler.
 */
#include "os.h"
#include "stack_urb.h"
#include "otg.h"

int
otg_td_ctrl_FillPacketWord(otg_etd_t *etd)
{
	stack_urb_t *urb;
	int last_size;
	int bytes_send;
	stack_pipe_t pipe;

	urb = etd->urb;
	pipe = stack_urb_GetPipe(urb);

	bytes_send = etd->dataLength;
	last_size = bytes_send % etd->bufferSize;
	if (!last_size)
	{
		last_size = etd->bufferSize;
	}
#ifdef DEBUG_OTG_SUBMIT
info("FillPacketWord: bytes_send=%d packets=%d last_size=%d etd->bufferSize=%d",
	bytes_send, etd->totalPackets, last_size, etd->bufferSize);
#endif

	/* 0 in otg_td_ctrl_NumPacketsPerBuffLoc_c means 1 packet per buffer */
	etd->td.ctrl.info = (last_size << otg_td_ctrl_LastPacketSizeLoc_c) |
				etd->totalPackets;

	etd->td.ctrl.ctrl = otg_etd_CC_NotAccessed_c;
#ifndef DEBUG_SHORT_PKG
	/* NB: Short packet is NOT an error (and blocks the chip), we don't care
	 *     about IsShortPacketAllowed... Check DoDoneEtd too.
	 */
//	if (stack_urb_IsShortPacketAllowed(urb))
//	{
	etd->td.ctrl.ctrl |= otg_td_ctrl_Rounding_c;
//	}
#endif

	if (otg_etd_IsTransferType(etd, otg_etd_Format_Intr_c))
	{
		int start;

		start = otg_hc_GetStartFrameNo(etd->hc, stack_urb_GetIntrInterval(urb));
		start <<= otg_td_ctrl_RelPollingPosLoc_c;
		start &= otg_td_ctrl_RelPollingPos_c;

		etd->td.ctrl.ctrl |= start |
			((stack_urb_GetIntrInterval(urb) - 1) & otg_td_ctrl_PollingInterval_c);
#ifdef DEBUG_OTG_SUBMIT
info("FillPacketWord: frameNo=0x%0x interval=%d",
	etd->hc->frameNo, stack_urb_GetIntrInterval(urb));
#endif
	}

	if (stack_pipe_IsIn(pipe))
	{
		etd->td.ctrl.ctrl |= otg_td_ctrl_Pid_In_c;
	}
	else
	{
		etd->td.ctrl.ctrl |= otg_td_ctrl_Pid_Out_c;
	}

	if (stack_urb_GetToggle(urb, pipe))
	{
		etd->endpoint |= otg_etd_ToggleCarry_c;
		etd->td.ctrl.ctrl |= otg_td_ctrl_DataToggle1_c;
	}
	else
	{
		etd->endpoint &= ~otg_etd_ToggleCarry_c;
		etd->td.ctrl.ctrl |= otg_td_ctrl_DataToggle0_c;
	}

#ifdef DEBUG_BULK
	if (otg_etd_IsTransferType(etd, otg_etd_Format_Bulk_c))
	{
		etd->endpoint |= otg_etd_StopOnNak_c;
	}
#endif

	return 0;
}

int
otg_td_intr_Fill(otg_etd_t *etd, otg_hc_t *hc)
{
	stack_urb_t *urb;

	urb = etd->urb;
	stack_urb_SetErrorCount(urb, 0);
	stack_urb_SetActualLength(urb, 0);
	stack_urb_SetStatus(urb, -EINPROGRESS);

	otg_td_ctrl_InitEtd(etd, urb);

	return otg_td_ctrl_FillPacketWord(etd);
}

int
otg_td_ctrl_Fill(otg_etd_t *etd, otg_hc_t *hc)
{
	stack_urb_t *urb;
	otg_etd_t *status;
	otg_etd_t *data1;

	urb = etd->urb;
	stack_urb_SetErrorCount(urb, 0);
	stack_urb_SetActualLength(urb, 0);
	stack_urb_SetStatus(urb, -EINPROGRESS);

	otg_td_ctrl_InitEtd(etd, urb);
	etd->td.ctrl.ctrl |= otg_td_ctrl_Pid_Setup_c;

	etd->dataLength = 8;
	etd->data = stack_urb_GetSetup(urb);
	etd->td.ctrl.ctrl = otg_etd_CC_NotAccessed_c | otg_td_ctrl_Rounding_c |
		otg_td_ctrl_DataToggle0_c | otg_td_ctrl_Pid_Setup_c;

	etd->td.ctrl.info = (8 << otg_td_ctrl_LastPacketSizeLoc_c) | 1;

	/* Control transfer may have three stages. Each need a ETD */
	if (!stack_urb_IsOneStepSetup())
	{
		return 0;
	}

	data1 = 0;
	if (stack_urb_GetBufferLength(etd->urb) > 0)
	{
		data1 = otg_etd_New(hc);
		if (!data1)
		{
			return -ENOMEM;
		}

		data1->dataLength = stack_urb_GetBufferLength(urb);
		data1->data = stack_urb_GetBuffer(urb);

		otg_etd_SetCtrlPipe(data1, etd->urb);
		otg_td_ctrl_InitEtd(data1, urb);
		otg_td_ctrl_FillPacketWord(data1);

		data1->td.ctrl.ctrl &= ~otg_td_ctrl_DataToggle_c;
		data1->td.ctrl.ctrl |= otg_td_ctrl_DataToggle1_c;
		OS_ListAddTail(&data1->pipeEtdList, &etd->pipeEtdList);
	}


	status = otg_etd_New(hc);
	if (!status)
	{
		if (data1)
		{
			otg_etd_Delete(data1);
		}

		return -ENOMEM;
	}

	otg_etd_SetCtrlPipe(status, etd->urb);
	otg_td_ctrl_InitEtd(status, urb);

	status->setupStatus = 1;
	status->dataLength = 0;
	status->data = 0;

	status->td.ctrl.ctrl = otg_etd_CC_NotAccessed_c | otg_td_ctrl_Rounding_c |
		otg_td_ctrl_DataToggle1_c;
	status->td.ctrl.ctrl &= ~otg_etd_Direction_c;
	if (stack_pipe_IsIn(stack_urb_GetPipe(urb)))
	{
		status->td.ctrl.ctrl |= otg_td_ctrl_Pid_Out_c;
	}
	else
	{
		status->td.ctrl.ctrl |= otg_td_ctrl_Pid_In_c;
	}

	status->td.ctrl.info = 1;
	OS_ListAddTail(&status->pipeEtdList, &etd->pipeEtdList);
#ifdef DEBUG_OTG_SUBMIT
info("CtrlFill: setup");
otg_etd_PrintEtdWord(etd);
if (data1)
{
	info("CtrlFill: data");
	otg_etd_PrintEtdWord(data1);
}
info("CtrlFill: status");
otg_etd_PrintEtdWord(status);
#endif

	return 0;
}

void
otg_td_ctrl_Delete(otg_etd_t *etd)
{
	otg_etd_t *e1;
	otg_etd_t *e2;

	e1 = otg_etd_GetNext(etd);
	while (e1 != etd && e1)
	{
		e2 = otg_etd_GetNext(e1);
		if (otg_etd_IsInHc(e1))
		{
			otg_etd_MarkUnlink(e1);
		}
		else
		{
			otg_etd_Delete(e1);
		}

		e1 = e2;
	}
}

void
otg_td_ctrl_NextPacket(otg_etd_t *etd, otg_hc_etd_t *hc_etd)
{
	int hc_addr;
	int length;
	int yBuffer;
	void *addr;

	yBuffer = otg_td_ctrl_GetBufferIndex(etd);
	if (!otg_etd_HasMorePacket(etd) || otg_etd_IsUnlink(etd))
	{
		/* Send or receive zero length package */
		if (otg_etd_IsInEtd(etd))
		{
			otg_hc_etd_ClearFill(hc_etd, etd->etdIndex, yBuffer);
		}
		else
		{
			otg_hc_etd_SetFill(hc_etd, etd->etdIndex, yBuffer);
		}

		return;
	}

	hc_addr = otg_etd_GetBuffer(etd, yBuffer);
	length = otg_td_ctrl_GetPacketSize(etd);
	addr = otg_td_ctrl_GetPacketAddr(etd);
	otg_td_ctrl_IncCompletePacket(etd);

#ifdef DEBUG_OTG_SUBMIT
info("NextPacket: etd=%p yBuffer=%d hc_addr=0x%0x length=0x%0x addr=%p",
	etd, yBuffer, hc_addr, length, addr);
#endif
	if (otg_etd_IsInEtd(etd))
	{
		otg_mem_BulkRead(etd->hc, hc_addr, addr, length);
		otg_hc_etd_ClearFill(hc_etd, etd->etdIndex, yBuffer);
	}
	else
	{
		otg_mem_BulkWrite(etd->hc, hc_addr, addr, length);
		otg_hc_etd_SetFill(hc_etd, etd->etdIndex, yBuffer);
	}
#ifdef DEBUG_OTG_SUBMIT
info("NextPacket: FilledStatus=0x%0x", otg_hc_ReadReg(etd->hc, otg_hc_etd_FilledStatus_c));
#endif
}

int
otg_td_ctrl_DoDoneEtd(otg_etd_t *etd)
{
	int cc;
	stack_urb_t *urb;
	int toggle;

	cc = otg_td_ctrl_GetCompleteCode(etd);
	urb = etd->urb;
#ifdef DEBUG_OTG_SUBMIT
info("DoDoneEtd: cc=0x%0x", cc);
#endif
	switch (cc)
	{
		case otg_etd_CC_NoError_c:
		case otg_etd_CC_ACK:
			if (otg_etd_IsTransferType(etd, otg_etd_Format_Ctrl_c))
			{
				if (otg_td_ctrl_IsSetupPid(etd))
				{
					/* Setup stage finished. Nothing to do */
					stack_urb_SetActualLength(urb, 0); /* in case there is no data */
#ifdef DEBUG_OTG_OTHERS
					info("DoDoneEtd: Setup stage finished. Nothing to do");
#endif
					break;
				}

				if (etd->setupStatus)
				{
					/* Setup status stage */
					stack_urb_SetStatus(urb, 0);
#ifdef DEBUG_OTG_OTHERS
					info("DoDoneEtd: Setup status stage");
#endif
					break;
				}
				else
				{
					/* Setup data stage */
#ifdef DEBUG_OTG_OTHERS
					info("DoDoneEtd: Setup data stage");
#endif
				}
			}
			else
			{
				stack_urb_SetStatus(urb, 0);
			}

#ifdef DEBUG_OTG_OTHERS
			info("DoDoneEtd: SetActualLength to %d", otg_td_ctrl_GetActualLength(etd));
#endif
			stack_urb_SetActualLength(urb, otg_td_ctrl_GetActualLength(etd));
#ifdef DEBUG_OTG_SUBMIT
info("DoDoneEtd: actual_length=%d", stack_urb_GetActualLength(urb));
#endif
			break;
		case otg_etd_CC_CRC_c:
			stack_urb_SetStatus(urb, -EILSEQ);
			break;
		case otg_etd_CC_BitStuffing_c:
		case otg_etd_CC_DataToggleMismatch_c:
			stack_urb_SetStatus(urb, -EPROTO);
			break;
		case otg_etd_CC_Stall_c:
			stack_urb_SetStatus(urb, -EPIPE);
			stack_urb_Halt(urb);
			break;
		case otg_etd_CC_DeviceNotResponding_c:
			stack_urb_SetStatus(urb, -ETIMEDOUT);
			stack_urb_AddFlagTimeout(urb);
			break;
		case otg_etd_CC_PidFailure_c:
			stack_urb_SetStatus(urb, -EPROTO);
			break;
		case otg_etd_CC_DataOverRun_c:
			stack_urb_SetStatus(urb, -EOVERFLOW);
			break;
		case otg_etd_CC_DataUnderRun_c:
/* NB: (p. 4-4 of the OTD243 datasheet)
 * There are two types of sequence errors:
 * DATAOVERRUN and DATAUNDERRUN. When either of these error conditions is
 * encountered the CompletionCode field is set accordingly, the TD is retired,
 * and the endpoint is halted.
 * There might be a way of putting back the controller to a reasonable state
 * after such an error, but I didn't find it.
 * What we do is simply to avoid this error by allowing short packets...
 */
			err("Data underrun occured");
			stack_urb_SetActualLength(urb, otg_td_ctrl_GetActualLength(etd));
#ifdef DEBUG_OTG_SUBMIT
info("DoDoneEtd: actual_length=%d", stack_urb_GetActualLength(urb));
#endif
//			stack_urb_SetStatus(urb, 0); /* NB: underrun is ok??? */
			//break; /* NB: weirdly placed break, was there before... */
			stack_urb_SetStatus(urb, -EREMOTEIO);
			break;
		case otg_etd_CC_BufferOverRun_c:
			stack_urb_SetStatus(urb, -ECOMM);
			break;
		case otg_etd_CC_BufferUnderRun_c:
			stack_urb_SetStatus(urb, -ENOSR);
			break;
		case otg_etd_CC_NotAccessed_c:
			stack_urb_SetStatus(urb, -ETIMEDOUT);
			break;
		case otg_etd_CC_NAK:
			stack_urb_SetStatus(urb, -EAGAIN);
			break;
	}

	if (etd->endpoint & otg_etd_Halted_c)
	{
		etd->endpoint &= ~otg_etd_Halted_c;
	}

	if (etd->endpoint & otg_etd_ToggleCarry_c)
	{
		toggle = 1;
	}
	else
	{
		toggle = 0;
	}

#ifdef DEBUG_OTG_SUBMIT
info("DoDoneEtd: ready to set toggle %d", toggle);
#endif

	stack_urb_SetToggle(urb, toggle);

#ifdef DEBUG_OTG_SUBMIT
	otg_etd_PrintErrorCode(urb);
#endif
	return stack_urb_GetStatus(urb);
}
