/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_td_ctrl.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    OTG243 TD for ctrl/int/bulk object function definitions.
 */
#ifndef OTG_TD_CTRL_H
#define OTG_TD_CTRL_H

#include "otg_td_ctrl_t.h"

int otg_td_ctrl_FillPacketWord(otg_etd_t *etd);
int otg_td_ctrl_Fill(otg_etd_t *etd, otg_hc_t *hc);
int otg_td_intr_Fill(otg_etd_t *etd, otg_hc_t *hc);
void otg_td_ctrl_NextPacket(otg_etd_t *etd, otg_hc_etd_t *hc_etd);
int otg_td_ctrl_DoDoneEtd(otg_etd_t *etd);
void otg_td_ctrl_Delete(otg_etd_t *etd);

static inline int
otg_td_ctrl_GetCompleteCode(otg_etd_t *etd)
{
	return etd->td.ctrl.ctrl & otg_td_ctrl_CompletionCode_c;
}

static inline int
otg_td_ctrl_GetPacketCount(otg_etd_t *etd)
{
	return etd->td.ctrl.info & otg_td_ctrl_PacketCount_c;
}

static inline int
otg_td_ctrl_GetLastPacketSize(otg_etd_t *etd)
{
	return (etd->td.ctrl.info & otg_td_ctrl_LastPacketSize_c) >>
		otg_td_ctrl_LastPacketSizeLoc_c;
}

static inline int
otg_td_ctrl_GetActualLength(otg_etd_t *etd)
{
	return (etd->totalPackets - otg_td_ctrl_GetPacketCount(etd) - 1) *
		etd->bufferSize + otg_td_ctrl_GetLastPacketSize(etd);
}

static inline void
otg_td_ctrl_SetEtdFromHc(otg_etd_t *etd, int index, UINT32 word)
{
	UINT32 w;

	if (index == 2)
	{
		w = otg_td_ctrl_ErrorCount_c | otg_td_ctrl_CompletionCode_c;
		word &= w;
		etd->td.ctrl.ctrl &= ~w;
		etd->td.ctrl.ctrl |= word;
	}
	else if (index == 3)
	{
		etd->td.ctrl.info = word;
	}
}

static inline int
otg_td_ctrl_IsSetupPid(otg_etd_t *etd)
{
	if ((etd->td.ctrl.ctrl & otg_td_ctrl_Pid_c) == otg_td_ctrl_Pid_Setup_c)
	{
		return 1;
	}

	return 0;
}

static inline int
otg_td_ctrl_NeedYBuffer(otg_etd_t *etd)
{
	return etd->totalPackets > 1;
}

static inline int
otg_td_ctrl_GetPacketSize(otg_etd_t *etd)
{
	int size;

	size = etd->dataLength - etd->completed;
	if (size > etd->bufferSize)
	{
		size = etd->bufferSize;
	}

	return size;
}

static inline void *
otg_td_ctrl_GetPacketAddr(otg_etd_t *etd)
{
	return etd->data + etd->completed;
}

static inline void
otg_td_ctrl_IncCompletePacket(otg_etd_t *etd)
{
    etd->completed += etd->bufferSize;

	if (etd->completed > etd->dataLength)
	{
		etd->completed = etd->dataLength;
	}
}

static inline int
otg_td_ctrl_GetBufferIndex(otg_etd_t *etd)
{
#ifdef DEBUG_OTG_SUBMIT
info("GetBufferIndex: completed=%d bufferSize=%d",
	etd->completed, etd->bufferSize);
info("GetBufferIndex: %d", ((int)(etd->completed / etd->bufferSize)) & 1);
#endif
	return ((int)(etd->completed / etd->bufferSize)) & 1;
}

static inline int
otg_td_ctrl_IsInTd(otg_etd_t *etd)
{
	if ((etd->td.ctrl.ctrl & otg_td_ctrl_Pid_c) == otg_td_ctrl_Pid_In_c)
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

static inline int
otg_td_bulk_Fill(otg_etd_t *etd, otg_hc_t *hc)
{
	return otg_td_intr_Fill(etd, hc);
}

static inline void
otg_td_ctrl_InitEtd(otg_etd_t *etd, struct urb *urb)
{
	etd->completed = 0;
	etd->lastCompleted = 0;
	etd->bufferSize = stack_urb_GetMaxPacketSize(urb, stack_urb_GetPipe(urb));
	etd->totalPackets = (etd->dataLength + etd->bufferSize - 1)
						/ etd->bufferSize;
}

#endif /* #ifndef OTG_TD_CTR_H */
