/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_td_iso.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    OTG243 TD for iso object function definitions.
 */
#ifndef OTG_TD_ISO_H
#define OTG_TD_ISO_H

#include "otg_td_iso_t.h"

void otg_td_iso_FillPacketWord(otg_etd_t *etd);
int otg_td_iso_Fill(otg_etd_t *etd, otg_hc_t *hc);
int otg_td_iso_DoDoneEtd(otg_etd_t *etd);
void otg_td_iso_NextPacket(otg_etd_t *etd, otg_hc_etd_t *hc_etd);

#ifdef OTG_ISO_SHARE_MEMORY
int otg_td_iso_DoPreDownload(otg_etd_t *etd, otg_hc_etd_t *hc_etd);
int otg_td_iso_CheckActiveFrame(otg_etd_t *etd);
#endif

static inline void
otg_td_iso_SetEtdFromHc(otg_etd_t *etd, int index, UINT32 word)
{
	if (index == 2)
	{
		UINT32 w;

		w = otg_td_iso_LastCompleteCode_c;
		word &= w;
		etd->td.iso.ctrl &= ~w;
		etd->td.iso.ctrl |= word;
	}
	else if (index > 2 && index < 7)
	{
		etd->td.iso.data[index - 3] = word;
	}
}

static inline int
otg_td_iso_GetCompleteCode(otg_etd_t *etd, int index)
{
	int w;

	if (index & 1)
	{
		w = etd->td.iso.data[index >> 1] >> otg_td_iso_UpperDwordLoc_c;
	}
	else
	{
		w = etd->td.iso.data[index >> 1];
	}

	return w & otg_td_iso_CompletionCode_c;
}

static inline int
otg_td_iso_GetLength(otg_etd_t *etd, int index)
{
	int w;

	if (index & 1)
	{
		w = etd->td.iso.data[index >> 1] >> otg_td_iso_UpperDwordLoc_c;
	}
	else
	{
		w = etd->td.iso.data[index >> 1];
	}

	return w & otg_td_iso_LowerLenPSW_c;
}

static inline int
otg_td_iso_NeedYBuffer(otg_etd_t *etd)
{
#ifdef OTG_ISO_SHARE_MEMORY
	return 1;
#else
	return etd->totalPackets > 1;
#endif
}

static inline int
otg_td_iso_GetPacketSize(otg_etd_t *etd)
{
	return stack_urb_GetIsoLength(etd->urb, etd->completed);
}
 
static inline void *
otg_td_iso_GetPacketAddr(otg_etd_t *etd)
{
	return etd->data + stack_urb_GetIsoOffset(etd->urb, etd->completed);
}
 
static inline void
otg_td_iso_IncCompletePacket(otg_etd_t *etd)
{
	etd->completed++;
}
 
static inline int
otg_td_iso_GetBufferIndex(otg_etd_t *etd)
{
	return etd->completed & 1;
}

#ifdef OTG_ISO_SHARE_MEMORY
static inline int
otg_td_iso_IsMemoryUsed(otg_etd_t *etd)
{
	if (otg_etd_GetNext(etd))
	{
		return 1;
	}

	if (otg_etd_GetPrev(etd))
	{
		return 1;
	}

	return 0;
}
#endif

static inline void
otg_td_iso_SetStartFrame(otg_etd_t *etd, int start_frame)
{
	etd->td.iso.ctrl = start_frame;
}

static inline int
otg_td_iso_GetNumPacketFinished(otg_etd_t *etd)
{
	return etd->completed - etd->lastCompleted;
}

static inline int
otg_td_iso_GetTotalPackets(otg_etd_t *etd)
{
	return etd->totalPackets;
}

static inline int
otg_td_iso_HasMorePacket(otg_etd_t *etd)
{
	return etd->completed < etd->dataLength &&
		etd->completed < etd->lastCompleted + otg_td_iso_MaxPackets;
}

static inline int
otg_td_iso_IsLastForUrb(otg_etd_t *etd)
{
	return etd->lastCompleted + otg_td_iso_MaxPackets >= etd->dataLength;
}

static inline void
otg_td_iso_SetStart(otg_etd_t *etd, int start)
{
	etd->lastCompleted = start;
	etd->completed = start;
}

#endif /* #ifndef OTG_TD_ISO_H */
