/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_urb.h
 * Author: Biao Jia		1/21/02
 * Description:
 *    Function side URB object function definitions.
 */
#ifndef OTG_URB_H
#define OTG_URB_H

#include "otg_urb_t.h"
#include "otg_fdevice_t.h"

void otg_urb_Init(otg_urb_t *urb, int ep, char *buffer, int length, otg_urb_complete_t c);

static inline int
otg_urb_GetEp(otg_urb_t *urb)
{
	return urb->ep;
}

static inline char*
otg_urb_GetBuffer(otg_urb_t *urb)
{
	return urb->transfer_buffer;
}

static inline int
otg_urb_GetLength(otg_urb_t *urb)
{
	return urb->transfer_length;
}

static inline int
otg_urb_GetActualLength(otg_urb_t *urb)
{
	return urb->actual_length;
}

static inline int
otg_urb_GetStatus(otg_urb_t *urb)
{
	return urb->status;
}

static inline void
otg_urb_SetBuffer(otg_urb_t *urb, char* buffer)
{
	urb->transfer_buffer = buffer;
}

static inline void
otg_urb_SetLength(otg_urb_t *urb, int length)
{
	urb->transfer_length = length;
}

static inline void
otg_urb_AddLength(otg_urb_t *urb, int length)
{
	urb->transfer_length += length;
}

static inline void
otg_urb_SetActualLength(otg_urb_t *urb, int length)
{
	urb->actual_length = length;
}

static inline void
otg_urb_AddActualLength(otg_urb_t *urb, int length)
{
	urb->actual_length += length;
}

static inline void
otg_urb_SetStatus(otg_urb_t *urb, int status)
{
	urb->status = status;
}

static inline otg_urb_t *
otg_urb_New(int ep, char *buffer, int length, otg_urb_complete_t c)
{
	otg_urb_t *urb;

	urb = OS_Malloc(sizeof(otg_urb_t));
	if (!urb)
	{
		return 0;
	}

	otg_urb_Init(urb, ep, buffer, length, c);

	return urb;
}

static inline void
otg_urb_Delete(otg_urb_t *urb)
{
	OS_Free(urb);
}

static inline void
otg_urb_SetDevice(otg_urb_t *urb, otg_fdevice_t *dev)
{
	urb->dev = dev;
}

static inline otg_fdevice_t *
otg_urb_GetDevice(otg_urb_t *urb)
{
	return urb->dev;
}

#endif /* #ifndef OTG_URB_H */
