/* NB: Some code related to stack_urb, which was in the header file, but
 * was too long and complicated to be inlined */

#include "stack_urb.h"

/* Using stubs seems to work MUCH better than using real functions for bandwidth control... */
#define USE_BANDWIDTH_STUB 1


/* NB: Bandwidth stuff taken back from here:
 * http://www.mail-archive.com/linux-usb-devel@lists.sourceforge.net/msg50551.html
 * It seems to be broken code, probably removed from the mainline kernel for good
 * reasons, but it looks like we need it.
 */

#ifndef USE_BANDWIDTH_STUB
struct urb_priv {
	int bandwidth;
};

typedef struct urb_priv urb_priv_t;

int
stack_urb_HasBandwidth(stack_urb_t *urb)
{
	if (!urb->hcpriv)
		return 0;
	else 
		return ((urb_priv_t*)urb->hcpriv)->bandwidth;
}

int
stack_urb_CheckBandwidth(stack_urb_t *urb)
{
	unsigned int    pipe = urb->pipe;
	long		bustime;
	int		is_in = usb_pipein (pipe);
	int		is_iso = usb_pipeisoc (pipe);
	int		old_alloc = urb->dev->bus->bandwidth_allocated;
	int		new_alloc;

	bustime = NS_TO_US (usb_calc_bus_time (urb->dev->speed, is_in, is_iso,
			  usb_maxpacket (urb->dev, pipe, !is_in)));
	if (is_iso)
		bustime /= urb->number_of_packets;

	new_alloc = old_alloc + (int) bustime;
	if (new_alloc > FRAME_TIME_MAX_USECS_ALLOC) {
//#ifdef DEBUG_OTG_USB_BANDWIDTH_MESSAGES 
		warn("usb_check_bandwidth FAILED: %d + %ld = %d usec",
			  old_alloc, bustime, new_alloc);
//#endif
#ifdef CONFIG_USB_BANDWIDTH
		bustime = -ENOSPC;      /* report error */
#endif 
	}

#ifdef DEBUG_OTG_STUB
	warn("stack_urb_CheckBandwidth=%d", bustime);
#endif

	return bustime;
}

void
stack_urb_ClaimBandwidth(stack_urb_t *urb, int bustime, int is_iso)
{
#ifdef DEBUG_OTG_STUB
	warn("stack_urb_ClaimBandwidth non-stub");
#endif
	//return usb_claim_bandwidth(urb->dev, urb, bustime, is_iso); //TODO: Not sure it's available anymore...
	urb->dev->bus->bandwidth_allocated += bustime;
	if (is_iso)
		 urb->dev->bus->bandwidth_isoc_reqs++;
	else
		 urb->dev->bus->bandwidth_int_reqs++;
	if (urb->hcpriv)
		kfree(urb->hcpriv);
	urb->hcpriv = kmalloc(sizeof(urb_priv_t), GFP_KERNEL);
	((urb_priv_t*)urb->hcpriv)->bandwidth = bustime;

#ifdef DEBUG_OTG_USB_BANDWIDTH_MESSAGES
	warn("bandwidth alloc increased by %d (%s) to %d for %d requesters\n",
		 bustime,
		 is_iso ? "ISOC" : "INTR",
		 urb->dev->bus->bandwidth_allocated,
		 urb->dev->bus->bandwidth_int_reqs + urb->dev->bus->bandwidth_isoc_reqs);
#endif
}

void
stack_urb_ReleaseBandwidth(stack_urb_t *urb, int is_iso)
{
	if (!urb->hcpriv)
		return;
	
	urb->dev->bus->bandwidth_allocated -= ((urb_priv_t*)urb->hcpriv)->bandwidth;
	if (is_iso)
		 urb->dev->bus->bandwidth_isoc_reqs--;
	else
		 urb->dev->bus->bandwidth_int_reqs--;

#ifdef DEBUG_OTG_USB_BANDWIDTH_MESSAGES
	warn("bandwidth alloc reduced by %d (%s) to %d for %d requesters\n",
		 ((urb_priv_t*)urb->hcpriv)->bandwidth,
		 is_iso ? "ISOC" : "INTR",
		 urb->dev->bus->bandwidth_allocated,
		 urb->dev->bus->bandwidth_int_reqs + urb->dev->bus->bandwidth_isoc_reqs);
#endif
	kfree(urb->hcpriv);
	urb->hcpriv = NULL;
}
#else

int
stack_urb_HasBandwidth(stack_urb_t *urb)
{
	return 1;
}

int
stack_urb_CheckBandwidth(stack_urb_t *urb)
{
	return 0;
}

void
stack_urb_ClaimBandwidth(stack_urb_t *urb, int bustime, int is_iso)
{

}

void
stack_urb_ReleaseBandwidth(stack_urb_t *urb, int is_iso)
{

}
#endif
