/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: storage_read.c
 * Author: Biao Jia		5/11/02
 * Description:
 *    Read storage file contents.
 */
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>

unsigned char filenameInfo[512];

int
print_table(int fd, off_t start, int len)
{
	unsigned char *bp;
	unsigned char *np;
	int i;
	int n;
	unsigned char table[len];
	off_t loc;

	loc = lseek(fd, start, SEEK_SET);
	if (loc != start)
	{
		printf("lseek failed with code %d\n", errno);
		return -1;
	}

	n = read(fd, table, len);
	if (n < len)
	{
		printf("read failed with code %d\n", errno);
		return -1;
	}

	bp = table;
	for(n = 0; n < len; n += 16)
	{
		np = bp;

		printf("    %02x %02x %02x %02x %02x %02x %02x %02x",
			*bp++, *bp++, *bp++, *bp++, *bp++, *bp++, *bp++, *bp++);
		printf(" %02x %02x %02x %02x %02x %02x %02x %02x  ",
			*bp++, *bp++, *bp++, *bp++, *bp++, *bp++, *bp++, *bp++);

		for(i = 0; i < 16; i++, np++)
		{
			if (isprint(*np))
			{
				putchar(*np);
			}
			else
			{
				putchar('.');
			}
		}

		putchar('\n');
		if (!((n + 16) % 64))
		{
			putchar('\n');
		}
	}

	return 0;
}

int
print_filetable(int fd, off_t start, int len)
{
	unsigned char *bp;
	int i;
	int n;
	off_t loc;
	int count;

	loc = lseek(fd, start, SEEK_SET);
	if (loc != start)
	{
		printf("lseek failed with code %d\n", errno);
		return -1;
	}

	n = read(fd, filenameInfo, len);
	if (n < len)
	{
		printf("read failed with code %d\n", errno);
		return -1;
	}

	bp = filenameInfo;
	count = 0;
	for(n = 0; n < 64; n += 32)
	{
		if (*bp)
		{
			printf("	%c%c%c%c%c%c%c%c.%c%c%c", *bp++, *bp++, *bp++,
				*bp++, *bp++, *bp++, *bp++, *bp++, *bp++, *bp++, *bp++);
			printf("	attribue: 0x%0x\n", *bp++);

			bp += 10; /* Reserved */
			printf("	time:	0x%0x 0x%0x\n", *bp++, *bp++);
			printf("	date:	0x%0x 0x%0x\n", *bp++, *bp++);
			printf("	start:	0x%0x 0x%0x\n", *bp++, *bp++);
			printf("	size:	0x%0x 0x%0x 0x%0x 0x%0x\n\n",
				*bp++, *bp++, *bp++, *bp++);

			count++;
		}
	}

	printf("Total %d files\n", count);

	return 0;
}

void
print_list(int fd)
{
	int size;
	int n;
	int start;

	n = lseek(fd, 0x117 * 512, SEEK_SET);
	n = read(fd, filenameInfo, 64);
	if (n < 0)
	{
		printf("read failed with code %d\n", errno);
		return;
	}

	printf("Partition Table:\n");
	print_table(fd, 0, 512);

	printf("\nBootsect Table:\n");
	print_table(fd, 0x20 * 512, 2048);

	printf("\nFile Info:\n");
	print_filetable(fd, 0x117 * 512, 512);

	start = filenameInfo[0x1A];
	start += filenameInfo[0x1B] << 8;

	size = filenameInfo[0x1C];
	size += filenameInfo[0x1D] << 8;
	size += filenameInfo[0x1E] << 16;
	size += filenameInfo[0x1F] << 24;

	printf("start=0x%0x size=0x%0x\n", start, size);

	print_table(fd, 0x137 * 512, 128);
}

void
extract_file(int fd)
{
	int size;
	int n;
	int start;
	off_t block;
	int fat;
	unsigned char buf[2048];
	int filed;
	char filename[15];
	unsigned char *np;
	char *p;
	int i;

	block = 0x117;
	n = lseek(fd, block * 512, SEEK_SET);
	n = read(fd, filenameInfo, 512);
	if (n < 512)
	{
		printf("failed read directory info with %d\n", errno);
		return;
	}

	np = filenameInfo;
	/* !*np ensure there is an entry. 0xE5 is the erase code
	 * np[0x1F] make sure size is not too large
	 * 0xB & 0x4 ignore the hidden file (may be long file name)
	 */
	while (!*np || *np == 0xE5 || np[0x1F] >= 0x04 || (np[0x0B] & 0x4))
	{
		np += 32;
		if (np - filenameInfo >= 512)
		{
			printf("No file found\n");
			return;
		}
	}

	p = filename;
	*p = '.';
	p++;
	*p = '/';
	p++;
	for (n = 0; n < 8; n++)
	{
		int ch;

		ch = np[n];
		if (ch != ' ')
		{
			if (ch >= 'A' && ch <= 'Z')
			{
				ch = ch - 'A' + 'a';
			}

			*p = ch;
			p++;
		}
	}

	*p = '.';
	p++;

	for (n = 8; n < 11; n++)
	{
		if (np[n] != ' ')
		{
			*p = np[n];
			if (*p >= 'A' && *p <= 'Z')
			{
				*p = *p - 'A' + 'a';
			}

			p++;
		}
	}

	*p = '\0';

	start = np[0x1A];
	start += np[0x1B] << 8;

	size = np[0x1C];
	size += np[0x1D] << 8;
	size += np[0x1E] << 16;
	size += np[0x1F] << 24;

	printf("extracting file %s with size %d\n", filename, size);

	filed = open((const char*)filename, O_WRONLY | O_CREAT | O_TRUNC);
	if (filed < 0)
	{
		printf("Failed to extract file %s with %d\n", filename, errno);
		return;
	}

	while (size > 0)
	{
		block = 0x137;
		block += (start - 2) << 2;

		n = lseek(fd, block * 512, SEEK_SET);
		n = read(fd, buf, 2048);
		if (n < 2048)
		{
			printf("read failed with code %d\n", errno);
			break;
		}

		if (size > 2048)
		{
			n = 2048;
		}
		else
		{
			n = size;
		}

		write(filed, buf, n);

		size -= n;

		/* Read FAT */
		block = 0x21;
		block += start / 256;
		n = lseek(fd, block * 512, SEEK_SET);
		n = read(fd, buf, 512);
		if (n < 512)
		{
			printf("read FAT failed with code %d\n", errno);
			break;
		}

		i = start;
		start %= 256;
		start <<= 1;
		fat = buf[start];
		fat += buf[start + 1] << 8;

		if (fat >= 0xFFF8)
		{
			printf("done!\n");
			break;
		}

		if (fat < 2 || fat > 0xFFF6)
		{
			printf("bad fat 0x%04x at cluster 0x%0x", fat, i);
			break;
		}

		start = fat;
	}

	printf("closing file %s\n", filename);
	close(filed);
}

#define option_List				't'
#define option_Extract			'x'
void
usage(void)
{
	printf("Usage: storage_read [options]\n");
	printf("	-t partial table contents\n");
	printf("	-x extract first file\n");
}

int main(int argc, char *argv[])
{
	int fd;
	const char *file_name = "storage";
	char *cp;
	int option;

	if (argc < 2)
	{
		usage();
		return 0;
	}

	cp = argv[1];
	if (*cp != '-')
	{
		usage();
		return 0;
	}

	cp++;

	fd = open(file_name, O_RDONLY);
	if (fd < 0)
	{
		printf("failed to open %s\n", file_name);
		return -1;
	}

	if (*cp == 't')
	{
		print_list(fd);
	}
	else
	{
		extract_file(fd);
	}

	close(fd);
}

