/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: test.c
 * Author: Biao Jia		1/21/02
 * Description:
 *    Test routine for on chip memory allocations.
 */
#include "os.h"
#include "stack_urb.h"
#include "otg.h"
#include "otg_chip.h"

void print_mem(otg_mem_t *mem)
{
	otg_mem_block_t *block;
	otg_mem_block_t *end;
	otg_mem_block_t *other;

	end = mem->mem_blocks + mem->blocks;
	printk("------------- mem -------------------\n");
	for (block = mem->mem_blocks; block <= end; block+=block->size+1)
	{
		other = block + block->size;
		printk("mem->start %d\n", block - mem->mem_blocks);
		printk("  block: size 0x%x  status 0x%x\n", block->size, block->status);
		printk("mem->start %d\n", other - mem->mem_blocks);
		printk("  other: size 0x%x  status 0x%x\n", other->size, other->status);
	}
	printk("--------------------------------\n");
	OS_Wait(1000);
}

int
do_alloc_mem(otg_mem_t *mem, int size)
{
	int start;

	start = otg_mem_Allocate(mem, size);
	printk("Requesting %d get start %d\n",size, start);

	print_mem(mem);

	return start;
}

void
do_free_mem(otg_mem_t *mem, int start)
{
	printk("Release %d\n", start);
	otg_mem_Release(mem, start);
	printk("After deallocating mem list is\n");
	print_mem(mem);
}

static int otg_test_mem(void)
{
	otg_mem_t *mem;
	int mem1;
	int mem2;
	int mem3;
	int mem4;
	int mem5;
	int mem6;

	info("otg_test_init called");
	mem = (otg_mem_t *)OS_Malloc(sizeof(otg_mem_t));
	if (!mem)
	{
		err("malloc failed");
		return 0;
	}

	otg_mem_Init(mem, 0, 4096);
	print_mem(mem);
	printk("Size of otg_mem_block is %d\n", sizeof(otg_mem_block_t));

	mem1 = do_alloc_mem(mem, 512);
	mem2 = do_alloc_mem(mem, 64);
	mem3 = do_alloc_mem(mem, 32);
	mem4 = do_alloc_mem(mem, 512);
	mem5 = do_alloc_mem(mem, 16);
	mem6 = do_alloc_mem(mem, 8);
	
	if (mem1<0 || mem2<0 || mem3<0 || mem4<0 || mem5<0 || mem6<0)
	{
		printk("allocated 0\n");
		return 0;
	}

	do_free_mem(mem, mem1);
	mem1 = do_alloc_mem(mem, 1024);
	if (mem1<0)
	{
		printk("allocated 0\n");
		return 0;
	}

	do_free_mem(mem, mem5);
	do_free_mem(mem, mem3);
	mem3 = do_alloc_mem(mem, 42);
	if (mem3<0)
	{
		printk("allocated 0\n");
		return 0;
	}

	mem5 = do_alloc_mem(mem, 128);
	if (mem5<0)
	{
		printk("allocated 0\n");
		return 0;
	}

	do_free_mem(mem, mem2);
	do_free_mem(mem, mem6);
	do_free_mem(mem, mem3);

	do_free_mem(mem, mem1);
	do_free_mem(mem, mem4);
	do_free_mem(mem, mem5);
	
	OS_Free(mem);
	return 0;
}

static int __init otg_test_init(void)
{
	otg_test_mem();
	return 0;
}

static void __exit otg_test_exit(void)
{
	info("otg_test_exit called");
}

module_init(otg_test_init);
module_exit(otg_test_exit);
