/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: test_kbd.c
 * Author: Biao Jia		1/21/02
 * Description:
 *    A function side keyboad driver.
 */
#include "os.h"
#include "stack_urb.h"

#include "otg.h"
#include "test_t.h"
#include "otg_device_request.h"
#include "otg_fd.h"

#ifdef SIM_OTG
#include "otg_chip.h"
#include "otg_sim_device.h"
#endif

#define TEST_KBD_MAJOR 181
#define BUF_MAX		256

static test_t *device = 0;
static int key_sent = 0;
static char buffer[BUF_MAX];
static int read_pos = 0;
static int write_pos = 0;

static otg_urb_t kbd_urb;
static char kbd_buffer[8];
static char init_report[8] = {0, 0, 0, 0, 0, 0, 0, 0};
static char hid_report_desc[256];
static int hid_report_desc_length = 0;
static int submitted = 0;

static otg_fdevice_t kbd_device;

static char kbd_usage_id[128] =
{
	0x00, /* 0x00 NULL */
	0x00, /* 0x01 SOH */
	0x00, /* 0x02 STX */
	0x00, /* 0x03 ETX */
	0x00, /* 0x04 EOT */
	0x00, /* 0x05 ENQ */
	0x00, /* 0x06 ACK */
	0x00, /* 0x07 BEL */
	0x2A, /* 0x08 BS */
	0x2B, /* 0x09 HT */
	0x28, /* 0x0A LF */
	0x00, /* 0x0B VT */
	0x00, /* 0x0C FF */
	0x28, /* 0x0D CR */
	0x00, /* 0x0E SO */
	0x00, /* 0x0F SI */
	0x00, /* 0x10 DLE */
	0x00, /* 0x11 DC1 */
	0x00, /* 0x12 DC2 */
	0x00, /* 0x13 DC3 */
	0x00, /* 0x14 DC4 */
	0x00, /* 0x15 NAK */
	0x00, /* 0x16 SYN */
	0x00, /* 0x17 ETB */
	0x00, /* 0x18 CAN */
	0x00, /* 0x19 EM */
	0x00, /* 0x1A SUB */
	0x29, /* 0x1B ESC */
	0x00, /* 0x1C FS */
	0x00, /* 0x1D GS */
	0x00, /* 0x1E RS */
	0x00, /* 0x1F US */
	0x2C, /* 0x20 Space */
	0x9E, /* 0x21 ! */
	0xB4, /* 0x22 " */
	0xA0, /* 0x23 # */
	0xA1, /* 0x24 $ */
	0xA2, /* 0x25 % */
	0xA4, /* 0x26 & */
	0x34, /* 0x27 ' */
	0xA6, /* 0x28 ( */
	0xA7, /* 0x29 ) */
	0xA5, /* 0x2A * */
	0xAE, /* 0x2B + */
	0x36, /* 0x2C , */
	0x2D, /* 0x2D - */
	0x37, /* 0x2E . */
	0x38, /* 0x2F / */
	0x27, /* 0x30 0 */
	0x1E, /* 0x31 1 */
	0x1F, /* 0x32 2 */
	0x20, /* 0x33 3 */
	0x21, /* 0x34 4 */
	0x22, /* 0x35 5 */
	0x23, /* 0x36 6 */
	0x24, /* 0x37 7 */
	0x25, /* 0x38 8 */
	0x26, /* 0x39 9 */
	0xB3, /* 0x3A : */
	0x33, /* 0x3B ; */
	0xB6, /* 0x3C < */
	0x2E, /* 0x3D = */
	0xB7, /* 0x3E > */
	0xB8, /* 0x3F ? */
	0x9F, /* 0x40 @ */
	0x84, /* 0x41 A */
	0x85, /* 0x42 B */
	0x86, /* 0x43 C */
	0x87, /* 0x44 D */
	0x88, /* 0x45 E */
	0x89, /* 0x46 F */
	0x8A, /* 0x47 G */
	0x8B, /* 0x48 H */
	0x8C, /* 0x49 I */
	0x8D, /* 0x4A J */
	0x8E, /* 0x4B K */
	0x8F, /* 0x4C L */
	0x90, /* 0x4D M */
	0x91, /* 0x4E N */
	0x92, /* 0x4F O */
	0x93, /* 0x50 P */
	0x94, /* 0x51 Q */
	0x95, /* 0x52 R */
	0x96, /* 0x53 S */
	0x97, /* 0x54 T */
	0x98, /* 0x55 U */
	0x99, /* 0x56 V */
	0x9A, /* 0x57 W */
	0x9B, /* 0x58 X */
	0x9C, /* 0x59 Y */
	0x9D, /* 0x5A Z */
	0x2F, /* 0x5B [ */
	0x31, /* 0x5C \ */
	0x30, /* 0x5D ] */
	0xA3, /* 0x5E ^ */
	0xAD, /* 0x5F _ */
	0x00, /* 0x60 ` */
	0x04, /* 0x61 a */
	0x05, /* 0x62 b */
	0x06, /* 0x63 c */
	0x07, /* 0x64 d */
	0x08, /* 0x65 e */
	0x09, /* 0x66 f */
	0x0A, /* 0x67 g */
	0x0B, /* 0x68 h */
	0x0C, /* 0x69 i */
	0x0D, /* 0x6A j */
	0x0E, /* 0x6B k */
	0x0F, /* 0x6C l */
	0x10, /* 0x6D m */
	0x11, /* 0x6E n */
	0x12, /* 0x6F o */
	0x13, /* 0x70 p */
	0x14, /* 0x71 q */
	0x15, /* 0x72 r */
	0x16, /* 0x73 s */
	0x17, /* 0x74 t */
	0x18, /* 0x75 u */
	0x19, /* 0x76 v */
	0x1A, /* 0x77 w */
	0x1B, /* 0x78 x */
	0x1C, /* 0x79 y */
	0x1D, /* 0x7A z */
	0xAF, /* 0x7B { */
	0xB1, /* 0x7C | */
	0xB0, /* 0x7D } */
	0xB2, /* 0x7E ~ */
	0x2A /* 0x7F DEL */
};

static int
otg_test_kbd_GetChar(void)
{
	int pos;

	if (read_pos == write_pos)
	{
#ifdef DEBUG_OTG_CHIP
		info("GetChar: no char read_pos=%d write_ps=%d", read_pos, write_pos);
#endif
		return 0;
	}

	pos = read_pos;
	read_pos++;
	if (read_pos >= BUF_MAX)
	{
		read_pos = 0;
	}

#ifdef DEBUG_OTG_CHIP
	info("GetChar: %c", buffer[pos]);
#endif
	return buffer[pos];
}

static void
otg_test_kbd_CallBack(otg_urb_t *urb);

static int
otg_test_kbd_PutChar(char ch)
{
	int rc;

	if ((write_pos + 1) == read_pos)
	{
		rc = 0;
	}
	else
	{
		buffer[write_pos] = ch;
		write_pos++;
		if (write_pos >= BUF_MAX)
		{
			write_pos = 0;
		}

		rc = 1;
	}

	if (!submitted)
	{
		kbd_urb.status = 0;
		otg_test_kbd_CallBack(&kbd_urb);
	}

	return rc;
}

static ssize_t
otg_test_kbd_Read(struct file* file, char* buf, size_t size, loff_t* ppos)
{
#ifdef DEBUG_OTG_CHIP
	info("Read: size=%d buf=%p", size, buf);
#endif
	if (read_pos == write_pos)
	{
		return 0;
	}

	/* do it later */
	return 0;
}

static ssize_t
otg_test_kbd_Write(struct file* fp, const char* buf, size_t size, loff_t* ppos)
{
	int len;
	int ch;

#ifdef DEBUG_OTG_CHIP
	info("Write: size=%d buf=%p", size, buf);
#endif
	for (len = 0; len < size; len++)
	{
		ch = buf[len];
		if (ch <= 0 || ch > 127)
		{
#ifdef DEBUG_OTG_CHIP
			info("Write: unknown character %c", ch);
#endif
			continue;
		}

		otg_test_kbd_PutChar(ch);
	}

	return size;
}

static int
otg_test_kbd_Open(struct inode* inode, struct file *filp)
{
	int minor;
	int i;
	
#ifdef DEBUG_OTG_CHIP
	info("Open: called");
#endif
	minor = MINOR(inode->i_rdev);
	read_pos = 0;
	write_pos = 0;
	key_sent = 0;

	for(i = 0; i < 8; i++)
	{
		kbd_buffer[i] = 0;
	}

	return 0;
}

static int
otg_test_kbd_Release(struct inode* inode, struct file *filp)
{
#ifdef DEBUG_OTG_CHIP
	info("Release: called");
#endif
	return 0;
}

static struct file_operations test_kbd_fops =
{
	read: otg_test_kbd_Read,
	write: otg_test_kbd_Write,
	open: otg_test_kbd_Open,
	release: otg_test_kbd_Release,
};

static void
otg_test_kbd_CallBack(otg_urb_t *urb)
{
	int ch;

#ifdef DEBUG_OTG_CHIP
	info("CallBack: urb=%p actual_length=%d status=0x%0x",
		urb, urb->actual_length, urb->status);
#endif

	if (urb->status == otg_urb_ErrorResetByHost)
	{
		/* Resubmit */
		info("CallBack: connection reset by host");
	}
	else
	{
		if (key_sent)
		{
#ifdef DEBUG_OTG_CHIP
			info("CallBack: send key up");
#endif
			kbd_buffer[0] = 0x00;
			kbd_buffer[2] = 0x00;
			otg_fd_Submit(&kbd_urb);
			key_sent = 0;
			submitted = 1;
		}
		else
		{
#ifdef DEBUG_OTG_CHIP
			info("CallBack: send key down");
#endif
			ch = 0;
			ch = otg_test_kbd_GetChar();
#ifdef DEBUG_OTG_CHIP
			info("CallBack: char = 0x%0x", ch);
#endif
			if (!ch)
			{
				kbd_buffer[0] = 0x00;
				kbd_buffer[2] = 0x00;
				submitted = 0;
				key_sent = 0;
#ifdef DEBUG_OTG_CHIP
			info("CallBack: no char to send, stop transfer");
#endif
				return;
			}

			kbd_buffer[2] = kbd_usage_id[ch] & 0x7F;
#ifdef DEBUG_OTG_CHIP
			info("CallBack: ch=0x%0x:%c", ch, ch);
#endif
			if (kbd_usage_id[ch] & 0x80)
			{
				/* Use 0x80 as a shift indicator */
				kbd_buffer[0] = 0x02;
			}
			else
			{
				kbd_buffer[0] = 0x00;
			}

			key_sent = 1;

			if (!otg_fd_Submit(&kbd_urb))
			{
#ifdef DEBUG_OTG_CHIP
				info("CallBack: calling otg_fd_Submit");
#endif
				submitted = 1;
			}
		}
	}
}

static int
otg_test_kbd_StandardRequest(otg_device_request_t *request, void *private)
{
	int type;
	int index;

#ifdef DEBUG_OTG_CHIP
	info("StandardRequest: bmRequestType=0x%0x bRequest=0x%0x",
		request->bmRequestType, request->bRequest);
	info("       wValue=0x%0x wIndex=0x%0x wLength=0x%0x",
		request->wValue, request->wIndex, request->wLength);
#endif
	
	request->wLength = 0;
	if (request->bmRequestType != 0x81)
	{
		return 0;
	}

	if (request->bRequest != 0x06)
	{
		return 0;
	}

	index = request->wValue & 0xFF;
	type = (request->wValue >> 8) & 0xFF;

	switch (type)
	{
		case 0x21:
			info("StandardRequest: HID descriptor not supported");
			return -1;
		case 0x22:
			request->wLength = hid_report_desc_length;
			request->data = hid_report_desc;
			break;
		case 0x23:
			info("StandardRequest: HID physical descriptor not supported");
			return -1;
		default:
			err("StandardRequest: unknown descriptor type 0x%0x", type);
			return -1;
	}

	return 0;
}

static int
otg_test_kbd_ClassRequest(otg_device_request_t *request, void *private)
{
#ifdef DEBUG_OTG_CHIP
	info("ClassRequest: bmRequestType=0x%0x bRequest=0x%0x",
		request->bmRequestType, request->bRequest);
	info("       wValue=0x%0x wIndex=0x%0x wLength=0x%0x",
		request->wValue, request->wIndex, request->wLength);
#endif
	
	if (request->bmRequestType == 0xA1)
	{
		if (request->bRequest == 0x01 && ((request->wValue) & 0xFF00) == 0x100)
		{
			request->data = init_report;
			if (request->wLength > 8)
			{
				request->wLength = 8;
			}

			return 0;
		}
	}

	request->wLength = 0;

	return 0;
}

static int
otg_test_kbd_VendorRequest(otg_device_request_t *request, void *private)
{
#ifdef DEBUG_OTG_CHIP
	info("VendorRequest: bmRequestType=0x%0x bRequest=0x%0x",
		request->bmRequestType, request->bRequest);
	info("		wValue=0x%0x wIndex=0x%0x wLength=0x%0x",
		request->wValue, request->wIndex, request->wLength);
#endif
	
	request->wLength = 0;

	return 0;
}

static int
otg_test_kbd_start(void)
{
	test_t *dev;
	int rc;
	int len;
	int strlen;

	len = 0;
	hid_report_desc[len++] = 0x05;
	hid_report_desc[len++] = 0x01;
	hid_report_desc[len++] = 0x09;
	hid_report_desc[len++] = 0x06;
	hid_report_desc[len++] = 0xA1;
	hid_report_desc[len++] = 0x01;
	hid_report_desc[len++] = 0x05;
	hid_report_desc[len++] = 0x07;
	hid_report_desc[len++] = 0x19;
	hid_report_desc[len++] = 0xE0;
	hid_report_desc[len++] = 0x29;
	hid_report_desc[len++] = 0xE7;
	hid_report_desc[len++] = 0x15;
	hid_report_desc[len++] = 0x00;
	hid_report_desc[len++] = 0x25;
	hid_report_desc[len++] = 0x01;
	hid_report_desc[len++] = 0x95; /* */
	hid_report_desc[len++] = 0x08;
	hid_report_desc[len++] = 0x75;
	hid_report_desc[len++] = 0x01;
	hid_report_desc[len++] = 0x81;
	hid_report_desc[len++] = 0x02;
	hid_report_desc[len++] = 0x95;
	hid_report_desc[len++] = 0x01;
	hid_report_desc[len++] = 0x75;
	hid_report_desc[len++] = 0x08;
	hid_report_desc[len++] = 0x81;
	hid_report_desc[len++] = 0x01;
	hid_report_desc[len++] = 0x95;
	hid_report_desc[len++] = 0x05;
	hid_report_desc[len++] = 0x75;
	hid_report_desc[len++] = 0x01;
	hid_report_desc[len++] = 0x05;
	hid_report_desc[len++] = 0x08;
	hid_report_desc[len++] = 0x19;
	hid_report_desc[len++] = 0x01;
	hid_report_desc[len++] = 0x29;
	hid_report_desc[len++] = 0x05;
	hid_report_desc[len++] = 0x91;
	hid_report_desc[len++] = 0x02;
	hid_report_desc[len++] = 0x95;
	hid_report_desc[len++] = 0x01;
	hid_report_desc[len++] = 0x75;
	hid_report_desc[len++] = 0x03;
	hid_report_desc[len++] = 0x91;
	hid_report_desc[len++] = 0x01;
	hid_report_desc[len++] = 0x95;
	hid_report_desc[len++] = 0x06;
	hid_report_desc[len++] = 0x75;
	hid_report_desc[len++] = 0x08;
	hid_report_desc[len++] = 0x15;
	hid_report_desc[len++] = 0x00;
	hid_report_desc[len++] = 0x25;
	hid_report_desc[len++] = 0x65;
	hid_report_desc[len++] = 0x05;
	hid_report_desc[len++] = 0x07;
	hid_report_desc[len++] = 0x19;
	hid_report_desc[len++] = 0x00;
	hid_report_desc[len++] = 0x29;
	hid_report_desc[len++] = 0x65;
	hid_report_desc[len++] = 0x81;
	hid_report_desc[len++] = 0x00;
	hid_report_desc[len++] = 0xC0;

	hid_report_desc_length = len;

	dev = device;
	dev->deviceDesc[0] = otg_device_desc_Max; /* bLength */
	dev->deviceDesc[1] = 0x01; /* bDescriptorType, device */
	dev->deviceDesc[2] = 0x10; /* bcdUSB lower byte */
	dev->deviceDesc[3] = 0x01; /* bcdUSB higher byte */
	dev->deviceDesc[4] = 0x00; /* bDeviceClass, hid */
	dev->deviceDesc[5] = 0x00; /* bDeviceSubClass */
	dev->deviceDesc[6] = 0x00; /* bDeviceProtocol, keyboard */
	dev->deviceDesc[7] = 0x08; /* bMaxPacketSize0, 64 bytes */
	dev->deviceDesc[8] = 0x11; /* idVendor lower byte */
	dev->deviceDesc[9] = 0x55; /* idVendor higher byte */
	dev->deviceDesc[10] = 0x56; /* idProduct lower byte */
	dev->deviceDesc[11] = 0x34; /* idProduct higher byte */
	dev->deviceDesc[12] = 0x34; /* bcdDevice lower byte */
	dev->deviceDesc[13] = 0x12; /* bcdDevice higher byte */
	dev->deviceDesc[14] = 0x01; /* iManufacturer */
	dev->deviceDesc[15] = 0x02; /* iProduct */
	dev->deviceDesc[16] = 0x03; /* iSeriaNumber */
	dev->deviceDesc[17] = 0x01; /* bNumConfigurations */

	len = 0;
	dev->configDesc[len++] = otg_config_desc_Max; /* bLength */
	dev->configDesc[len++] = 0x02; /* bDescriptorType, configuration */
	dev->configDesc[len++] = 0x00; /* wTotalLength lower byte */
	dev->configDesc[len++] = 0x00; /* wTotalLength higher byte */
	dev->configDesc[len++] = 0x01; /* bNumInterfaces */
	dev->configDesc[len++] = 0x01; /* bConfigurationValue */
	dev->configDesc[len++] = 0x00; /* iConfiguration */
	dev->configDesc[len++] = 0xA0; /* bmAttributes, Bus powered,Remote Wakeup */
	dev->configDesc[len++] = 0x01; /* MaxPower */

	dev->configDesc[len++] = otg_interface_desc_Max;
	dev->configDesc[len++] = 0x04; /* bDescriptorType, Interface */
	dev->configDesc[len++] = 0x00; /* bInterfaceNumber */
	dev->configDesc[len++] = 0x00; /* bAlternateSetting */
	dev->configDesc[len++] = 0x01; /* bNumEndpoints */
	dev->configDesc[len++] = 0x03; /* bInterfaceClass, HID */
	dev->configDesc[len++] = 0x01; /* bInterfaceSubClass, boot inferface */
	dev->configDesc[len++] = 0x01; /* bInterfaceProtocol, keyboard */
	dev->configDesc[len++] = 0x00; /* iInterface */

	dev->configDesc[len++] = 9;
	dev->configDesc[len++] = 0x21; /* bDescriptorType, HID class */
	dev->configDesc[len++] = 0x00; /* bcdHID */
	dev->configDesc[len++] = 0x01;
	dev->configDesc[len++] = 0x00; /* bCountryCode */
	dev->configDesc[len++] = 0x01; /* bNumDescriptors */
	dev->configDesc[len++] = 0x22; /* bDescriptorType, report */
	/* wDescriptorLength */
	dev->configDesc[len++] = hid_report_desc_length & 0xFF;
	dev->configDesc[len++] = (hid_report_desc_length >> 8) & 0xFF;

	/* Interrupt In endpoint */
	dev->configDesc[len++] = otg_ep_desc_Max;
	dev->configDesc[len++] = 0x05; /* bDescriptorType, Endpoint */
	dev->configDesc[len++] = 0x81; /* bEndpointAddress */
	dev->configDesc[len++] = 0x03; /* bmAttributes, interrupt */
	dev->configDesc[len++] = 0x08; /* wMaxPacketSize lower byte */
	dev->configDesc[len++] = 0x00; /* wMaxPacketSize higher byte */
	dev->configDesc[len++] = 0x02; /* bInterval */

	/* Fillin wTotalLength */
	dev->configDesc[otg_config_desc_wTotalLength] = len & 0xFF;
	dev->configDesc[otg_config_desc_wTotalLength + 1] = (len >> 8) & 0xFF;

	dev->strDesc[0] = 4;	/* bLength */
	dev->strDesc[1] = 3;	/* bDescriptorType, String */
	dev->strDesc[2] = 0x09; /* LangID codes, English */
	dev->strDesc[3] = 0x00;

	strlen = 4;
	dev->strDesc[strlen++] = 8;	/* bLength */
	dev->strDesc[strlen++] = 3;	/* bDescriptorType, String */
	dev->strDesc[strlen++] = 'T'; /* T */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = 'D'; /* D */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = 'I'; /* I */
	dev->strDesc[strlen++] = 0x00;

	dev->strDesc[strlen++] = 32;	/* bLength */
	dev->strDesc[strlen++] = 3;		/* bDescriptorType, String */
	dev->strDesc[strlen++] = 'O'; /* O */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = 'T'; /* T */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = 'G'; /* G */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = '2'; /* 2 */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = '4'; /* 4 */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = '3'; /* 3 */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = ' '; /* space */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = 'K'; /* K */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = 'e'; /* e */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = 'y'; /* y */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = 'b'; /* b */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = 'o'; /* o */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = 'a'; /* a */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = 'r'; /* r */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = 'd'; /* d */
	dev->strDesc[strlen++] = 0x00;

	dev->strDesc[strlen++] = 10;	/* bLength */
	dev->strDesc[strlen++] = 3;		/* bDescriptorType, String */
	dev->strDesc[strlen++] = '0'; /* 0 */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = '1'; /* 1 */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = '5'; /* 5 */
	dev->strDesc[strlen++] = 0x00;
	dev->strDesc[strlen++] = '2'; /* 2 */
	dev->strDesc[strlen++] = 0x00;

	otg_urb_Init(&kbd_urb, 0x81, kbd_buffer, 8, otg_test_kbd_CallBack);
	otg_urb_SetDevice(&kbd_urb, &kbd_device);

	OS_MemSet(&kbd_device, 0, sizeof(kbd_device));

	kbd_device.strDesc = dev->strDesc;
	kbd_device.strDescLength = strlen;
	kbd_device.deviceDesc = dev->deviceDesc;
	kbd_device.deviceDescLength = otg_device_desc_Max;
	kbd_device.configDesc = dev->configDesc;
	kbd_device.configDescLength = len;
	kbd_device.standardSetup = otg_test_kbd_StandardRequest;
	kbd_device.classSetup = otg_test_kbd_ClassRequest;
	kbd_device.vendorSetup = otg_test_kbd_VendorRequest;

	rc = otg_fd_Register(&kbd_device);
	if (rc < 0)
	{
		err("SetCallBack failed");
		return -1;
	}

	return 0;
}

static int
otg_test_kbd_InitOtg(void)
{
	device = OS_Malloc(sizeof(test_t));
	if (!device)
	{
		return -1;
	}

	OS_MemSet(device, 0, sizeof(test_t));

	return 0;
}

static void
otg_test_kbd_ExitOtg(void)
{
	otg_fd_Unregister(&kbd_device);

	if (device)
	{
		OS_Free(device);
		device = 0;
	}
}

static int __init otg_test_kbd_init(void)
{
	int rc;
#ifdef SIM_OTG
	otg_chip_t *chip;
#endif

	rc = otg_test_kbd_InitOtg();
	if (rc)
	{
		return rc;
	}

#if 0 // TODO: fix devfs
    if (devfs_register_chrdev(TEST_KBD_MAJOR, "test_kbd", &test_kbd_fops))
	{
		err("unable t get major %d for test_kbd devices", TEST_KBD_MAJOR);
		otg_test_kbd_ExitOtg();
		return -EBUSY;
	}
#endif

#ifdef SIM_OTG
	chip = otg_chip_GetChip();
	chip->ports[1] = otg_sim_device_New();
	if (!chip->ports[1])
	{
		info("Failed to allocate memory");
		return 0;
	}
#endif

	rc = otg_test_kbd_start();
	if (rc)
	{
		err("OTG start function controller failed");
		otg_test_kbd_ExitOtg();
		return rc;
	}

#ifdef SIM_OTG
	otg_chip_RhPortStatusChange(1);
#endif

	return 0;
}

static void __exit otg_test_kbd_exit(void)
{
#ifdef SIM_OTG
	otg_chip_t *chip;
#endif

#if 0 // TODO: fix devfs
	devfs_unregister_chrdev(TEST_KBD_MAJOR, "test_kbd");
#endif

	otg_test_kbd_ExitOtg();

#ifdef SIM_OTG
	chip = otg_chip_GetChip();
	if (chip->ports[1])
	{
		otg_chip_RhPortStatusChange(1);
		otg_sim_device_Delete(chip->ports[1]);
		chip->ports[1] = 0;
		device = 0;
	}
#endif
}

module_init(otg_test_kbd_init);
module_exit(otg_test_kbd_exit);
